#==============================================================================
# package      DE_EPAGES::Catalog::UI::Category
# state        private
#==============================================================================
package DE_EPAGES::Catalog::UI::Category;
use base DE_EPAGES::Content::UI::ContentObject;

use strict;
use DE_EPAGES::Catalog::API::Catalog qw (
    AssignProductToCategory
    RenumberCategoryProducts
    SortProductList
);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias Transaction );
use DE_EPAGES::Presentation::API::Object::Tray;
use DE_EPAGES::Presentation::API::PageCache qw ( UpdatePageCache );
use DE_EPAGES::Product::API::Shop qw ( GetProduct );
use DE_EPAGES::Product::UI::ProductFolder;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

#==============================================================================
# Event handlers
#==============================================================================
sub AddProductsFromTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $aNewProducts = $Tray->objectsByClass(LoadClassByAlias('Product'));

    RunBatchProcess('UpdateProducts', sub{
        Transaction( sub {
            foreach my $Product(@$aNewProducts) {
                AssignProductToCategory($Product, $Category);
            }
            RenumberCategoryProducts($Category);
        });
    });
    UpdatePageCache($Category);
    return;
}

sub RemoveProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Category = $Servlet->object;

    my $MapClass = LoadClassByAlias('CategoryProductMap');

    my @MapIDs = $Servlet->form->value('ListObjectID');
    RunBatchProcess('UpdateProducts', sub{
        $MapClass->loadObject($_)->delete foreach grep { $MapClass->existsObject($_) } @MapIDs;
        RenumberCategoryProducts($Category);
    });
    UpdatePageCache($Category);
    return;
}

sub SaveProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Category = $Servlet->object;

    # add product to category by product number
    my $Form = $Servlet->form;
    my $Alias  = $Form->value('NewProduct');
    if (defined $Alias and $Alias =~ /\S/) {
        my $TargetProduct = GetProduct($Category->getSite(), $Alias, 1);
        $Servlet->form->executeFormError({
            'Reason'=> 'ERROR_INVALIDPRODUCT',
            'Form'  => 'NoForm',
            'Name'  => 'NewProduct',
        }) unless defined $TargetProduct;
        AssignProductToCategory($TargetProduct, $Category, $Form->value('NewPosition'));
    }

    my $ahListItems = $self->listItems( $Servlet, $Category, 'CategoryProductMap' );

    # save the new positions
    RunBatchProcess('UpdateProducts', sub{
        foreach my $hMap (@$ahListItems) {
            $hMap->{'Object'}->set({'Position', $hMap->{'Position'}});
        }
        RenumberCategoryProducts($Category);
    });
    UpdatePageCache($Category);
    return;
}

sub AddProductListToTray {
    my $self = shift;
    my ($Servlet) = @_;
    my $Category = $Servlet->object;

    my $hValues = $Servlet->form->form($Category, 'ListedObjects');
    my $ahListObjectIDs = $hValues->{'ListObjectIDs'};
    my @ListObjects;
    my $Class = LoadClassByAlias('CategoryProductMap');
    foreach my $hValue (@$ahListObjectIDs) {
        my $Product = $Class->loadObject($hValue->{'ListObjectID'})->get('Product');
        push @ListObjects, $Product if defined $Product;
    }

    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    $Tray->addObjectsToClassGroup(\@ListObjects);
    return;
}

sub Alphabetize {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'alphabetize');
    UpdatePageCache($Category);
    return;
}

sub SortByPriceASC {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'byPriceASC');
    UpdatePageCache($Category);
    return;
}

sub SortByPriceDESC {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'byPriceDESC');
    UpdatePageCache($Category);
    return;
}

sub SortByNew {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'byNew');
    UpdatePageCache($Category);
    return;
}

sub SortByStockLevel {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'byStockLevel');
    UpdatePageCache($Category);
    return;
}

sub SortByProductAlias {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    SortProductList($Category, 'byProductAlias');
    UpdatePageCache($Category);
    return;
}

sub NewProduct {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    my $Shop = $Category->getSite();
    my $ProductFolder = $Shop->get('ProductFolder');
    my $Product = $self->DE_EPAGES::Product::UI::ProductFolder::NewObject( $Servlet, $ProductFolder );

    Transaction( sub {
        AssignProductToCategory($Product, $Category);
        RenumberCategoryProducts($Category);
    });

    return $Product;
}

sub ProductsSetVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    my $hValues = $Servlet->form->form($Category, 'ListedObjects');
    my $ahListObjectIDs = $hValues->{'ListObjectIDs'};

    RunBatchProcess('UpdateProducts', sub{
        foreach my $hValue (@$ahListObjectIDs){
            my $Product = LoadClassByAlias('CategoryProductMap')->loadObject($hValue->{'ListObjectID'})->get('Product');
            if (defined $Product){
                $Product->set({'IsVisible' => '1'});
            }
        }
    });
    # EPG-10728:
    $Category->getSite()->invalidatePageCache;
    return;
}

sub ProductsSetInvisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    my $hValues = $Servlet->form->form($Category, 'ListedObjects');
    my $ahListObjectIDs = $hValues->{'ListObjectIDs'};

    RunBatchProcess('UpdateProducts', sub{
        foreach my $hValue (@$ahListObjectIDs){
            my $Product = LoadClassByAlias('CategoryProductMap')->loadObject($hValue->{'ListObjectID'})->get('Product');
            if (defined $Product){
                $Product->set({'IsVisible' => '0'});
            }
        }
    });
    # EPG-10728:
    $Category->getSite()->invalidatePageCache;
    return;
}

sub UnityView {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

1;
