package DE_EPAGES::Catalog::t::CategoryResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Data::Dumper;
use DE_EPAGES::Catalog::API::Resource::Category;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Permission::API::Permission qw ( ActionByName );
use DE_EPAGES::Permission::API::Table::Permission qw ( InsertPermission );
use Test::Exception;
use Test::More;

## no critic qw(ProhibitMagicNumbers)

sub init {
    my $self = shift;
    $self->SUPER::init;

    # set up test data
    my $Shop = $self->shop;
    $Shop->set( {
            DomainName        => 'test-domain',
            HasSSLCertificate => 1
        }
    );
    $Shop->featureMaxValue( 'Pages',         5 );
    $Shop->featureMaxValue( 'Subcategories', 1 );
    my $Categories = $Shop->child('Categories');
    my $TestCat    = $Categories->insertChild(
        'Category',
        {
            'Alias'     => 'Testing',
            'IsVisible' => 1,
        }
    );
    # add some localized info.
    $TestCat->set( {
            'Description' => 'SomeText',
            'Name'        => 'SomeName',
        },
        GetPKeyLanguageByCode('en')
    );
    # this shows up
    $TestCat->insertChild(
        'Category',
        {
            'Alias'     => 'TestVisible',
            'IsVisible' => 1,
            'Position'  => 0,
        }
    );
    # this does not show up
    $TestCat->insertChild(
        'Category',
        {
            'Alias'     => 'TestInvisible',
            'IsVisible' => 0,
            'Position'  => 100,
        }
    );
    # this should raise an Forbidden exeption.
    my $NonPermittedCategory = $Categories->insertChild(
        'Category',
        {
            Alias     => 'VisibleForRegisteredUsers',
            IsVisible => 1,
            Inherit   => 0,
        }
    );
    InsertPermission( {
            ObjectID     => $NonPermittedCategory->id,
            TrusteeID    => LoadObjectByPath('/Groups/Registered')->id,
            MetaActionID => ActionByName( $NonPermittedCategory->class, 'View' )->id,
            Allow        => 1,
        }
    );
}

sub test {
    my $self = shift;

    my $Shop = $self->shop;

    ## test GET
    my $Resource  = DE_EPAGES::Catalog::API::Resource::Category->new;
    my $Category  = $Shop->child('Categories')->child('Testing');
    my $hResult   = $Resource->get( $Category, 'en_US' );
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    is_deeply $hResult,
      {
        'categoryId' => $Category->guid,
        'alias'     => 'Testing',
        'specialOffer' => 0,
        'visible' => 'true',
        'parent'     => {
            'rel'   => 'Categories',
            'href'  => $SSLDomain . '/api/categories/' . $Category->parent->guid,
            'title' => 'Categories'
        },
        'images'        => [],
        'pageTitle'     => 'SomeName',
        'name'          => 'SomeName',
        'navigationCaption' => undef,
        'subCategories' => [ {
                'rel'   => 'TestVisible',
                'href'  => $SSLDomain . '/api/categories/' . $Category->child('TestVisible')->guid,
                'title' => 'TestVisible'
            }
        ],
        'description' => 'SomeText',
        'sfUrl'      => 'http://test-domain/epages/Store.sf/?ObjectPath=/Test/TestShop/Categories/Testing',
      },
      'GET category'
      or Test::More::diag Dumper($hResult);

    ## test PUT
    $hResult = $Resource->put($Category, 'en_US', { alias =>'ALIAS', name => 'NAME', pageTitle=>'TITLE', description=>'DESCRIPTION'});
    is_deeply $hResult,
      {
        'categoryId' => $Category->guid,
        'specialOffer' => 0,
        'visible' => 'true',
        'parent'     => {
            'rel'   => 'Categories',
            'href'  => $SSLDomain . '/api/categories/' . $Category->parent->guid,
            'title' => 'Categories'
        },
        'images'        => [],
        'pageTitle'     => 'TITLE',
        'alias'         => 'ALIAS',
        'name'          => 'NAME',
        'navigationCaption' => undef,
        'subCategories' => [ {
                'rel'   => 'TestVisible',
                'href'  => $SSLDomain . '/api/categories/' . $Category->child('TestVisible')->guid,
                'title' => 'TestVisible'
            },
            {
                'rel'   => 'TestInvisible',
                'href'  => $SSLDomain . '/api/categories/' . $Category->child('TestInvisible')->guid,
                'title' => 'TestInvisible'
            }
        ],
        'description' => 'DESCRIPTION',
        'sfUrl'      => 'http://test-domain/epages/Store.sf/?ObjectPath=/Test/TestShop/Categories/ALIAS',
      },
      'PUT category'
      or Test::More::diag Dumper($hResult);

    ## check Unauthorized
    $Category = $Shop->child('Categories')->child('VisibleForRegisteredUsers');
    throws_ok(sub { $Resource->get( $Category, 'en_US' ) }, qr/Forbidden/, 'Accessing a protected category is forbidden via REST');

    lives_ok(sub { $Resource->delete($Category);}, 'Delete Category - test if delete can be called withour errors');
    throws_ok(sub { $Resource->delete($Shop->child('Categories'));}, qr/BadRequest/,'Delete Category - test if delete root category throws error');
    throws_ok(sub { $Resource->get( $Category, 'en_US' ) }, qr/NoResults/, 'Deleted Category does not exist!');
}

__PACKAGE__->runScript;
done_testing();
