use strict;
use Test::More tests => 17;
use Data::Dumper;
use HTTP::Status;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Net qw( GetHttpResponse );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::DataCache::API::Config qw( DataCacheConfig );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath LoadRootObject);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ShopPath;
my $ShopAlias = 'TestShop';

use constant MAX_HTML_SIZE => 35000; # 6.17.45: 30930; 6.17.42: 30484; 6.17.26.1: 25502; 6.17.13.1: 25247; 6.16.2: 24000; 6.15.1: 22667; 6.14.0: 24530; 6.13.0-pre: 24119 (2011/12/08); 6.12.2-pre: 26634; 25273 (2011/06/22), 6.12.0-pre: 24495 bytes, 6.11.3: 25261 bytes; 6.11.0: 27906 bytes
use constant MAX_JS_SIZE => 370000;  # 6.15.1: 360952; 6.14.0: 347758; 6.13.0-pre: 335716 (2011/12/08); 6.12.0 (2011/05/31): 312956; 6.11.3: 302471 bytes; 6.11.0: 690253 bytes (CRLF)
use constant MAX_JS_COUNT => 2;
use constant MAX_CSS_SIZE => 292000; # 6.16.3: 291202; 6.16.1: 290445; 6.16.0: 271867; 6.15.1: 261130; 6.14.0: 249815; 6.13.0-pre: 294848 (2011/12/20); 6.11.2: 264541 bytes; 6.11.0: 283019 Bytes (CRLF)
use constant MAX_CSS_COUNT => 5;
use constant MAX_TOTAL_SIZE => MAX_HTML_SIZE + MAX_JS_SIZE + MAX_CSS_SIZE; # 6.16.1: 677216; 6.15.1: 645868; 6.14.0: 622251; 6.13.0-pre: 654735 (2011/12/20); 6.12.2-pre: 601480; 600620 (2011/06/22); 6.12.0 (2011/05/31): 589803 bytes; 6.11.3: 561906 bytes; 6.11.0: 1001178 bytes
use constant MAX_TOTAL_COUNT => 8;

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
# syntax       $Time = TimeThis( sub { sleep 3; } );
#----------------------------------------------------------------------------------------
# description  Returns the real time required to execute the code block.
#----------------------------------------------------------------------------------------
# input        $Code | code to execute | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# function     SetShopAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopAttributes( $StoreName, $hValues );
#----------------------------------------------------------------------------------------
# description  Sets attributes on the DemoShop object.
#----------------------------------------------------------------------------------------
# input        $StoreName | store name | string
# input        $hValues | attribute values | ref.hash
#========================================================================================
sub SetShopAttributes {
    my ($StoreName, $hValues) = @_;

    RunOnStore(
        Store      => $StoreName,
        Sub        => sub {
            LoadObjectByPath($ShopPath)->set($hValues);
        }
    );
    return;
}

sub TestExternalIncludes {
    my ($FakeRequest, $ShopURL) = @_;

    my $Response = $FakeRequest->get( $ShopURL );
    die "Can't get $ShopURL: ".$Response->status_line unless $Response->is_success;

    my $Content = $Response->content;

    # filter debug info
    $Content =~ s/\n.*?BEGIN INCLUDE.*?\n//g;
    $Content =~ s/\n.*?END INCLUDE.*?\n//g;
    $Content =~ s/\n.*?END OF PAGE.*?\n//g;
    $Content =~ s/\n.*?CachedInclude.*?\n//g;

    #   use DE_EPAGES::Core::API::File qw( WriteFile );
    #   WriteFile('content.html', \$Content, { binary => 1 });

    my $ContentLength = length($Content);
    cmp_ok( $ContentLength, '<', MAX_HTML_SIZE, "HTML code size (actual: $ContentLength)" );

    my $hCSSLinks = _ExtractExternalStyles( $ShopURL, $Content );
    my $hJSLinks = _ExtractExternalScripts( $ShopURL, $Content );
    cmp_ok( $hCSSLinks->{'Count'}, '<=', MAX_CSS_COUNT, "Number of CSS files (actual: $hCSSLinks->{'Count'})" );
    cmp_ok( $hCSSLinks->{'Size'}, '<', MAX_CSS_SIZE, "CSS size (actual: $hCSSLinks->{'Size'})" );
    ok( scalar keys %{$hCSSLinks->{'DoubleLinks'}} == 0 , 'All CSS files are imported only once');
    if ( scalar keys %{$hCSSLinks->{'DoubleLinks'}} > 0 ) {
        Test::More::diag join "\n", map { "  -> $_ is used $hCSSLinks->{'DoubleLinks'}->{$_}x times" } keys %{$hCSSLinks->{'DoubleLinks'}};
    }

    cmp_ok( $hJSLinks->{'Count'}, '<=', MAX_JS_COUNT, "Number of JS files (actual: $hJSLinks->{'Count'})" );
    cmp_ok( $hJSLinks->{'Size'}, '<', MAX_JS_SIZE, "JS size (actual: $hJSLinks->{'Size'})" );

    my $TotalCount = 1 + $hCSSLinks->{'Count'} + $hJSLinks->{'Count'};
    my $TotalSize = $ContentLength + $hJSLinks->{'Size'} + $hCSSLinks->{'Size'};
    cmp_ok( $TotalCount, '<=', MAX_TOTAL_COUNT, "Total number of files (actual: $TotalCount)" );
    cmp_ok( $TotalSize, '<=', MAX_TOTAL_SIZE, "Total page size (actual: $TotalSize)" );

    if( $TotalCount > MAX_TOTAL_COUNT || $TotalSize > MAX_TOTAL_SIZE ) {
        Test::More::diag Dumper( $hCSSLinks, $hJSLinks );
    }
}

sub _ExtractExternalScripts {
    my ($ShopURL, $Content) = @_;

    my $BaseURI = URI->new($ShopURL);

    my $Count = 0;
    my $Size = 0;
    my @Links;
    foreach my $hAttributes( grep { exists $_->{'src'} } _AttributesByTagName($Content, 'script') ) {
        my $ScriptURL = URI->new( $hAttributes->{'src'} )->abs( $BaseURI )->as_string;
        my $Response = GetHttpResponse( $ScriptURL );
        die "Can't get script '$ScriptURL': " . $Response->status_line unless $Response->is_success;

        $Count++;
        $Size += length( $Response->content );
        push @Links, {
            URL => $hAttributes->{'src'},
            Size => length( $Response->content ),
        };
    }

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
    };
}

sub _ExtractExternalStyles {
    my ($ShopURL, $Content) = @_;

    my $BaseURI = URI->new( $ShopURL );
    my @StyleLinks = map { $_->{'href'} } grep { $_->{'rel'} =~ /^stylesheet$/i } _AttributesByTagName($Content, 'link');

    my $Count = 0;
    my $Size = 0;
    my %LinkCounter;
    my @Links;
    foreach my $StyleLink( @StyleLinks ) {
        my $StyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $StyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        foreach my $tempLink ( @{ $hStyle->{'Links'} } ){
            $LinkCounter{$tempLink->{'URL'}} = ($LinkCounter{$tempLink->{'URL'}} // 0) + 1;
        }
        push @Links, @{ $hStyle->{'Links'} };
    }

    my @StylesWithIE10 = map { _ParseImports( $_ ) } _TextsByTagName( $Content, 'style' );

    # Filter out all IE10 imports like:
    # @import url('style.css') screen and (-ms-high-contrast: active), (-ms-high-contrast: none);
    my @StyleURLsWithoutIE10 = map { $_->{'URL'} } grep { $_->{'MediaTypes'} !~ /-ms-high-contrast/ } @StylesWithIE10;

    foreach my $StyleLink ( @StyleURLsWithoutIE10 ) {
        my $StyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $StyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        foreach my $tempLink ( @{ $hStyle->{'Links'} } ){
            $LinkCounter{$tempLink->{'URL'}} = ($LinkCounter{$tempLink->{'URL'}} // 0) + 1;
        }
        push @Links, @{ $hStyle->{'Links'} };
    }

    my %DoubleLinks = map { $_ => $LinkCounter{$_}} grep { $LinkCounter{$_} > 1 } keys %LinkCounter;

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
        DoubleLinks => \%DoubleLinks,
    };
}

sub _ExtractImportStyles {
    my ($StyleURL) = @_;

    my $Response = GetHttpResponse( $StyleURL );
    die "Can't get style '$StyleURL': " . $Response->status_line unless $Response->is_success;
    my $Content = $Response->content;

    my $BaseURI = URI->new( $StyleURL );
    my $Count = 1;
    my $Size = length $Content;
    my @Links = ( {
        URL => $StyleURL,
        Size => length $Content,
    } );

    foreach my $StyleLink ( map { $_->{'URL'} } _ParseImports( $Content ) ) {
        my $ImportStyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $ImportStyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        push @Links, @{ $hStyle->{'Links'} }
    }

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
    };
}

sub _ParseImports {
    my ($Content) = @_;

    # eg. @import url( 'import4.css' ) tv, print;
    my @Imports;
    $Content =~ s/
        \@import
        (?:\s+url\s*\()? # open (
        \s*
        (["']?)    # open quote (optional) "
        ([^"'\)]*) # $2 = relative file name
        \1         # close quote
        \s*
        \)?        # close )
        \s*
        (.*?)      # $3 = media types
        \s*
        ;
    /push @Imports, { URL => $2, MediaTypes => $3};/xges;

    return @Imports;
}

sub _ParseAttributes {
    my ($Attributes) = @_;

    my %Attributes;
    while( $Attributes =~ /(\w+)\s*=\s*["'](.*?)["']/sgx ) { # "
        $Attributes{$1} = $2;
    }

    return \%Attributes;
}

sub _AttributesByTagName {
    my ($Content, $TagName) = @_;

    my @Tags;
    while( $Content =~ m{
        <
        \Q$TagName\E\s+     # tag name
        (.*?)               # attributes string
        (?:/)?             # optional /
        >
    }isgx
    ) {
        push @Tags, _ParseAttributes( $1 );
    }

    return @Tags;
}

sub _TextsByTagName {
    my ($Content, $TagName) = @_;

    my @Texts;
    while( $Content =~ m{
        <\Q$TagName\E\b.*?>     # open tag
        (.*)                    # text
        <\/\Q$TagName\E>        # close tag
    }isgx
    ) {
        push @Texts, $1;
    }

    return @Texts;
}

sub Main {

    my $StoreName = GetTestSite();
    my $TestShopURL;
    my $JSDebugLevel;

    RunOnStore(
        Store      => $StoreName,
        Sub        => sub {

            my $TestShop = LoadObjectByPath($ShopPath);
            $TestShop->set({
                WebServerScriptNamePart  => $ShopAlias,
                IsClosed => 0,
                ClosedByMerchant => 0,
                ClosedByProvider => 0,
            });
            $TestShopURL = $TestShop->get('WebUrl');

            $JSDebugLevel = LoadRootObject()->get('JSDebugLevel');
        }
    );

    SKIP: {
        skip 'no TestShop installed', 3 unless defined $TestShopURL;

        is( $JSDebugLevel,0 , "System.JSDebugLevel" );
        ok( DataCacheConfig()->get('PageCache', 'DisableCtmplStat'), 'DisableCtmplStat' );

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

        my $Sub = sub {
            my $Response = $FakeRequest->get( $TestShopURL );
            if( !$Response->is_success ) {
                die $Response->status_line;
            }
        };

        SKIP: {
            skip( 'Forced skip of timing tests.', 3 ) if $ENV{'SKIP_TIMING'};

            my $FirstTime = TimeThis( $Sub );
            cmp_ok( $FirstTime, '<', 4, sprintf('start page: first call: %0.3f s', $FirstTime) ); ## no critic qw(ProhibitMagicNumbers)

            my $SecondTime = TimeThis( $Sub );
            cmp_ok( $SecondTime, '<', 0.3, sprintf('start page: second call: %0.3f s', $SecondTime) ); ## no critic qw(ProhibitMagicNumbers)

            my $ThirdTime = TimeThis( $Sub );
            cmp_ok( $ThirdTime, '<', 0.3, sprintf('start page: third call: %0.3f s', $ThirdTime) ); ## no critic qw(ProhibitMagicNumbers)
        }

        SetShopAttributes( $StoreName,{
            'IsClosed' => 1,
        });

        is( $FakeRequest->get( $TestShopURL )->code, RC_SERVICE_UNAVAILABLE, 'status code Site.IsClosed' );

        SetShopAttributes( $StoreName,{
            'IsClosed' => 0,
            'ClosedByMerchant' => 1,
        });

        is( $FakeRequest->get( $TestShopURL )->code, RC_OK, 'status code Site.ClosedByMerchant' );

        SetShopAttributes( $StoreName,{
            'ClosedByMerchant' => 0,
            'ClosedByProvider' => 1,
        });

        is( $FakeRequest->get( $TestShopURL )->code, RC_SERVICE_UNAVAILABLE, 'status code Site.ClosedByProvider' );

        SetShopAttributes( $StoreName,{
            'ClosedByProvider' => 0,
        });

        is( $FakeRequest->get( $TestShopURL )->code, RC_OK, 'status code (not closed)' );

        TestExternalIncludes( $FakeRequest, $TestShopURL );
    };
    return;
}

sub Init {
    Cleanup();

    LoadRootObject()->child('Shops')->insertChild('Shop', { 'Alias'=>$ShopAlias});
    $ShopPath = 'Shops/'.$ShopAlias;
}

sub Cleanup {
    my $Folder = LoadRootObject()->child('Shops');
    $Folder->child($ShopAlias)->delete if $Folder->existsChild($ShopAlias);
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Init();
        Main();
        Cleanup();
    });
});
