#========================================================================================
# package      DE_EPAGES::ContextualAppAds::UI::JSON
# state        public
# description  General UI functions of the ContextualAppAds cartridge.
#========================================================================================
package DE_EPAGES::ContextualAppAds::UI::JSON;
use base Exporter;

use strict;
use JSON;
use DE_EPAGES::AppStore::API::Constants qw ( APPSTORE_APP_TYPES );

our @EXPORT_OK = qw (
    JSONWriteAppData
);

#========================================================================================
# function     JSONWriteAppData
# state        public
#----------------------------------------------------------------------------------------
# syntax       JSONWriteAppData($Servlet, $MboContext);
# example      JSONWriteAppData($Servlet, 'product');
#----------------------------------------------------------------------------------------
# description  Function to get the data of all apps related to a given mboContext,
#               count the uninstalled ones and write the values to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $MboContext | MBO context as defined in the MATE tool,
#                   such as 'product', 'order', 'customer', 'shipping', ... | string
#========================================================================================
sub JSONWriteAppData {
    my ($Servlet, $MboContext) = @_;
    my $Shop = $Servlet->object->getSite();

    # do not show any apps unless the feature is active
    unless ($Shop->featureMaxValue('ContextualAppAds')) {
        $Servlet->vars('ExistAppsForContext', 0);
        return;
    }

    # get all apps with the given mboContext
    my $AppsClient = DE_EPAGES::AppStore::API::Object::AppsClient->create(
        {
            'Shop'   => $Shop,
            'Locale' => $Servlet->locale
        }
    );
    my $ResultAppsForContext = $AppsClient->listApps( 1, APPSTORE_APP_TYPES, undef, $MboContext );
    my @AppsForContext = @{ $ResultAppsForContext->{'content'} };

    my $ExistAppsForContext = scalar @AppsForContext ? 1 : 0;
    $Servlet->vars('ExistAppsForContext', $ExistAppsForContext);
    # do not proceed if we have no apps for this context
    return unless $ExistAppsForContext;

    # get all installed apps
    my $aInstalledApps = $AppsClient->myApps();
    my @InstalledAppIDs;
    push @InstalledAppIDs, $_->{'id'} foreach @$aInstalledApps;

    # add attribute "installed" to the apps for the context
    # and count the uninstalled apps
    my $CountUninstalledApps = 0;
    foreach my $AppForContext (@AppsForContext) {
        if ($AppForContext->{'id'} ~~ @InstalledAppIDs) {
            $AppForContext->{'installed'} = JSON::true;
        } else {
            $AppForContext->{'installed'} = JSON::false;
            $CountUninstalledApps++;
        }
    }

    # set up JSON parser
    my $json = JSON->new;
    # allow blessed objects to avoid errors
    $json = $json->allow_blessed;
    # enable pretty JSON output
    $json = $json->pretty;

    # write values to the servlet
    $Servlet->vars('AppsForContext', $json->encode(\@AppsForContext));
    $Servlet->vars('CountUninstalledApps', $CountUninstalledApps);

    return;
}

1;
