#========================================================================================
# package      DE_EPAGES::Core::API::FTP
# state        public
# description  Wrapper over WWW::Curl::Easy
#========================================================================================
package DE_EPAGES::Core::API::FTP;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::File qw ( OpenFile ) ;
use DE_EPAGES::Core::API::CurlEasy;
use DE_EPAGES::Core::API::TestParameter qw (TestString);
use File::Temp;
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->new();
#----------------------------------------------------------------------------------------
# description  Creates an empty DE_EPAGES::Core::API::FTP object
#----------------------------------------------------------------------------------------
# return       FTP object |  DE_EPAGES::Core::API::FTP object
#========================================================================================
sub new {
    my $class = shift;

    my $self = {
        url => undef,
        username  => undef,
        password => undef,
        curlobject => DE_EPAGES::Core::API::CurlEasy->new(),
    };

    bless $self,$class;
    return $self;

}

#========================================================================================
# function     connect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->connect($Url,$username,$password);
#----------------------------------------------------------------------------------------
# description  Connect to a FTP server
#----------------------------------------------------------------------------------------
# input        $Url | FTP server Url | string
# input        $username | FTP server login username | string
# input        $password | FTP server login password | string
#========================================================================================
sub connect {
    my $self = shift;
    my ($Url,$UserName,$Password) = @_ ;

    TestString('URL', $Url);

    $self->{'url'} = $Url;
    $self->{'username'} = $UserName;
    $self->{'password'} = $Password;

    my $Curl = $self->{'curlobject'};

    if (defined($ENV{'ftp_proxy'})){  # especially for installations with a proxy server (Everest)
        my @Headers = ( 'Expect:' );
        $Curl->setopt(CURLOPT_HTTPHEADER, \@Headers);
        $Curl->setopt(CURLOPT_PROXY, $ENV{'ftp_proxy'});
    }

    $Curl->setopt(CURLOPT_URL,$Url);
    $Curl->setopt(CURLOPT_USERPWD, "$UserName:$Password");
    $Curl->setopt(CURLOPT_FORBID_REUSE, 1);

    my $RetCodeConnect = $Curl->perform();

    if ($RetCodeConnect != 0){
       Error( 'FTPError', { 'Method' => 'connect', 'RetCode' => $RetCodeConnect, 'ErrMsg' => $self->message($RetCodeConnect)});
    }
}

#========================================================================================
# function     putFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->putFile($LocalFileName,$RemoteFileName);
#----------------------------------------------------------------------------------------
# description  Send a file to a FTP url once the connection is created
#----------------------------------------------------------------------------------------
# input        $LocalFileName | Name of the file in local system (Absolute Path!) | string
# input        $RemoteFileName | Name that the file will have in FTP server (Absolute Path!) | string
#========================================================================================
sub putFile {                ## no critic (RequireCommentForExportedFunctions)
    my $self = shift;
    my ($LocalFileName,$RemoteFileName) = @_;

    if(!$self->connectionDataOK){
        Error( 'FTPError', { 'Method' => 'putFile', 'ErrMsg' => 'Connection credentials not defined'});
    };

    my $RetCodePutFile;
    my $Curl = $self->{'curlobject'};
    my $FinalUrl = $self->{'url'} . "$RemoteFileName";
    my $filesize = -s $LocalFileName;

    OpenFile( '<', $LocalFileName, sub {
        my( $FileHandle ) = @_ ;

        $Curl->setopt(CURLOPT_URL,$FinalUrl);
        $Curl->setopt(CURLOPT_UPLOAD,1);
        $Curl->setopt(CURLOPT_INFILESIZE, $filesize);

        $Curl->setopt(CURLOPT_READFUNCTION, sub {
            if ($filesize > 0) {
                my $data = <$FileHandle>;
                $filesize=$filesize-length($data);
                return $data;
            } else {
                return '';
            }
        });

        $RetCodePutFile = $Curl->perform();
    });

    $Curl->setopt(CURLOPT_UPLOAD, undef);
    $Curl->setopt(CURLOPT_INFILESIZE, undef);
    $Curl->setopt(CURLOPT_READFUNCTION, undef);

    if ($RetCodePutFile != 0){
       Error( 'FTPError', { 'Method' => 'putFile', 'RetCode' => $RetCodePutFile , 'ErrMsg' => $self->message($RetCodePutFile)});
    }
}

#========================================================================================
# function     getFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->getFile($LocalFileName,$RemoteFileName);
#----------------------------------------------------------------------------------------
# description  Get a file from a FTP url once the connection is created
#----------------------------------------------------------------------------------------
# input        $LocalFileName | Name of the file in local system (Absolute Path!) | string
# input        $RemoteFileName | Name of the file in FTP url (Absolute Path!) | string
#========================================================================================
sub getFile {
    my $self = shift;
    my ($LocalFileName,$RemoteFileName) = @_;

    if(!$self->connectionDataOK){
        Error( 'FTPError', { 'Method' => 'getFile', 'ErrMsg' => 'Connection credentials not defined'})
    }

    my $RetCodeGetFile;
    my $Curl = $self->{'curlobject'};
    my $FinalUrl = $self->{'url'} . "$RemoteFileName";

    $Curl->setopt(CURLOPT_URL,$FinalUrl);

    OpenFile( '>', $LocalFileName, sub {
        my( $FileHandle ) = @_ ;
        $Curl->setopt(CURLOPT_WRITEDATA, $FileHandle);

        $RetCodeGetFile = $Curl->perform();
    });

    $Curl->setopt(CURLOPT_WRITEDATA, undef);

    if ($RetCodeGetFile != 0){
       Error( 'FTPError', { 'Method' => 'getFile', 'RetCode' => $RetCodeGetFile , 'ErrMsg' => $self->message($RetCodeGetFile)});
    }
}

#========================================================================================
# function     listFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       @aReturnFiles = $ftp->listFiles($Directory);
#----------------------------------------------------------------------------------------
# description  List the files,links,directories,... that exist in $Directory of FTP server
#----------------------------------------------------------------------------------------
# input        $Directory | Name of the directory in FTP server (Absolute Path!) | string
# return       @aReturnFiles | list of file names without path | array
#========================================================================================
sub listFiles  {
    my $self = shift;
    my ($Directory) = @_;

    if(!$self->connectionDataOK){
        Error( 'FTPError', { 'Method' => 'listFiles', 'ErrMsg' => 'Connection credentials not defined'})
    }

    my $Curl = $self->{'curlobject'};
    my $FinalUrl = $self->{'url'} . "$Directory";
    my $response_body = File::Temp->new();
    my @aReturnFiles;

    $Curl->setopt(CURLOPT_URL,$FinalUrl);
    $Curl->setopt(CURLOPT_WRITEDATA, $response_body);

    my $RetCodeListFiles = $Curl->perform();

    seek($response_body,0,0);
    my @adata = <$response_body>;

    foreach my $filename ( @adata[1 .. $#adata])  { # We ignore the first line because it includes the total number of files listed.
        my @value = split(' ', $filename); # we split the line in order to can get at the last token of each line.
        push @aReturnFiles, $value[-1]; # the last token of each line is the file name.
    }

    $Curl->setopt(CURLOPT_WRITEDATA, undef);

    if ($RetCodeListFiles != 0){
       Error( 'FTPError', { 'Method' => 'listFiles', 'RetCode' => $RetCodeListFiles , 'ErrMsg' => $self->message($RetCodeListFiles)});
    }

    return @aReturnFiles;
}

#========================================================================================
# function     renameFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->renameFile($OldFileName,$NewFileName);
#----------------------------------------------------------------------------------------
# description  Rename a file of the FTP server
#----------------------------------------------------------------------------------------
# input        $OldFileName | Old Name of the file in FTP server (Absolute Path, without first '/') | string
# input        $NewFileName | New Name of the file in FTP server (Absolute Path, without first '/') | string
#========================================================================================
sub renameFile  {

    my $self = shift;
    my ($OldFileName,$NewFileName) = @_;

    if(!$self->connectionDataOK){
        Error( 'FTPError', { 'Method' => 'renamefile', 'ErrMsg' => 'Connection credentials not defined'})
    }

    my @emptyCommands;
    my $Curl = $self->{'curlobject'};

    $Curl->setopt(CURLOPT_URL, $self->{'url'});

    my $CommandFrom = 'RNFR '. $OldFileName;
    my $CommandTo = 'RNTO '. $NewFileName;
    my @Commands = ($CommandFrom, $CommandTo);


    $Curl->setopt(CURLOPT_POSTQUOTE, \@Commands);

    my $RetCodeRenameFile = $Curl->perform();

    $Curl->setopt(CURLOPT_POSTQUOTE, \@emptyCommands);

    if ($RetCodeRenameFile != 0){
       Error( 'FTPError', { 'Method' => 'renameFile', 'RetCode' => $RetCodeRenameFile , 'ErrMsg' => $self->message($RetCodeRenameFile)});
    }
}

#========================================================================================
# function     connectionDataOK
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->connectionDataOK;
#----------------------------------------------------------------------------------------
# description  Check if connection data is setup OK
#----------------------------------------------------------------------------------------
# return       $retcode | 1 if successful | int
#========================================================================================
sub connectionDataOK  {
    my $self = shift;

    return isValid($self->{'url'}) &&
           isValid($self->{'username'}) &&
           defined($self->{'password'});
}

#========================================================================================
# function     message
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ftp->message($RetCode);
#----------------------------------------------------------------------------------------
# description  Returns the description of the return code.
#----------------------------------------------------------------------------------------
# input        $retcode | Code to descript | int
# return       Error description | Description of the last execution | String
#========================================================================================
sub message {
    my $self = shift;
    my ( $retcode ) = @_;

    my $CurlMessage = $self->{'curlobject'};

    return  $CurlMessage->strerror($retcode);
}

1;
