#==============================================================================
# package      DE_EPAGES::Core::API::ResultTable
# state        public
#
# Example      my $table = DE_EPAGES::Core::API::ResultTable->new(
#                   'DBIResult => [ [..] ],
#                   'Header' => [ 'Col1', 'Col2', 'Col3' ],
#               );
#               $table->paintResultSetTable();
#------------------------------------------------------------------------------
# description  OO interface to operate with project.conf and dependent
#               files to produce files from templates (e.g. makefile)
#==============================================================================
package DE_EPAGES::Core::API::ResultTable;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        MissingTableHeader    => 'Missing parameter Header for ResultTable',
        MissingDBIResult      => 'Missing parameter DBIResult for ResultTable',
        HeaderNotFittingTable => 'Header defines #HeaderCol columns while DBIResult has #ResultCol columns per row',
    };
}

sub new {
    my $self = shift;
    my (%Options) = @_;

    my $Class = ref($self) || $self;
    if ( not defined $Options{'Header'} )    { Error('MissingTableHeader'); }
    if ( not defined $Options{'DBIResult'} ) { Error('MissingDBIResult'); }
    $Options{'HSeperator'}   = '-' unless defined $Options{'HSeperator'};
    $Options{'VSeperator'}   = ' ' unless defined $Options{'VSeperator'};
    $Options{'Border'}       = 1   unless defined $Options{'Border'};
    $Options{'FinalNewLine'} = 1   unless defined $Options{'FinalNewLine'};
    $Options{'NeatValues'}   = 0   unless defined $Options{'NeatValues'};
    $self = bless { %Options, }, $Class;

    if ( $self->colcount != scalar $self->header ) {
        Error( 'HeaderNotFittingTable', { 'HeaderCol' => scalar $self->header, 'ResultCol' => $self->colcount } );
    }
    return $self;
}

sub sort {
    my $self = shift;
    my ($Column) = @_;

    my @Result = sort { _alphanum( $a->[$Column], $b->[$Column] ) } @{ $self->{'DBIResult'} };
    $self->{'DBIResult'} = \@Result;
    return $self;
}

sub reverse {
    my $self = shift;

    my @Reverse = reverse @{ $self->{'DBIResult'} };
    $self->{'DBIResult'} = \@Reverse;
    return $self;
}

sub header {
    my $self = shift;

    return @{ $self->{'Header'} };
}

sub linecount {
    my $self = shift;

    if ( defined $self->{'DBIResult'} ) {
        return scalar @{ $self->{'DBIResult'} };
    } else {
        return 0;
    }
}

sub colcount {
    my $self = shift;

    if ( defined $self->{'DBIResult'} ) {
        my $ResultSet = $self->{'DBIResult'};
        if ( defined  $ResultSet->[0]  ) {
            return scalar @{ $ResultSet->[0] };
        } else {
            return scalar $self->header;
        }
    } else {
        return scalar $self->header;
    }
}

sub paintResultSetTable {
    my $self = shift;
    my ($Border) = @_;

    my $VChar  = $self->{'VSeperator'};
    my $HChar  = $self->{'HSeperator'};
    my $Result = '';
    if ( defined $self->{'DBIResult'} ) {
        my $aaResultSet = $self->{'DBIResult'};
        my $NrOfResults = $self->linecount;
        if ( $NrOfResults > 0 ) {

            # find longest entry to set up table witdh accordingly
            my @EntryLen = ( map { length($_) } $self->header );
            foreach my $aRow ( @{$aaResultSet} ) {
                my $EntryNr = 0;
                foreach my $Value (@$aRow) {
                    if ( $self->{'NeatValues'} ) {
                        $Value =~ s/\s*$//;
                        $Value =~ s/^\s*//;
                    }
                    if ( $EntryLen[$EntryNr] < length($Value) ) {
                        $EntryLen[$EntryNr] = length($Value);
                    }
                    $EntryNr++;
                }
            }
            my $Format;
            foreach my $Len (@EntryLen) {
                $Format .= $VChar . ' %-' . $Len . 's ';
            }
            $Format .= $VChar . "\n";
            my $Header = sprintf( $Format, $self->header() );
            my $Line = $HChar x ( length($Header) - 1 ) . "\n";
            $Result .= sprintf($Line) if $self->{'Border'};
            $Result .= sprintf($Header);
            $Result .= sprintf($Line);
            if ( not scalar @{ $self->{'DBIResult'}->[0] } ) {
                $Result .= "  no results\n";
            } else {
                foreach my $aRow ( @{ $self->{'DBIResult'} } ) {
                    $Result .= sprintf( $Format, @{$aRow} );
                }
            }
            $Result .= sprintf($Line) if $self->{'Border'};
            $Result .= "\n" if $self->{'FinalNewLine'};
        }
    }
    return $Result;
}

#========================================================================================
# function     _alphanum
# state        public
#----------------------------------------------------------------------------------------
# syntax       _alphanum( 'test123', 'test10' )
#----------------------------------------------------------------------------------------
# description  compares two strings with numbers in them and returns the bigger one
#               by considering the interger value of number parts, otherwise
#               sorts lexicographically
#----------------------------------------------------------------------------------------
# input        $a | Text | String
# input        $b | Text | String
# return       $CompareResult | cmp/<=> result | result of compare
#========================================================================================
sub _alphanum {
    my ($a, $b) = @_;

    # split strings into chunks
    my @a = _chunkify( $a );
    my @b = _chunkify( $b );

    # while we have chunks to compare.
    while ( @a && @b ) {
        my $a_chunk = shift @a;
        my $b_chunk = shift @b;

        my $test = ( ( $a_chunk =~ /\d/ ) && ( $b_chunk =~ /\d/ ) )
            ?    # if both are numeric
            $a_chunk <=> $b_chunk
            :                         # compare as numbers
            $a_chunk cmp $b_chunk;    # else compare as strings

        # return comparison if not equal.
        return $test if $test != 0;
    }

    # return longer string.
    return @a <=> @b;
}

# split on numeric/non-numeric transitions
sub _chunkify {
    my ($String) = @_;

    my @chunks = split m{ # split on
    (?= # zero width
      (?<=\D)\d | # digit preceded by a non-digit OR
      (?<=\d)\D # non-digit preceded by a digit
    )
    }x, $String;
    return @chunks;
}

1;
