#!perl -w
use Test::More tests => 19;

use strict;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Program::java;
use DE_EPAGES::Core::API::Program::mongodb;
use DE_EPAGES::Core::API::Program::mp3splt;
use DE_EPAGES::Core::API::Program::nodejs;
use DE_EPAGES::Core::API::Program::jdk;
use DE_EPAGES::Core::API::Script qw( RunScript );

sub TestError($$$) {
    my ($Sub, $aErrorCodes, $Message) = @_;
    $aErrorCodes = [$aErrorCodes] unless ref $aErrorCodes eq 'ARRAY';

    eval { $Sub->() };
    if( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code;
        ok( $ErrorCode ~~ $aErrorCodes, $Message ) or Test::More::diag $Error->longMessage;
    }
    else {
        fail( "no error: $Message" );
    }
}

RunScript( Debug => 1, Sub => sub {

    my $Dummy = DummyProgram->new;
    ok( !$Dummy->isInstalled, 'Dummy.isInstalled' );
    TestError( sub { $Dummy->version }, ['CommandFailed', 'CommandError'], 'Dummy.version' );

    my $Node = DE_EPAGES::Core::API::Program::nodejs->new;
    ok( $Node->isInstalled, 'node.isInstalled' );
    SKIP: {
        skip 'Node is not installed', 2 unless $Node->isInstalled;

        like( $Node->version, qr/^\d+\.\d+\.\d+$/x, 'node.version' );
        like( $Node->command, qr/node/, 'node.command' );
    }

    my $Mp3splt = DE_EPAGES::Core::API::Program::mp3splt->new;
    ok( $Mp3splt->isInstalled, 'mp3splt.isInstalled' );
    SKIP: {
        skip 'Mp3splt is not installed', 2 unless $Mp3splt->isInstalled;

        like( $Mp3splt->version, qr/^\d+\.\d+\.\d+$/x, 'mp3splt.version' );
        like( $Mp3splt->command, qr/mp3splt/, 'mp3splt.command' );
    }

    SKIP: {
        skip 'JDK is not required. Therefore we skip the test (EPG-34067).', 3;
        my $JDK = DE_EPAGES::Core::API::Program::jdk->new;
        ok( $JDK->isInstalled, 'jdk.isInstalled' );
        SKIP: {
            skip 'JDK is not installed', 2 unless $JDK->isInstalled;

            like( $JDK->version, qr/^\d+\.\d+\.\d+(?:_\d+)$/x, 'jdk.version' );
            like( $JDK->command, qr/javac/, 'jdk.command' );
        }
    }

    my $Java = DE_EPAGES::Core::API::Program::java->new;
    ok( $Java->isInstalled, 'java.isInstalled' );
    SKIP: {
        skip 'Java is not installed', 4 unless $Java->isInstalled;

        like( $Java->version, qr/^\d+\.\d+\.\d+(?:_\d+)$/x, 'java.version' );
        like( $Java->command, qr/java/, 'java.command' );

        local $ENV{'EPAGES_JAVACMD'} = undef;
        local $ENV{'EPAGES_JAVA'} = '/testdir';
        like( $Java->command, qr'^[/\\]testdir[/\\]bin[/\\]java'x, 'EPAGES_JAVA' );

        local $ENV{'EPAGES_JAVACMD'} = '/testdir/java';
        is( $Java->command, '/testdir/java', 'EPAGES_JAVACMD' );
    }

    my $Mongo = DE_EPAGES::Core::API::Program::mongodb->new;
    ok( $Mongo->isInstalled, 'mongodb.isInstalled' );
    SKIP: {
        skip 'MongoDB is not installed', 2 unless $Mongo->isInstalled;

        like( $Mongo->version, qr/^\d+\.\d+\.\d+$/x, 'mongodb.version' );
        like( $Mongo->command, qr/mongo/, 'mongodb.command' );
    }
});

#========================================================================================
# package      DummyProgram
#========================================================================================
package DummyProgram; ## no critic qw(RequireCommentForExportedFunctions)
use base 'DE_EPAGES::Core::API::Program::Base';

use strict;
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );

sub command {
    return 'dummyxy';
}

sub version {
    my $self = shift;

    my $Command = $self->command;
    return SystemCommand( "\"$Command\" -v", { 'Capture' => 1 } );
}
