use strict;
use File::Find;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Dictionary::API::Compare qw ( CompareDictionaryFile CompareTranslationFile );

#========================================================================================
# function     CheckFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       CheckFile ( $Language1, $Language2, $File);
# example      CheckFile ( "en", "de", "example");
#----------------------------------------------------------------------------------------
# description  This function compares two localization files, finds
#               missing keywords in file1 & file2 and print them out.
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $File | filename without extention | string
#========================================================================================
sub CheckFile {
    my ( $Language1, $Language2, $DummyFilename) = @_;
    my $aErrors = CompareDictionaryFile($Language1, $Language2, $DummyFilename);
    print "Missing language file: $_\n" foreach ( @$aErrors );
}

#========================================================================================
# function     Wanted
# state        private
#----------------------------------------------------------------------------------------
# syntax       Wanted ($Language1, $Language2, $hFiles);
# example      Wanted ("en", "de", \%Files);
#----------------------------------------------------------------------------------------
# description  This function is used by the function File::Find. All localization files
#               in the curent directory (and the recursive following directories)
#               will be stored in hFiles. The extention of the filenames will cutted.
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $hFiles | hash to store filenames | ref.hash
#========================================================================================
sub Wanted {
    my ($Language1, $Language2, $hFiles) = @_;

    ## no critic qw( ProhibitComplexRegexes )
    if (    $File::Find::name =~ /((\/|\\)Templates(\/|\\)|(\/|\\)javascript(\/|\\)).*(\.$Language1|\.$Language2)\.xml$/g
         && $File::Find::name !~ /\/CodeGenerator\/Templates\//
         && $File::Find::name !~ /\/Diagnostics\/Templates\/Dictionary\./
         && $File::Find::name !~ /\/Presentation\/Templates\/SimpleError\./
         && $File::Find::name !~ /\/t\//
    ## use critic
    ) {
        my $Filename = $File::Find::name;
        $Filename =~ s/(\.$Language1|\.$Language2)\.xml$//;
        $hFiles->{$Filename} = 1;
    }
}

#========================================================================================
# function     WantedTranslation
# state        private
#----------------------------------------------------------------------------------------
# syntax       WantedTranslation ($Language1, $Language2, $hFiles);
# example      WantedTranslation ("en", "de", \%Files);
#----------------------------------------------------------------------------------------
# description  This function is used by the function File::Find. All localization files
#               in the curent directory (and the recursive following directories)
#               will be stored in hFiles. The extention of the filenames will cutted.
#                The file have the form Translation.**.xml.
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $hFiles | hash to store filenames | hHash ??
#========================================================================================
sub WantedTranslation {
    my ($Language1, $Language2, $hFiles) = @_;

    if (
        $File::Find::name =~ /Translation\..*(?:\.$Language1|\.$Language2)\.xml$/g and
        $File::Find::name !~ /[\/\\]Patches[\/\\].*(?:\.$Language1|\.$Language2)\.xml$/ and
        $File::Find::name !~ /\/t\//
    ){
        my $Filename = $File::Find::name;
        $Filename =~ s/(\.$Language1|\.$Language2)\.xml$//;
        $hFiles->{$Filename} = 1;
    }
}

#========================================================================================
# function     CheckTranslationFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       CheckTranslationFile ( $Language1, $Language2, $File);
# example      CheckTranslationFile ( "en", "de", "example");
#----------------------------------------------------------------------------------------
# description  This function compares two localization files, finds
#               missing keywords in file1 & file2 and print them out.
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $File | filename without extention | string
#========================================================================================
sub CheckTranslationFile {
    my ( $Language1, $Language2, $DummyFilename ) = @_;
    my $aErrors = CompareTranslationFile($Language1, $Language2, $DummyFilename);
    print "Missing translation file: $_\n" foreach ( @$aErrors );
}

sub Main {
    my $Language1;
    my $Language2;
    my $Directory = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES";
    GetOptions(
        'dir=s'        => \$Directory,
        'lang1=s'      => \$Language1,
        'lang2=s'      => \$Language2,
    );

    #comparing language files
    my %Files;

    if ($Language1 eq '' or $Language2 eq '') { usage();}

    # get all needed filenames and save them as keys in %Files
    find( sub { Wanted($Language1, $Language2, \%Files) }, $Directory);

    my @Files = keys %Files;

    foreach my $File ( @Files ) {
        CheckFile ( $Language1, $Language2, $File);
    }

    #comparing translation files
    my %TranslationFiles;
    find( sub { WantedTranslation($Language1, $Language2, \%TranslationFiles) }, $Directory);

    my @TranslationFiles = keys %TranslationFiles;

    foreach my $TranslationFile ( @TranslationFiles ) {
        CheckTranslationFile ( $Language1, $Language2, $TranslationFile);
    }
}

#========================================================================================
# function     usage
# state        private
#----------------------------------------------------------------------------------------
# syntax       usage ()
#----------------------------------------------------------------------------------------
# description  Prints the required parameters of the script
#========================================================================================
sub usage() {
    print <<END_USAGE;
Description:
    This script compares two languages of all localization files in the passed
    directory (and the recursive following directories). The result of this
    script is a list of missing keywords in these files.

Usage:
    perl $0 -dir -lang1 -lang2

options:
    -dir   directory           ($ENV{EPAGES_CARTRIDGES}/DE_EPAGES) optional
    -lang1 reference language  ()
    -lang2 translated language ()

Example:
    perl $0 -dir C:\\temp -lang1 en -lang2 de
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
