#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::StatusRequest
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Status request.
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::StatusRequest;

use base DE_EPAGES::Core::API::Test;
use strict;

use DE_EPAGES::DocdataPay::API::SOAP::Request::StatusRequest;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Readonly;
use Test::MockObject::Extends;
use Test::Exception;
use Test::More;

sub test {
    my $self = shift;
    $self->_test_positive_status_request();
    $self->_test_status_request_without_APIVersion();
    $self->_test_status_request_without_PaymentOrderKey();
    $self->_test_status_request_without_MerchantPassword();
    done_testing();
}

sub _test_positive_status_request {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAPIVersion      = 1;
    my $IsPaymentOrderKey = 1;
    my $IsPasswordSet     = 1;
    my $MockedLineItemPaymentDocdataPay =
      $self->_getMockedLineItemPaymentDocdataPay( $IsAPIVersion, $IsPaymentOrderKey, $IsPasswordSet );
    my $MockedStatusRequest    = $self->_getMockedStatusRequest();
    my $GeneratedStatusRequest = $MockedStatusRequest->buildStatusRequest($MockedLineItemPaymentDocdataPay);
    my $ExpectedStatusRequest  = $self->_getExpectedStatusRequest();
    is_deeply( $GeneratedStatusRequest, $ExpectedStatusRequest, "$SubTestName - Checks the correctness of the request" )
      or Test::More::diag Dumper($GeneratedStatusRequest);
    return;
}

sub _test_status_request_without_APIVersion {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAPIVersion      = 0;
    my $IsPaymentOrderKey = 1;
    my $IsPasswordSet     = 1;
    my $MockedLineItemPaymentDocdataPay =
      $self->_getMockedLineItemPaymentDocdataPay( $IsAPIVersion, $IsPaymentOrderKey, $IsPasswordSet );
    my $MockedStatusRequest = $self->_getMockedStatusRequest();
    throws_ok { $MockedStatusRequest->buildStatusRequest($MockedLineItemPaymentDocdataPay) } qr/APIVersionNotSet/,
      "$SubTestName - Expected use APIVersionNotSet exception.";
    return;
}

sub _test_status_request_without_PaymentOrderKey {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAPIVersion      = 1;
    my $IsPaymentOrderKey = 0;
    my $IsPasswordSet     = 1;
    my $MockedLineItemPaymentDocdataPay =
      $self->_getMockedLineItemPaymentDocdataPay( $IsAPIVersion, $IsPaymentOrderKey, $IsPasswordSet );
    my $MockedStatusRequest = $self->_getMockedStatusRequest();
    throws_ok { $MockedStatusRequest->buildStatusRequest($MockedLineItemPaymentDocdataPay) } qr/PaymentOrderKeyNotSet/,
      "$SubTestName - Expected use PaymentOrderKeyNotSet exception.";
    return;
}

sub _test_status_request_without_MerchantPassword {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAPIVersion      = 1;
    my $IsPaymentOrderKey = 1;
    my $IsPasswordSet     = 0;
    my $MockedLineItemPaymentDocdataPay =
      $self->_getMockedLineItemPaymentDocdataPay( $IsAPIVersion, $IsPaymentOrderKey, $IsPasswordSet );
    my $MockedStatusRequest = $self->_getMockedStatusRequest();
    throws_ok { $MockedStatusRequest->buildStatusRequest($MockedLineItemPaymentDocdataPay) }
    qr/MerchantPasswordNotSet/,
      "$SubTestName - Expected use MerchantPasswordNotSet exception.";
    return;
}

sub _getMockedLineItemPaymentDocdataPay {
    my $self = shift;
    my ( $IsAPIVersion, $IsPaymentOrderKey, $IsPasswordSet ) = @_;

    my $hAttributes = {
        'APIVersion'      => $IsAPIVersion      ? '1.2'                              : undef,
        'PaymentOrderKey' => $IsPaymentOrderKey ? '8190C56FAE3DF3D0E9920A200B5481AB' : undef,
        'PaymentMethod'   => $self->_getMockedPaymentMethod($IsPasswordSet)
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay',
        $hAttributes );
}

sub _getMockedPaymentMethod {
    my $self = shift;
    my ($IsPasswordSet) = @_;

    my $hAttributes = {
        'MerchantName'     => 'faked_username',
        'MerchantPassword' => $IsPasswordSet ? 'faked_username' : undef
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay',
        $hAttributes );
}

sub _getMockedStatusRequest {
    my $self = shift;

    my $aParameterList =
      [ 'RequestParameters', 'LineItemPaymentDocdataPay', 'PaymentMethodDocdataPay', 'PaymentOrderKey' ];


    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::DocdataPay::API::SOAP::Request::StatusRequest',
        $aParameterList );
    $Mockify->addMock(
        '_TestObject',
        sub {
            shift;
            return;
        }
    );

    return $Mockify->getMockObject();
}

sub _getExpectedStatusRequest {
    my $self = shift;

    my $ExpectedStatusRequest = {
        'version'  => '1.2',
        'merchant' => {
            'password' => 'faked_username',
            'name'     => 'faked_username'
        },
        'paymentOrderKey' => '8190C56FAE3DF3D0E9920A200B5481AB'
    };

    return $ExpectedStatusRequest;
}

__PACKAGE__->runScript();
