#=================================================================================
# package      DE_EPAGES::Installer::API::CPAN
#---------------------------------------------------------------------------------
# description  Functions for installing $EPAGES_CARTRIDGES/CPAN into $EPAGES_PERL
#---------------------------------------------------------------------------------
# state        public
#=================================================================================
package DE_EPAGES::Installer::API::CPAN;
use strict;

use base qw (Exporter);

our @EXPORT_OK = qw (
    TestInstallCPAN
    InstallCPAN
);

use Config;
use Config::IniFiles;
use Sort::Versions qw(versioncmp);
use ExtUtils::Install qw ();
use File::Path qw ( mkpath rmtree );
use File::Spec;
use File::Find;
use Cwd;
use DE_EPAGES::Core::API::Constants qw (SET_EFFECTIVE BASE_RW BASE_RWX);

#========================================================================================
# function     TestInstallCPAN
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahReasons = TestInstallCPAN($aPackages);
# example      $ahReasons = TestInstallCPAN(['CPAN::EPAGES']);
#----------------------------------------------------------------------------------------
# description  Tests if the installation of CPAN modules can succeed for the given cartridges.
#----------------------------------------------------------------------------------------
# input        $aPackages | test these packages | ref.array.string
# input        $Options   | flags to change the behaviour. Following flags can be used:
#                          <ul>
#                           <li>rewrite - reinstall even the package is already installed</li>
#                           <li>verbose - verbose output</li>
#                          </ul>
#                          | hash ref
# return       $ahReasons | reasons why install is not possible | ref.array.hash.string
#========================================================================================
sub TestInstallCPAN ($;$) {
    my ($aPackages, $Options) = @_;
    local $| = 1;

    my $CpanPkg = _ReadModulesIni($aPackages);
    my (undef, $ahReasons) = _TestAllPackages($CpanPkg, $Options);

    return $ahReasons;
}

#========================================================================================
# function     InstallCPAN
# state        public
#----------------------------------------------------------------------------------------
# syntax       InstallCPAN($aPackages);
# example      InstallCPAN(['CPAN::EPAGES']);
#----------------------------------------------------------------------------------------
# description  Installs CPAN modules for the given cartridges.
#----------------------------------------------------------------------------------------
# input        $aPackages | list of cartridge packages | ref.array.string
# input        $Options   | flags to change the behaviour. Following flags can be used:
#                          <ul>
#                           <li>rewrite - reinstall even the package is already installed</li>
#                           <li>verbose - verbose output</li>
#                          </ul>
#                          | hash ref
# return       $ahReasons | reasons why install is not possible | ref.array.hash.string
#========================================================================================
sub InstallCPAN ($;$) {
    my ($aPackages, $Options) = @_;
    local $| = 1;

    my $CpanPkg = _ReadModulesIni($aPackages);

    my ($sortedPackages, $ahReasons) = ([], []);
    if ($Options->{deps}) {
        ($sortedPackages, $ahReasons) = _TestAllPackages($CpanPkg, $Options);
        return $ahReasons if scalar @$ahReasons;
    } else {
        $sortedPackages = _TestAllPackagesNoDeps($CpanPkg, $Options, $aPackages);
    }

    foreach my $CpanDir (@$sortedPackages) {
        print "Installing: $CpanDir\n" if $Options->{verbose};
        my $PackageName = $CpanPkg->{$CpanDir}{'NAME'};
        my $ModulesDir = $CpanPkg->{$CpanDir}{'MODULES_DIR'};
        my $blibDirectory = File::Spec->catdir($ModulesDir, $CpanDir, 'blib');

        _UnpackPackage($ModulesDir, $CpanDir);

        die "blibdir '$blibDirectory' doesn't exist " unless -d $blibDirectory;

        _DeleteObsoleteFiles($CpanPkg->{$CpanDir}{'DELETE'});

        my $archDirUsed = ExtUtils::Install::directory_not_empty("$blibDirectory/arch");
        my $privDirUsed = ExtUtils::Install::directory_not_empty("$blibDirectory/priv");
        my $ArchDir = $privDirUsed ? $Config{installarchlib} : $Config{installsitearch};

        my $FromTo = {
            read => File::Spec->catfile($ArchDir,'auto', split (/:/, $PackageName), '.packlist'),
            write => File::Spec->catfile($ArchDir,'auto', split (/:/, $PackageName), '.packlist'),
            File::Spec->catdir($blibDirectory,'priv') => $archDirUsed ? $ArchDir : $Config{installprivlib},
            File::Spec->catdir($blibDirectory,'lib') => $archDirUsed ? $ArchDir : $Config{installsitelib},
            File::Spec->catdir($blibDirectory,'arch')   => $ArchDir,
            File::Spec->catdir($blibDirectory,'bin')    => $Config{'installbin'},
            File::Spec->catdir($blibDirectory,'script') => $Config{'installscript'},
            };

        $FromTo->{File::Spec->catdir($blibDirectory,'bindoc')} = $Config{'installman1dir'} if $Config{'osname'} ne 'MSWin32';
        $FromTo->{File::Spec->catdir($blibDirectory,'libdoc')} = $Config{'installman3dir'} if $Config{'osname'} ne 'MSWin32';
        $FromTo->{File::Spec->catdir($blibDirectory,'man1')} = $Config{'installman1dir'} if $Config{'osname'} ne 'MSWin32';
        $FromTo->{File::Spec->catdir($blibDirectory,'man3')} = $Config{'installman3dir'} if $Config{'osname'} ne 'MSWin32';

        ExtUtils::Install::install($FromTo);

        # append pod
        my %PODInfo = %{$CpanPkg->{$CpanDir}};
        delete $PODInfo{$_} foreach qw(NAME MODULES_DIR VERSION_FROM PREREQ_PM DELETE INTRODUCED);

        $PODInfo{'LINKTYPE'}  = 'dynamic' unless exists $PODInfo{'LINKTYPE'};
        $PODInfo{'EXE_FILES'} = '';
        my $POD = perllocal_install(
            'Module'          => $PackageName,
            'installed into'  => $Config{'installarchlib'},
            %PODInfo,
            );
        open my $FILE, '>>', File::Spec->catfile($Config{'installarchlib'},'perllocal.pod')
            or die File::Spec->catfile($Config{'installarchlib'},'perllocal.pod'). ": open() failed: $!";
        print $FILE $POD;
        close $FILE or die "close() failed: $!";
    }

    if ($Config{osname} ne 'MSWin32' and scalar @$sortedPackages) {
        File::Find::find({ wanted => \&chperm }, $ENV{EPAGES_PERL});
    }

    return $ahReasons;
}

#========================================================================================
# state        internal
#========================================================================================

sub chperm {
    if ($> == 0) {
        my $Uid = (getpwnam 'ep_appl')[2];
        my $Gid = (getgrnam 'epages')[2];
        chown($Uid, $Gid, $_) or die "chown($Uid, $Gid, $File::Find::name): $!";
    }
    my $Mode = (stat $_)[2] & (BASE_RWX|SET_EFFECTIVE);
    $Mode = -d $_ ? $Mode | BASE_RWX : $Mode | BASE_RW;
    $Mode = BASE_RWX if /\.so$/;
    chmod($Mode, $_) or die "chmod($Mode, $File::Find::name): $!";
    return;
}

# unpack $CpanDir.tgz into $CpanDir (if exist)
sub _UnpackPackage($) {
    my ($ModulesDir, $CpanDir) = @_;

    $CpanDir = File::Spec->catdir($ModulesDir, $CpanDir);

    my $Package;
    if (-f "$CpanDir.tgz-$ENV{EPAGES_KERNEL}") { $Package = "$CpanDir.tgz-$ENV{EPAGES_KERNEL}" }
    elsif (-f "$CpanDir.tgz") { $Package = "$CpanDir.tgz" }
    elsif (-f "$CpanDir.tar.gz") { $Package = "$CpanDir.tar.gz" }
    elsif (-f "$CpanDir.tar") { $Package = "$CpanDir.tar" }
    else { return; }

    _DeleteDir($CpanDir) if -d $CpanDir;

    my $aErrors;
    mkpath($ModulesDir, { mode => BASE_RWX, error => \$aErrors, });
    die "mkpath($ModulesDir): ". join '; ', map { sprintf '%s: %s', each %$_ } @$aErrors
        if scalar @$aErrors;

    my $Decompress = $Package =~ /\.(tgz|tar\.gz)/ ? 'z' : '';
    my $cmd = "tar ${Decompress}xf \"$Package\" -C \"$ModulesDir\"";
    system($cmd);
    die "$cmd: $?" if $?;

    File::Find::find({ wanted => \&chperm }, $ModulesDir) if $^O ne 'MSWin32';
}

# delete directory
sub _DeleteDir($) {
    my ($Dir) = @_;
    return unless -d $Dir;

    if ($Config{'osname'} eq 'MSWin32' && $Dir =~ /^\\\\/) {
        # workaround for UNC paths, see EPG-9636, http://rt.cpan.org/Ticket/Display.html?id=34701
        my $cmd = "rmdir /S /Q \"$Dir\"";
        system($cmd);
        die "$cmd: $?" if $?;
    } else {
        rmtree($Dir) or die "rmtree($Dir): $!";
    }
}

# delete obsolete files
sub _DeleteObsoleteFiles($) {
    my ($DELETE) = @_;
    return unless $DELETE;

    my $ObsoleteFiles = eval $DELETE; ##no critic qw(ProhibitStringyEval)
    die $@ if $@;

    return unless $ObsoleteFiles;

    my %InstallPlace = (
        'priv'   => $Config{'installprivlib'},
        'privarch' => $Config{'installarchlib'},
        'lib'    => $Config{'installsitelib'},
        'arch'   => $Config{'installsitearch'},
        'bin'    => $Config{'installbin'},
        'script' => $Config{'installscript'},
        'man1'   => $Config{'installman1dir'},
        'man3'   => $Config{'installman3dir'},
        );

    foreach my $Key (%$ObsoleteFiles) {
        next unless $InstallPlace{$ObsoleteFiles->{$Key}};
        my $File = "$InstallPlace{$ObsoleteFiles->{$Key}}/$Key";
        $File =~ s/\//\\/go if $Config{'osname'} eq 'MSWin32';

        if (-f $File) {
            print "Deleting $File\n";
            unlink($File);
            if (-f $File) {
                # if move doesn't work (e.g. windows holds a files)
                my $Time = time();
                # rename it
                rename($File, "$File-$Time.obsolete")
            }
            die "unlink($File)" if -f $File;
        } elsif (-d $File) {
            print "Deleting $File\n";
            _DeleteDir($File);
        }
    }
}

# read all modules.ini files and return CPAN package hash
sub _ReadModulesIni($) {
    my ($aPackages) = @_;

    my $CpanPkg;

    foreach my $CartridgePackage ( @$aPackages ) {
        my $ModulesDir = _GetModulesDir($CartridgePackage);
        my $ModulesIni = File::Spec->catfile($ModulesDir, 'modules.ini');

        my %Ini;
        tie %Ini, 'Config::IniFiles', ( -file => $ModulesIni ) or die "\nERROR: $ModulesIni: $!";

        foreach my $Key (keys %Ini) {
            $CpanPkg->{$Key} = $Ini{$Key};
            $CpanPkg->{$Key}{MODULES_DIR} = $ModulesDir;
            ($CpanPkg->{$Key}{VERSION}) = $Key =~ /-(\d.*)$/ unless exists $CpanPkg->{$Key}{VERSION};
        }
    }
    return $CpanPkg;
}

# returns a list of packages to be installed in right order
sub _TestAllPackagesNoDeps($;$) {
    my ($CpanPkg, $Options, $aPackages) = @_;
    my ($found, $sorted, $failed) = ([], [], {});

    my $PkgHash = {};
    my @KeyList;
    foreach my $CartridgePackage ( @$aPackages ) {
        my $ModulesDir = _GetModulesDir($CartridgePackage);
        my $ModulesIni = File::Spec->catfile($ModulesDir, 'modules.ini');

        foreach (@{_GetFileContentLines($ModulesIni)}) {
            next unless /^\s*\[([^]]+)\]/;
            my $Key = $CpanPkg->{$1}{NAME};
            push @KeyList, $Key unless $Key ~~ @KeyList;
            $PkgHash->{$Key}{path} = $1;
            $PkgHash->{$Key}{vers} = $CpanPkg->{$1}{VERSION};
            $PkgHash->{$Key}{from} = $CpanPkg->{$1}{VERSION_FROM};
            $PkgHash->{$Key}{deps} = {};
        }
    }

    map { _TestPackage($_, $PkgHash, $found, $sorted, $failed, $Options); } @KeyList;

    return [ map { $PkgHash->{$_}{path}; } @$sorted ];
}

# returns a list of messages if any prerequisites are missing
# and a list of packages to be installed in right order
sub _TestAllPackages($;$) {
    my ($CpanPkg, $Options) = @_;
    my ($found, $sorted, $failed) = ([], [], {});

    my $PkgHash = {};
    foreach my $Key (keys %$CpanPkg) {
        $PkgHash->{$CpanPkg->{$Key}{NAME}}{path} = $Key;
        $PkgHash->{$CpanPkg->{$Key}{NAME}}{vers} = $CpanPkg->{$Key}{VERSION};
        $PkgHash->{$CpanPkg->{$Key}{NAME}}{from} = $CpanPkg->{$Key}{VERSION_FROM};
        if (exists $CpanPkg->{$Key}{PREREQ_PM}) {
            ##no critic qw(ProhibitStringyEval)
            $PkgHash->{$CpanPkg->{$Key}{NAME}}{deps} = eval $CpanPkg->{$Key}{PREREQ_PM};
            ## use critic
            die $@ if $@;
        } else {
            $PkgHash->{$CpanPkg->{$Key}{NAME}}{deps} = {};
        }
        if (exists $CpanPkg->{$Key}{VERSION_MAP}) {
            ##no critic qw(ProhibitStringyEval)
            $PkgHash->{$CpanPkg->{$Key}{NAME}}{map} = eval $CpanPkg->{$Key}{VERSION_MAP};
            ## use critic
            die $@ if $@;
        }
    }

    map { _TestPackage($_, $PkgHash, $found, $sorted, $failed, $Options); } keys %$PkgHash;

    # TestInstallCPAN and InstallCPAN shall return variable of same type as before
    my @aReasons;
    push @aReasons, { 'Module' => $_, 'Reasons' => $failed->{$_} } foreach sort keys %$failed;

    return ([ map { $PkgHash->{$_}{path}; } @$sorted ], \@aReasons);
}

# check prerequisites for a single package
sub _TestPackage {
    my ($Pkg, $PkgHash, $found, $sorted, $failed, $Options) = @_;

    return if $Pkg ~~ @$found;
    push @$found, $Pkg;

    foreach my $prereq (keys %{$PkgHash->{$Pkg}{deps}}) {
        my $version = _GetModuleVersion($prereq, $PkgHash->{$prereq}{from});
        print "package $Pkg: requires=$prereq >= $version\n" if $Options->{test} and $Options->{verbose};

        my $NotInstalled = $version ? 0 : 1;
        my $NotUpToDate = versioncmp( $version, $PkgHash->{$Pkg}{deps}{$prereq} ) < 0 ? 1 : 0;

        if ($NotUpToDate and exists $PkgHash->{$prereq}{vers}) {
            $NotUpToDate = versioncmp( $PkgHash->{$prereq}{vers}, $PkgHash->{$Pkg}{deps}{$prereq} ) < 0 ? 1 : 0;
        }

        if ($NotUpToDate) {
            push @{$failed->{$Pkg}}, $NotInstalled ? "$prereq: prerequisite not installed (req. version: $PkgHash->{$Pkg}{deps}{$prereq})"
                : "$prereq: prerequisite not up-to-date (has/req. version: $version/$PkgHash->{$Pkg}{deps}{$prereq})";
        }

        _TestPackage($prereq, $PkgHash, $found, $sorted, $failed, $Options) if exists $PkgHash->{$prereq};
    }

    my $RunVersion = _GetModuleVersion($Pkg, $PkgHash->{$Pkg}{from});
    $RunVersion = $PkgHash->{$Pkg}{map}{$RunVersion} if defined $PkgHash->{$Pkg}{map}{$RunVersion};

    my $PkgVersion = $PkgHash->{$Pkg}{vers};
    $PkgVersion = $PkgHash->{$Pkg}{map}{$PkgVersion} if defined $PkgHash->{$Pkg}{map}{$PkgVersion};

    my $UpToDate = versioncmp($RunVersion, $PkgVersion) >= 0;
    if (not $UpToDate and not $Options->{rewrite}) {
        # version might start with v, e.g. v1.1.1
        $RunVersion =~ s,^v,,;
        $PkgVersion =~ s,^v,,;
        $UpToDate = versioncmp($RunVersion, $PkgVersion) >= 0;
        if (not $UpToDate and $PkgVersion =~ s/(\d)000+$/$1/) {
            # exception: VERSION in modules.ini ends with at least 3 zeroes
            $UpToDate = versioncmp($RunVersion, $PkgVersion) >= 0;
        }
    }
    print "package $Pkg: has=$RunVersion cpan=$PkgVersion\n" if $Options->{verbose};
    return if $UpToDate and not $Options->{rewrite};

    push @$sorted, $Pkg;
    return;
}

# returns undef   if file is undef
# returns undef   if file is installed and version isn't found
# returns version if file is installed and version is found
# returns 0       if file is not installed
sub _GetModuleVersionFromFile {
    my ($File) = @_;

    # version is undef if argument is undef
    return unless defined $File;

    $File =~ s,^(priv|lib|arch)/,,;

    my $IncFile;
    foreach my $IncDir (@INC) {
        if (-f "$IncDir/$File") {
            $IncFile = "$IncDir/$File";
            last;
        }
    }

    # version is 0 unless $IncFile exists
    return 0 unless $IncFile;

    my $InstalledVersion;
    foreach (@{_GetFileContentLines($IncFile)}) {
        next unless /(\$|::)VERSION\s*=(\s*q.?)?\W*(\d[\d.]*)/;
        $InstalledVersion = $3;
        last;
    }

    # version is undef if can't get version from $IncFile
    return unless defined $InstalledVersion;

    # delete leading zeros
    $InstalledVersion =~ s,^0+([1-9]),$1,;
    return $InstalledVersion;
}

# returns the version of the installed package, otherwise returns 0
sub _GetModuleVersion($;$) {
    my ($Package, $File) = @_;

    die "Invalid Package $Package!" unless $Package =~/^[a-zA-Z_]\w*(?:::\w+)*\z/x;

    # first try: read file
    my $InstalledVersion = _GetModuleVersionFromFile($File);

    return $InstalledVersion if defined $InstalledVersion;

    # second try: execute perl
    # get the Version with perl -e because we get the version from modules who are using dll's.
    # if we "require" this module we can not overwrite dll's because they are locked!!
    my $quot = $^O eq 'MSWin32' ? '"' : '\'';
    my $cmd = "$ENV{PERL} -e ${quot}eval {require ($Package);}; if(\$\@) {print 0;} else {print $Package->VERSION;}${quot}";
    $InstalledVersion = `$cmd`;
    die "$cmd: $?\n" if $?;

    # delete leading zeros
    $InstalledVersion =~ s,^0+([1-9]),$1,;
    return $InstalledVersion;
}

# returns directory PerlLib/OS depending on OS, e.g. $EPAGES_CARTRIDGES/CPAN/EPAGES/PerlLib/MSWin32
sub _GetModulesDir {
    my ($CartridgePackage ) = @_;

    my ($ArchName, $ArchName32) = _GetArchName();
    my $ModulesDir;

    if ($CartridgePackage =~ m,[/\\],) {
        $ModulesDir = File::Spec->catdir($CartridgePackage, 'PerlLib', $ArchName);
        # if special directory does not exist, use 32-bit directories
        $ModulesDir = File::Spec->catdir($CartridgePackage, 'PerlLib', $ArchName32) unless -d $ModulesDir;
        # if special directory does not exist, use old directories with osname (linux, solaris, MSWin32)
        $ModulesDir = File::Spec->catdir($CartridgePackage, 'PerlLib', $Config{osname}) unless -d $ModulesDir;
        $ModulesDir = File::Spec->rel2abs($ModulesDir);
    } else {
        $ModulesDir = File::Spec->catdir($ENV{'EPAGES_CARTRIDGES'}, split (/::/, $CartridgePackage), 'PerlLib', $ArchName);
        # if special directory does not exist, use 32-bit directories
        $ModulesDir = File::Spec->catdir($ENV{'EPAGES_CARTRIDGES'}, split (/::/, $CartridgePackage), 'PerlLib', $ArchName32) unless -d $ModulesDir;
        # if special directory does not exist, use old directories with osname (linux, solaris, MSWin32)
        $ModulesDir = File::Spec->catdir($ENV{'EPAGES_CARTRIDGES'}, split (/::/, $CartridgePackage), 'PerlLib', $Config{osname}) unless -d $ModulesDir;
    }
    return $ModulesDir;
}

sub _GetArchName () {
    my ($ArchName, $ArchName32);
    if ( $Config{osname} eq 'solaris' ) {
        if ( $Config{byteorder} eq '12345678' ) {
            $ArchName = 'i86pc-solaris64';
            $ArchName32 = 'i86pc-solaris';
        } elsif  ( $Config{byteorder} eq '87654321' ) {
            $ArchName = 'sun4-solaris64';
            $ArchName32 = 'sun4-solaris';
        } elsif ( $Config{byteorder} eq '1234' ) {
            $ArchName =  'i86pc-solaris';
            $ArchName32 = $ArchName;
        } elsif ( $Config{byteorder} eq '4321' ) {
            $ArchName =  'sun4-solaris';
            $ArchName32 = $ArchName;
        }
    } elsif ( $Config{byteorder} eq '12345678' )  {
        $ArchName = "x86_64-$Config{osname}";
        $ArchName32 = "i586-$Config{osname}";
    } elsif ( $Config{byteorder} eq '1234' )  {
        $ArchName = "i586-$Config{osname}";
        $ArchName32 = $ArchName;
    }
    return ($ArchName, $ArchName32);
}

sub _GetFileContentLines {
    my ($FileName) = @_;
    die "$FileName: file not found\n" unless -f $FileName;

    local $/ = "\n";
    my @Content;

    open my $FILE_DESCR, '<', $FileName or ##no critic (RequireBriefOpen)
        die "$FileName: file open failed: $!\n";
    @Content = <$FILE_DESCR>;
    close $FILE_DESCR or die "$FileName: file close failed: $!\n";

    return \@Content;
}

# copy from package ExtUtils::Command::MM;
sub perllocal_install { ##no critic qw(RequireArgUnpacking)
    my($type, $name) = splice(@_, 0, 2);

    my $pod;
    $pod = sprintf <<POD, scalar localtime;
 =head2 %s: C<$type> L<$name|$name>

 =over 4

POD

    do {
        my($key, $val) = splice(@_, 0, 2);

        $pod .= <<POD
 =item *

 C<$key: $val>

POD

    } while(@_);

    $pod .= "=back\n\n";
    $pod =~ s/^ //mg;
    return $pod;
}

1;
