use strict;
use warnings;
use File::Copy::Recursive qw(dircopy pathrm);
use File::Find;


# Prototypes and globals
sub Main ($$$);
my %newModules;

# Call:
# perl copyBinaryModule.pl Perl::Critic  d:\devel\cpan D:\epages\Cartridges\CPAN\EPAGES\PerlLib\MSWin32\
# ------------------------------------------------------------------------------------

if (scalar @ARGV == 0) {
    print "usage: copyBinaryModule.pl Module::Name [module_source_directory] [install_destination_directory]\n";
    exit;
}

unless ($ENV{TEMP}) {
    print "No TEMP variable exists; please create one pointing to your temporary directory using e.g.'export TEMP=/tmp'\n";
    exit;
}

my ($MODULE, $MODULE_SRC, $DST) = @ARGV;
my $SRC = MakeModules($MODULE, $MODULE_SRC);
unless ($DST) {
   my $osPath = $ENV{windir} ? 'MSWin32' : 'linux';
   $DST = "$ENV{EPAGES_CARTRIDGES}/CPAN/EPAGES/PerlLib/$osPath/"
}

Main($SRC, $DST, $MODULE_SRC);


# ------------------------------------------------------------------------------------


sub Main($$$) {
    my ($Src, $Dest, $ModuleSrc) = @_;

    print "Using SRC $Src....\n";

    # Read all meta information from the new module.
    my $meta = ReadMetaInfo ($Src);
    return unless $meta;


    # Copy director with blib files from the new module.
    my %requiredModules = $meta->GetRequiredModules();
    my $aModulesToInstall = CopyFiles($Src, $Dest, \%requiredModules);

    # Do the same for all prerequesites recursively.
    foreach my $modulePath ( keys %$aModulesToInstall ) {
        Main($modulePath, $Dest, $ModuleSrc);
    }

    # Update modules.ini file with the new module.
    UpdateModulesIni( $meta, $Dest, \%requiredModules );

    return;
}

sub MakeModules {
    my ($mod, $mod_dst) = @_;

    my $cpan = CPANPLUS::Backend->new;

    # configure build and fetch paths
    my $conf = $cpan->configure_object;
    my $extractdir = "$ENV{TEMP}/cpanbuild";
    mkdir $extractdir;
    $conf->set_conf( extractdir => $extractdir );
    $conf->set_conf( fetchdir => $mod_dst ) if $mod_dst;

    my $obj = $cpan->module_tree($mod) or die('No such module found!');

    my $ok = $obj->create;

    if ($ok) {
        my $name = $obj->name . '-' . $obj->version;
        $name =~ s/::/-/g;

        return $extractdir . '\/' . $name;
    }
    else {
        die "Error during make of $mod or required modules!\n";
    }
}

sub UpdateModulesIni ($$$) {
    my ($meta, $Dest, $hRequiredModules) = @_;

    open my $MODULES_INI, '>>', "$Dest/modules.ini" or die $!;
    print $MODULES_INI  "\n";
    print $MODULES_INI  '[' . $meta->{'ymlname'} . '-' . $meta->{'ymlversion'} . "]\n";
    print $MODULES_INI  'NAME=' . $meta->{'name'} . "\n";
    print $MODULES_INI  'VERSION_FROM=' . $meta->{'version_from'} . "\n";
    print $MODULES_INI  'PREREQ_PM={' ;
    while ( my ($key, $value) = each(%$hRequiredModules) ) {
        $key =~ s/\//::/g;
        print $MODULES_INI "'$key' => $value, ";
    }
    print $MODULES_INI   "}\n";
    close $MODULES_INI or die $!;
    $newModules{$meta->{'name'}} = 1;

    return;
}

sub ReadMetaInfo ($) {
    my ($Src) = @_;

    my $meta = ModuleMetaInfo->new();
    my $isInRequiresSection = 0;

    open my $META_YML, '<', "$Src/META.yml" or return; ##no critic (RequireBriefOpen)
    while (<$META_YML>) {
        # parse name
        if(/^name\:\s+(.*)/) {
            $meta->{'ymlname'} = $1;
            $meta->{'name'} = $1;
            $meta->{'name'} =~ s/-/::/g;
        }
        # parse version
        elsif(/^version\:\s+(.*)/) {
            $meta->{'ymlversion'} = $1;
        }
        # find required modules
        elsif(/^requires\:/) {
            $isInRequiresSection = 1;
        }
        elsif(/^build_requires\:/) {
            $isInRequiresSection = 1;
        }
        elsif(/^\w+\:/) {
            $isInRequiresSection = 0;
        }
        elsif($ENV{PERL} ne '/var/epages/bin/perl' and $isInRequiresSection) {
            my($key, $value) = $_ =~ /\s+(.*)\: ([0-9\.]+)/;

            if($key && $key ne 'perl') {
                $key =~ s/::/\//g;
                my $perlDir1 = "$ENV{EPAGES_PERL}/site/lib/$key";
                my $perlDir2 = "$ENV{EPAGES_PERL}/lib/$key";

                unless( -e $perlDir1 || -e $perlDir2 || -e "$perlDir1.pm" || -e "$perlDir2.pm" ) {
                    #print "Adding: $key\n";
                    $meta->AddRequiredModule($key, $value);
                }
            }
        }
    }
    close $META_YML or return;

    # read module infos from makefile
    open my $MAKEFILE, '<', "$Src/Makefile.PL" or die 'Makefile not found!!';
    while (<$MAKEFILE>) {
        # parse version
        if(/VERSION_FROM\W+[q'\[\"]*([\w\/\.]+)/i) {
            $meta->{'version_from'} = $1;
        }
        elsif(/ALL_FROM\W+[q'\[\"]*([\w\/\.]+)/i) {
            $meta->{'version_from'} = $1;
        }
    }
    close $MAKEFILE or die $!;

    return $meta;
}

sub CopyFiles ($$$) {
    my ($Src, $Dst, $hRequiredModules) = @_;

    my @srcPaths = split /\//, $Src;

    my $lastPath = $srcPaths[scalar @srcPaths-1];

    mkdir "$Dst/$lastPath";
    mkdir "$Dst/$lastPath/blib";
    dircopy("$Src/blib","$Dst/$lastPath/blib");

    # delete man directories
    pathrm("$Dst/$lastPath/blib/man1");
    pathrm("$Dst/$lastPath/blib/man3");

    if(-d "$Src/bin") {
        mkdir "$Dst/$lastPath/bin";
        dircopy("$Src/bin","$Dst/$lastPath/bin");
    }

    # replace eproot string
    find(\&eprootSubst, ("$Dst/$lastPath/blib"));

    print "blib and lib path copied to $Dst/$lastPath\n";

    my %modulesToInstall;
    if( $Src =~ /(.*)$lastPath/ ) {
        my $srcRootPath = $1;

        while ( my ($key, $value) = each(%$hRequiredModules) ) {
           # key ~ PPI/Document/File
           # value ~ 1.024

            my @pathParts = split /\//, $key;
            $key =~ s/\//-/g;
            my $pathTest = $key;
            foreach my $pathPart (reverse @pathParts) {
                opendir (DIR, $srcRootPath) or die "Couldn't open directory, $!";
                while (my $file = readdir DIR) {
                    if( $file =~ /^\Q$pathTest/ and not -d "$Dst/$file" ) {
                        $modulesToInstall{"$srcRootPath/$file"} = 1;
                        last;
                    }
                }
                $pathTest =~ s/$pathPart-*//;
            }
        }
    }
    return \%modulesToInstall;
}

sub eprootSubst {
    #print "File: $File::Find::name\n";
    my $quotedString = "\Q$ENV{EPAGES}";
    my $oldPerl = "\Q#!/usr/bin/perl";
    my $newPerl = "\Q#!/bin/env perl";

    if(-e $File::Find::name ) {
        open my $INF, '<', $File::Find::name or die "Could not read file! $1";
        my @contents = (<$INF>);
        close $INF or die "Could not close file! $1";

        open my $OUF, '>', $File::Find::name or die "Could not write file! $1";
        foreach my $line (@contents) {
            my $x = $line;
            $x =~ s/$quotedString//g;
            $x =~ s/$oldPerl/$newPerl/g;
            print $OUF $x."\n";
        }
        close $OUF or die "Could not close file! $1";
    }

    return;
}
# install CPAN
#system("perl $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Installer/Scripts/installCPAN.pl CPAN::EPAGES")


