package DE_EPAGES::Object::t::Varnish; ## no critic (RequireCommentForExportedFunctions)
use base DE_EPAGES::Core::API::Test;

use strict;
use feature 'switch';

use File::Basename qw( dirname );
use Test::More;
use DE_EPAGES::Core::API::EpagesConfig;
use DE_EPAGES::Object::API::PageCache;
use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;

sub test {
    my $self = shift;

    # given a mock configuration
    my $MockEpagesConfig = TestConfig->new();

    # when I instantiate a varnish page cache
    my $VarnishBackend = DE_EPAGES::Object::API::PageCache::Varnish->new('EpagesConfig' => $MockEpagesConfig);

    # then the varnish urls should have been registered.
    is_deeply($VarnishBackend->varnishUrls(), [qw( http://localhost:23456 http://localhost:23457 )], 'Varnish URLs have been registered');

    # and we should be able to clear the page cache
    $self->test_clear_pagecache($VarnishBackend);

    done_testing();
}

sub test_clear_pagecache {
    my $self = shift;
    my ($VarnishBackend) = @_;

    # given a mock log
    my $TestLog = TestLog->new();
    $VarnishBackend->{'Log'} = $TestLog;

    # given a mock site
    my $MockSite = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new()->buildSite({
        'GUID' => 'beac5d12-2806-11e6-9897-670ebd39606a'
    })->getMockObject();

    # when i clear site page cache
    $VarnishBackend->clearSiteCache($MockSite);

    # then there should be two debug statements.
    is_deeply($TestLog->{debugs}, [
    'Cleaning Varnish Cache via BAN request. Site=beac5d12-2806-11e6-9897-670ebd39606a VarnishUrl=http://localhost:23456',
    'Cleaning Varnish Cache via BAN request. Site=beac5d12-2806-11e6-9897-670ebd39606a VarnishUrl=http://localhost:23457'
    ], 'Contains two debug statements');

    # and there should be two error statements (as we don't mock the HTTP call yet)
    is(scalar @{$TestLog->{errors}}, 2, 'Contains two error statements');
    like($TestLog->{errors}[0],
         qr|^Failed updating Varnish Cache .* 500 Can't connect to localhost:23456|,
         q|Expected error: Can't connect to localhost:23456|);
    like($TestLog->{errors}[1],
         qr|^Failed updating Varnish Cache .* 500 Can't connect to localhost:23457|,
         q|Expected error: Can't connect to localhost:23457|);

    # and http timeout should be read from the configuration
    is($VarnishBackend->httpTimeout(),'20', 'HTTP timeout is read from the configuration.');

}

__PACKAGE__->runScript;

package TestConfig; ## no critic (ProhibitMultiplePackages)
sub new {
    my $class = shift;
    return bless {}, $class;
}
sub get {
    my $self = shift;
    my ($Section, $Key) = @_;
    given ($Section . '::' . $Key) {
        when ('DE_EPAGES::Object::VarnishUrls') { return 'http://localhost:23456,http://localhost:23457' }
        when ('DE_EPAGES::Object::VarnishHttpTimeOut') { return '20' }
        default { }
    }
    return undef;
}

package TestLog; ## no critic (ProhibitMultiplePackages)
sub new {
    my $class = shift;
    return bless {'debugs' => [], 'errors' => [] }, $class;
}
sub debug {
    my $self = shift;
    push @{$self->{debugs}}, shift;
}
sub error {
    my $self = shift;
    push @{$self->{errors}}, shift;
}

1;
