#==============================================================================
# package      DE_EPAGES::Order::UI::Shop
# state        public
# description  UI class for Shop
#==============================================================================
package DE_EPAGES::Order::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use Config;

use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::File qw( ConvertPathChars );
use DE_EPAGES::Core::API::Log qw ( SetLogLevel LogDebug LogError );
use DE_EPAGES::Core::API::PerlTools qw( AddDuration ElementOf SubtractDuration GetCurrentDateTime );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Design::API::Style qw(InvalidateAllShopStyles);
use DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Order::API::Basket qw (GetBasket);
use DE_EPAGES::Order::API::OrderDocument qw (GetCustomerInformationPDFAttachment);
use DE_EPAGES::Order::API::SalesVolume qw (GetSalesVolume);
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template  qw ( ProcessPage );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards ReplaceWildcardsWithoutPosOne);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#==============================================================================
sub SearchOrders {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Shop, 'SearchOrders');
    $SearchForm->{'OrderAliasString'} = $SearchForm->{'WildSearchString'} if defined $SearchForm->{'WildSearchString'};

    return $self->_SearchOrders($Servlet, $SearchForm);
}

#==============================================================================
sub SearchOrdersInbox {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Shop, 'SearchOrders');

    $SearchForm->{'ViewedOn'} = 2;
    $Servlet->vars('Inbox', 1);

    return $self->_SearchOrders($Servlet, $SearchForm);
}

sub UnitySearchOrders {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnitySearchOrders', { 'Servlet' => $Servlet });

    return $self->SearchOrders($Servlet);
}

sub UnitySearchOrdersInbox {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnitySearchOrdersInbox', { 'Servlet' => $Servlet });

    return $self->SearchOrdersInbox($Servlet);
}

#==============================================================================
sub _SearchOrders {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;
    my $Shop = $Servlet->object;

    # define which search to use, a detailed one or a simple
    my $SearchName = 'DetailedSearch';
    $SearchName = 'SimpleSearch' if
        !defined($SearchForm->{'Search'})
        || (ElementOf($Servlet->params('OrderBy'), qw(Alias CreationDate))
          && $SearchForm->{'Search'} eq 'SimpleSearch');
    $SearchForm->{'Search'} = $SearchName; #needed to tell pager with search to use

    # set default sort order
    $Servlet->vars('OrderBy', 'CreationDate') unless $Servlet->params('OrderBy');
    $Servlet->vars('OrderDesc', 1) unless defined $Servlet->params('OrderDesc');

    # EPG-2879: provide original input values as TLE
    $Servlet->vars('CreationDateFrom', $Servlet->form->value('CreationDateFrom'));
    $Servlet->vars('CreationDateTo', $Servlet->form->value('CreationDateTo'));

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('CustomerOrder'), $SearchName);
    my $ClassLineItemShipping = LoadClassByAlias('LineItemShipping');
    my @LineItemShippingClassIDs = ( $ClassLineItemShipping->id, map { $_->id } @{$ClassLineItemShipping->subClasses} );
    my $AttributeShippingMethodID = $ClassLineItemShipping->attribute('ShippingMethod')->id;
    my $AttrPaymentMethodNameID   = LoadClassByAlias('PaymentMethod')->attribute('Name')->id;
    my $AttrShippingMethodNameID  = LoadClassByAlias('ShippingMethod')->attribute('Name')->id;

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
            'LineItemShippingClassIDs' => \@LineItemShippingClassIDs,
            'AttributeShippingMethodID' => $AttributeShippingMethodID,
            'AttrShippingMethodNameID' => $AttrShippingMethodNameID,
            'AttrPaymentMethodNameID'  => $AttrPaymentMethodNameID,
        },
        # set default values (no custom values!)
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => 1,
    );
    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } elsif ($Parameter eq 'CreationDateTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);
        } else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-2879: use original input values in pager URL
        if( $Parameter =~ /^CreationDate(From|To)$/ ) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    if (defined $Options{'Parameters'}{'ProductAliasString'}) {
        #ProductLineItemClassIDs
        my $LineItemProduct = LoadClassByAlias('LineItemProduct');
        my $Classes = $LineItemProduct->subClasses;
        unshift @$Classes, $LineItemProduct;
        $Options{'Parameters'}{'ProductLineItemClassIDs'} = [ map { $_->id } @$Classes ];
    }

    if (defined $Options{'Parameters'}{'ShippingMethodSelector'}) {
        $Options{'Parameters'}{'ShippingMethodID'} = $Options{'Parameters'}{'ShippingMethodSelector'};
    }
    if (defined $Options{'Parameters'}{'PaymentMethodSelector'}) {
        $Options{'Parameters'}{'PaymentMethodID'} = $Options{'Parameters'}{'PaymentMethodSelector'};
    }

    LogDebug('SearchForm', $SearchForm);
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

#==============================================================================
# function     SaveOrderSettings
# state        public
#------------------------------------------------------------------------------
# syntax       $Shop->SaveOrderSettings();
#------------------------------------------------------------------------------
# description  Saves the settings of an order. For this some settings will be
#               deleted if the feature is disabled.
#==============================================================================
sub SaveOrderSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hValues = $Servlet->form->form($Shop, 'OrderSettings');

    # EPG-8337: some settings are only availabe with certain features
    delete $hValues->{$_} foreach grep { !defined $hValues->{$_} } keys %$hValues;

    # EPG-14241 After changing basket settings "allow order" both options, "yes" and "no", are activated and saved
    # add a missing formfield
    if ($hValues->{'LoginRequired2Order'} == 0) {
        $hValues->{'LoginRequired2InstantlyOrder'} = 0;
        $hValues->{'IsPriceOnlyForRegisteredUser'} = 0;
    }

    # get setting for invalidating shop styles, if order process settings were changed
    InvalidateAllShopStyles($Shop) unless ($hValues->{'OrderProcess'} == $Shop->get('OrderProcess'));

    $Shop->set($hValues);

    $Shop->updatePageCache;
}

#==============================================================================
sub SaveOrderSettingsTexts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    $Form->form($Shop, 'OrderSettingsTexts');

    $self->SUPER::Save($Servlet);
    $Shop->updatePageCache;
}

#==============================================================================
sub SaveUnityOrderSettingsTexts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    $Form->form($Shop, 'UnityOrderSettingsTexts');

    $self->SUPER::Save($Servlet);
    $Shop->updatePageCache;
}

#========================================================================================
# function     TransactionPaymentName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  Returns the name of the payment type as string. This name is used
#               for the transaction search. A search definition with the name
#               'Search'.$self->TransactionPaymentName().'Transactions' must exist
#               in class Shop.
#               This method has to be overloaded by sub classes. The default
#               implementation throws an error.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { die 'Method TransactionPaymentName is not implemented'; }

#========================================================================================
# function     ViewTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Simple user interface for searching transactions if
#               <function SearchTransactions> failed.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub ViewTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my %SearchForm = ();

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'TransTime');
        $Servlet->params('OrderDesc', 1);
        $SearchForm{'OrderBy'}   = 'TransTime';
        $SearchForm{'OrderDesc'} = 1;
    }

    #--- add string fields if submitted
    foreach my $field (qw(TransID TransStatus)) {
        $SearchForm{$field} = $Servlet->params($field) if (defined $Servlet->params($field));
    }

    #--- add time fields if submitted
    foreach my $field (qw(TransTimeFrom TransTimeTo)) {
        eval {
            $SearchForm{$field} = $Servlet->form()->convertDateTime($Servlet->params($field),{'Type'=>'reg_date'}) if defined $Servlet->params($field);
        };
        if (ExistsError() ){
            Warning(GetError()) ;
        }
    }

    #--- remove empty entries
    delete $SearchForm{$_} foreach grep { $SearchForm{$_} !~ /\S/ } keys %SearchForm;

    #--- do the search
    return $self->_ExecuteTransactionSearch($Servlet, \%SearchForm);
}

#========================================================================================
# function     SearchTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SearchTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Searches for transactions of the current payment type.
#               The TLE variable #Pager contains the search results.
#               See also <function TransactionPaymentName>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub SearchTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $SearchForm = $Servlet->form->form($Shop, 'Search' . $self->TransactionPaymentName() . 'Transactions');

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'TransTime');
        $Servlet->params('OrderDesc', 1);
# don't add values to search form because they would than appear in the pager utr leading to duplicate get parameters OrderBy and OrderDesc in sort links
#        $SearchForm->{'OrderBy'}   = 'TransTime';
#        $SearchForm->{'OrderDesc'} = 1;
    }

    return $self->_ExecuteTransactionSearch($Servlet, $SearchForm);
}

#========================================================================================
# function     ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub _ExecuteTransactionSearch {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'Search'} = 'Transactions' unless exists $SearchForm->{'Search'};
    my $Shop = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('LineItemPayment' . $self->TransactionPaymentName()), $SearchForm->{'Search'});

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    my $hSearchOptions = $self->SearchOptions();
    $Options{'Parameters'}{$_} = $hSearchOptions->{$_} foreach keys %$hSearchOptions;

    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } elsif ($Parameter eq 'TransTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);

        }else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-12969: use original input values in pager URL
        if( $Parameter =~ /^TransTime(From|To)$/ ) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

#========================================================================================
# function     SearchOptions
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hOptions = $self->SearchOptions();
#----------------------------------------------------------------------------------------
# description  return search options of payment type as hash reference.
#----------------------------------------------------------------------------------------
# return       $hOptions | additional search options for payment type | ref.hash
#========================================================================================
sub SearchOptions { return {}; }

#==============================================================================
sub CreateBasket{
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $User = $Servlet->user;
    $User = $Servlet->createAnonymousUser($Shop) unless defined $User;
    return GetBasket($User,$Shop);
}

#========================================================================================
# function     ViewPrintMultipleDocuments
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewPrintMultipleDocuments( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction to show the print view containing the invoice and packing slip
#               of the requested list of orders. Generates new #Orders array ref TLE
#               containing an #Invoice and a #PackingSlip TLE hash ref.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ViewPrintMultipleDocuments {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    my @Orders = $Form->value('ListObjectID');
    my $aAllOrders;     # TLE array ref containing orders together with their latest invoice & packing slip
    my $PageTypePackingSlip = LoadPageTypeByAlias('MBO-PrintDocumentPackingSlip');
    my $PageTypeInvoice = LoadPageTypeByAlias('MBO-PrintDocumentInvoice');

    foreach my $OrderID (@Orders) {
        my $Order = $Shop->loadObject($OrderID, 'CustomerOrder');
        # don't allow to access orders of other shops
        $Servlet->testPermission($Order, 'MBO-ViewGeneral');

        my $hOrderDocuments = { 'OrderID' => $OrderID };
        my $Container = $Order->container;
        my $LanguageCode = defined $Container ? GetCodeByLanguageID($Container->get('LanguageID')) : undef;

        # latest invoice
        if($Form->value('PrintInvoices')){
            my @Invoices;
            if($Order->findChild('Invoices') and $Order->child('Invoices')->children) {
                @Invoices = @{$Order->child('Invoices')->children};
            }
            if(@Invoices and scalar(@Invoices) > 0) {
                # find latest invoice
                my $LatestInvoice = (sort { $b->id <=> $a->id} @Invoices)[0];
                my $ObjectPageTypeInvoice = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageTypeInvoice, $Shop );
                $hOrderDocuments->{'Invoice'} = ProcessPage($LatestInvoice, $ObjectPageTypeInvoice, $LanguageCode, $LatestInvoice->tleHash);
            }
        }

        # latest packing slip
        if($Form->value('PrintPackingSlips')){
            my @PackingSlips;
            if($Order->findChild('PackingSlips') and $Order->child('PackingSlips')->children) {
                @PackingSlips = @{$Order->child('PackingSlips')->children};
            }
            if(@PackingSlips and scalar(@PackingSlips) > 0) {
                # find latest PackingSlip
                my $LatestPackingSlip = (sort { $b->id <=> $a->id} @PackingSlips)[0];
                my $ObjectPageTypePackingSlip = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageTypePackingSlip, $LatestPackingSlip );
                $hOrderDocuments->{'PackingSlip'} = ProcessPage($Shop, $ObjectPageTypePackingSlip, $LanguageCode, $LatestPackingSlip->tleHash);
            }
        }

        push(@$aAllOrders, $hOrderDocuments);
    }
    $Servlet->vars('Orders', $aAllOrders);
}

#==============================================================================
sub SaveOrderDocumentText{
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_SaveOrderDocuments($Servlet, 'OrderDocumentText');
}

#==============================================================================
sub _SaveOrderDocuments{
    my $self = shift;
    my ($Servlet, $FormName) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($Shop, $FormName);
    my $LanguageID = delete $hValues->{'LanguageID'};

    $Shop->set($hValues, $LanguageID);
    return;
}

#==============================================================================
sub UnitySaveOrderDocumentInvoice{
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_SaveOrderDocuments($Servlet, 'UnityOrderDocumentInvoice');
}

#==============================================================================
sub UnitySaveOrderDocumentPackingSlip{
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_SaveOrderDocuments($Servlet, 'UnityOrderDocumentPackingSlip');
}

#==============================================================================
sub UnitySaveOrderDocumentInvoiceCorrection{
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_SaveOrderDocuments($Servlet, 'UnityOrderDocumentInvoiceCorrection');
}

#==============================================================================
sub SaveOrderDocumentInvoiceInMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my %ActiveAliases = map { $_ => 1 } $Servlet->form->value('Alias');
    my $aShopMailTypes = LoadRootObject()->folder('ShopMailTypeTemplates')->children;

    my %FormErrorAlias; # list of aliases with errors
    foreach my $MailTypeTemplate ( @$aShopMailTypes ) {
        my $Alias = $MailTypeTemplate->alias;
        if( $ActiveAliases{$Alias} ) {
            my $ShopMT = GetShopMailType( $Shop, $Alias );
            $ShopMT->set({'OrderDocumentPdfAttachment'=>1});
        }
        else {
            my $ShopMT = GetShopMailType( $Shop, $Alias, 1 );
            $ShopMT->set({'OrderDocumentPdfAttachment'=>0}) if defined $ShopMT;
        }
    }

    $Servlet->vars( 'FormErrorAlias', \%FormErrorAlias );
    $Servlet->form->executeFormError;

    return;
}

#==============================================================================
sub ViewOrderDocumentCustomerInformationInMail {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $aOrderDocumentTypes = [];

    foreach my $OrderDocumentType (qw( Invoice PackingSlip CreditNote )) {
        push @$aOrderDocumentTypes, {
            'Alias'  => $OrderDocumentType,
            'Attach' => $Shop->get("OrderDocumentsAttachCustomerInformationPDFTo${OrderDocumentType}Email")
        };
    }

    $Servlet->params('OrderDocumentTypes', $aOrderDocumentTypes);

    return;
}

#==============================================================================
sub SaveOrderDocumentCustomerInformationInMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my %ActiveAliases = map { $_ => 1 } $Servlet->form->value('Alias');

    _SaveEmailEventsCustomerInformationInMail($Shop, \%ActiveAliases);

    foreach my $OrderDocumentType (qw( Invoice PackingSlip CreditNote )) {
        $Shop->set({"OrderDocumentsAttachCustomerInformationPDFTo${OrderDocumentType}Email" => $ActiveAliases{$OrderDocumentType} ? 1 : 0 });
    }

    if (scalar keys %ActiveAliases) {
        foreach my $hShopLanguage (@{$Shop->get('Languages')}) {
            # create PDF document if necessary
            GetCustomerInformationPDFAttachment($Shop, $hShopLanguage->{'LanguageID'});
        }
    }

    return;
}

#==============================================================================
sub SaveEmailSettingsCustomerInformationInMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my %ActiveAliases = map { $_ => 1 } $Servlet->form->value('Alias');

    _SaveEmailEventsCustomerInformationInMail($Shop, \%ActiveAliases);
}

#==============================================================================
sub _SaveEmailEventsCustomerInformationInMail {
    my ($Shop, $hActiveAliases) = @_;
    my $aShopMailTypes = LoadRootObject()->folder('ShopMailTypeTemplates')->children;

    foreach my $MailTypeTemplate (@$aShopMailTypes) {
        my $Alias = $MailTypeTemplate->alias;
        if ($hActiveAliases->{$Alias}) {
            my $ShopMailType = GetShopMailType($Shop, $Alias);
            $ShopMailType->set({'AttachCustomerInformationPDF' => 1});
        }
        else {
            my $ShopMailType = GetShopMailType($Shop, $Alias, 1);
            $ShopMailType->set({'AttachCustomerInformationPDF' => 0}) if defined $ShopMailType;
        }
    }
}

sub UnitySaveLegalDocumentsInMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $hValues = $Servlet->form->formValues;
    my $aShopMailTypes = LoadRootObject()->folder('ShopMailTypeTemplates')->children;
    my @LegalDocs = grep { $_->alias ne 'Imprint'} @{$Shop->child('LegalPagesUnity')->children};

    foreach my $MailTypeTemplate (@$aShopMailTypes) {

        my $MailTypeAlias = $MailTypeTemplate->alias;
        my $ShopMailType = GetShopMailType($Shop, $MailTypeAlias, 1);

        next unless defined $ShopMailType;

        my @UnityAttachLegalDocumentsPDF;

        foreach my $LegalDocument (@LegalDocs) {
            push @UnityAttachLegalDocumentsPDF, $LegalDocument
                if exists $hValues->{$LegalDocument->alias . '_' . $MailTypeAlias};
        }

        $ShopMailType->set({'UnityAttachLegalDocumentsPDF' => \@UnityAttachLegalDocumentsPDF});
    }
}

#==============================================================================
sub ViewOrderDocumentPrintPreview {
    my $self = shift;
    my ($Servlet) = @_;

    my $DocType = $Servlet->params('DocType') ? $Servlet->params('DocType') : 'Invoice';
    $Servlet->params('DocType', $DocType);
    $Servlet->params('PDF', 0);

    my $LanguageID = defined $Servlet->params('AdjustedLanguageID')
                           ? $Servlet->params('AdjustedLanguageID')
                           : $Servlet->params('LanguageID');

    my $Shop = $Servlet->object;
    $Servlet->params('LanguageID', $LanguageID);
    $Servlet->params('Locale', $Shop->localeByLanguage($LanguageID));
    $Servlet->params('Language', GetCodeByLanguageID($LanguageID));

    return;
}

#==============================================================================
sub ViewOrderDocumentPdfPreview {
    my $self = shift;
    my ($Servlet) = @_;

    my $DocType = defined $Servlet->params('DocType') ? $Servlet->params('DocType') : 'Invoice';
    my $DocName = defined $Servlet->params('DocName') ? $Servlet->params('DocName') : 'Preview';
    $DocName =~ s/ /_/g;
    $DocName = ConvertPathChars($DocName);

    my $Shop = $Servlet->object;
    my $LanguageID = defined $Servlet->params('AdjustedLanguageID')
                           ? $Servlet->params('AdjustedLanguageID')
                           : $Shop->get('LanguageID');

    my $LanguageCode = GetCodeByLanguageID($LanguageID);

    my $hVars = $Shop->tleHash;
    $hVars->{'DocType'} = $DocType;
    $hVars->{'PDF'} = 1;
    $hVars->{'OSNAME'} = $Config{'osname'};

    $hVars->{'LanguageID'} = $LanguageID;
    $hVars->{'Locale'} = $Shop->localeByLanguage($LanguageID);
    $hVars->{'Language'} = $LanguageCode;

    my $PageType = LoadPageTypeByAlias('MBO-ViewOrderDocumentPdfPreview');
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$Shop);

    my $HtmlContent;
    SetLogLevel('WARN', sub {
        $HtmlContent = ProcessPage($Shop, $ObjectPageType, $LanguageCode, $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    my $PdfContent = eval{ DE_EPAGES::EPagesJ::API::PDF::PdfClient->create($Servlet->object->getSite)->transform($HtmlContent) };

    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
        $Servlet->form->executeFormError({
            'Name' => 'PdfPreview',
            'Reason' => 'PdfFileNotGenerated',
            'ViewAction' => 'MBO-ViewOrderDocumentText'
        });
    }
    my $PdfFileName = $DocName.'.pdf';

    my $Response = $Servlet->response;
    $Response->content($PdfContent);
    $Response->header( 'Content-Type' => 'application/pdf',
                       'Content-Disposition' => 'attachment; filename="' . $PdfFileName. '"',
                       'Content-Description' => 'order document pdf' );
    $Servlet->vars('ExitAfterEvent',1);
    return;
}

sub SearchLastOrders {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->BlockMonitor($Servlet);

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('CustomerOrder'), 'SimpleSearch');
    my $ClassLineItemShipping = LoadClassByAlias('LineItemShipping');
    my @LineItemShippingClassIDs = ( $ClassLineItemShipping->id, map { $_->id } @{$ClassLineItemShipping->subClasses} );
    my $AttributeShippingMethodID = $ClassLineItemShipping->attribute('ShippingMethod')->id;
    my $AttrPaymentMethodNameID   = LoadClassByAlias('PaymentMethod')->attribute('Name')->id;
    my $AttrShippingMethodNameID  = LoadClassByAlias('ShippingMethod')->attribute('Name')->id;
    my %Options = (
        'ObjectID'  => $Shop->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
            'LineItemShippingClassIDs' => \@LineItemShippingClassIDs,
            'AttributeShippingMethodID' => $AttributeShippingMethodID,
            'AttrShippingMethodNameID' => $AttrShippingMethodNameID,
            'AttrPaymentMethodNameID'  => $AttrPaymentMethodNameID,
        },
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => '1',
        'UriParameters'  => {
            'Search' => 'SimpleSearch',
        },
        'Page' => 1,
        'PageSize' => 5,
    );
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub SearchLastUpdatedOrders {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->BlockMonitor($Servlet);

    my $ClassCustomerOrder = LoadClassByAlias('CustomerOrder');
    my $SearchDefinition = GetSearchDefinition($ClassCustomerOrder, 'UpdateStatus');
    my %Options = (
        'ObjectID'  => $Shop->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
        },
        'OrderBy'    => 'LastStateDate',
        'OrderDesc'  => '1',
        'UriParameters'  => {
            'Search' => 'UpdateStatus',
        },
        'Page' => 1,
        'PageSize' => 5,
    );
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub SearchSalesVolume {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->BlockMonitor($Servlet);

    # use the shop default currency
    my $CurrencyID = $Shop->get('CurrencyID');

    # use merchant locale id
    my $LocaleID = $Servlet->user->get('Locale')->get('LocaleID');
    my $aLocales = $Shop->get('Locales');

    # test if week starts on monday or sunday
    my ($LocaleDef) = grep {$LocaleID eq $_->get('LocaleID')} @$aLocales;
    my $FirstDayOfWeek = $LocaleDef->get('FirstDayOfWeek');

    # reset today to 00:00:00
    my $Now = GetCurrentDateTime();
    my $StartToday = $Now->clone->truncate( to => 'day' );

    # reset today to weekday 1 (is sundoay or monday)
    my $StartWeek = $StartToday->clone->truncate( to => 'week' );
    $StartWeek->subtract( days => 1 ) if $FirstDayOfWeek == 0;

    # reset today to 1st day of the current month
    my $StartMonth = $StartToday->clone->truncate( to => 'month' );

    # orders from today
    my $DateFrom = $StartToday  ;
    my $DateTo = $Now ;
    my ($SalesVolumeToday,$SalesVolumeTodayCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeToday',  $SalesVolumeToday );
    $Servlet->params( 'SalesVolumeTodayCount', $SalesVolumeTodayCount );

    # orders from yesterday
    $DateFrom = SubtractDuration($StartToday->clone,'days',1);
    $DateTo = $StartToday ;
    my ($SalesVolumeYesterday,$SalesVolumeYesterdayCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeYesterday', $SalesVolumeYesterday );
    $Servlet->params( 'SalesVolumeYesterdayCount', $SalesVolumeYesterdayCount );

    # orders from  this week
    $DateFrom = $StartWeek  ;
    $DateTo = $Now ;
    my ($SalesVolumeWeek,$SalesVolumeWeekCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeWeek', $SalesVolumeWeek );
    $Servlet->params( 'SalesVolumeWeekCount', $SalesVolumeWeekCount );

    # orders from last week
    $DateFrom = SubtractDuration($StartWeek->clone,'weeks',1);
    $DateTo = $StartWeek ;
    my ($SalesVolumeLastWeek,$SalesVolumeLastWeekCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeLastWeek', $SalesVolumeLastWeek );
    $Servlet->params( 'SalesVolumeLastWeekCount', $SalesVolumeLastWeekCount );

    # orders from this month
    $DateFrom = $StartMonth  ;
    $DateTo = $Now ;
    my ($SalesVolumeMonth,$SalesVolumeMonthCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeMonth', $SalesVolumeMonth );
    $Servlet->params( 'SalesVolumeMonthCount', $SalesVolumeMonthCount );

    # orders from last month
    $DateFrom = SubtractDuration($StartMonth->clone,'months',1);
    $DateTo = $StartMonth ;
    my ($SalesVolumeLastMonth,$SalesVolumeLastMonthCount) = GetSalesVolume($Shop,$DateFrom,$DateTo,$CurrencyID);
    $Servlet->params( 'SalesVolumeLastMonth', $SalesVolumeLastMonth );
    $Servlet->params( 'SalesVolumeLastMonthCount', $SalesVolumeLastMonthCount );

    return;
}

#==============================================================================
sub UnityViewOrderDocumentText {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();

    # load the same  placeholder as for the mail type 'CustomerOrderConfirmation'
    my $CustomerOrderConfirmationMailType = $Shop->child('MailTypes')->child('CustomerOrderConfirmation');
    my @FormattedTLEs = map {
            {
                'description' => $_->get('NameOrAlias'),
                'code' => $_->get('Code')
            }
        } @{ $CustomerOrderConfirmationMailType->get('MailTypeTemplate')->get('MailTypeTemplateTLEs') };

    $Servlet->vars('PLACEHOLDER_JSON', { 'placeholder' => \@FormattedTLEs });

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    return;
}

sub UnityViewBasketText {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

sub UnityViewOrderSetup {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewOrderSetup', { 'Servlet' => $Servlet });

    $self->SUPER::View($Servlet);
}

1;
