use strict;

use Readonly;
use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (GetTestSite RunOnStore);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByGUID LoadRootObject);
use DE_EPAGES::Core::API::Error qw (ExistsError);
use DE_EPAGES::POSTPAY::t::API::TestHelper qw (
        Cleanup
        Init
        LoadPOSTPAYPayment
        LoadTestShop
        LoadPOSTPAYAnonymusUserBasket
);

use DE_EPAGES::POSTPAY::t::API::Constants qw (
        @POSITIV_MANDATORY_HASH
        @NEGATIV_MANDATORY_HASH_WRONG_STRUCTURE
        @NEGATIV_MANDATORY_HASH_WRONG_STRUCTURE2
        %KEY_VALUES_TOSTRING_HASH
);
use DE_EPAGES::POSTPAY::t::API::Format qw (heading_1);
use DE_EPAGES::POSTPAY::API::Tools qw (
        GetPOSTPAYTax
        GetDisplayName
        GetUrl
        GetPOSTPAYPaymentMethod
        GetAllPOSTPAYShops
        CheckMandatoryFields
        ParseKeyValues
        KeyValuesToString
);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # startUp
            ok( Init(), 'initialization' );
            my $Shop = LoadTestShop();
            my $Basket =LoadPOSTPAYAnonymusUserBasket;
            my $PaymentMethod = LoadPOSTPAYPayment();
            _checkGetPOSTPAYPaymentMethod($Shop);
            _checkGetAllPOSTPAYShops();
            _checkGetPOSTPAYTax();
            _checkGetDisplayName();
            _checkGetUrl($PaymentMethod);
            _checkCheckMandatoryFields($Shop,$Basket);
            _checkKeyValuesToStringParseKeyValues();
            Cleanup();
            done_testing();
        }
    );
}
sub _checkGetAllPOSTPAYShops {
    heading_1('GetAllPOSTPAYShops');
    ## TestShop need to be in atleast
    my $aPOSTPAYShopList = GetAllPOSTPAYShops();
    my @ShopAliases = map {$_->alias} @$aPOSTPAYShopList;#
    ok ('TestShopPOSTPAY' ~~ @ShopAliases,'Shop correct');
    return;
}

sub _checkGetPOSTPAYPaymentMethod {
    my ($Shop) = @_;
    heading_1('GetPOSTPAYPaymentMethod');
    my $PaymentMethod = GetPOSTPAYPaymentMethod($Shop);
    ok(defined $PaymentMethod,'PaymentMethod defined');
    return unless defined $PaymentMethod;
    is($PaymentMethod->alias,'POSTPAY','Alias Correct');
    is($PaymentMethod->get('IsInTestMode'),1,'IsInTestMode Correct');
    ok(defined $PaymentMethod->get('PaymentType'),'PaymentType set');
    return unless defined $PaymentMethod->get('PaymentType');
    is($PaymentMethod->get('PaymentType')->alias,'POSTPAY','PaymentType Correct');
    return;

}

sub _checkGetUrl{
    my ($PaymentMethod) = @_;
    heading_1('GetUrl');
    is(GetUrl($PaymentMethod),LoadRootObject()->get('POSTPAY_ServiceTestURL'),'Url correct');
    return;
}

sub _checkGetDisplayName {
    heading_1('GetDisplayName');
    is(GetDisplayName('Tobias','Test'),'Tobias Test','Display name Tobias Test correct');
    is(GetDisplayName('','Test'),'Test','Display name Test correct');
    is(GetDisplayName('Tobias',''),'Tobias','Display name Tobias correct');
    is(GetDisplayName('',''),'','Display name correct');
    return;
}

sub _checkGetPOSTPAYTax {
    heading_1('GetPOSTPAYTax');
    # TaxClass normal
    my $LineItemProductNormalTax = LoadObjectByGUID('51652550-1B1F-0803-365F-D5809AA37AA9');
    is(GetPOSTPAYTax($LineItemProductNormalTax),'Standard','TaxClass normal correct');
    # TaxClass reduced
    my $LineItemProductReducedTax = LoadObjectByGUID('51652BE1-8942-6D43-1ACD-D5809AA37AA2');
    is(GetPOSTPAYTax($LineItemProductReducedTax),'Reduced','TaxClass reduced correct');
    # TaxClass none
    my $LineItemProductNoneTax = LoadObjectByGUID('5165253B-4416-3838-9DCB-D5809AA37AAE');
    is(GetPOSTPAYTax($LineItemProductNoneTax),'Free','TaxClass none correct');
    return;
}

# i dont know if it is possible to check if warning is execeuted
sub _checkCheckMandatoryFields {
    my ($Shop,$Basket) = @_;
    heading_1('CheckMandatoryFields');

    # Case1: no error should be executed
    eval {
        CheckMandatoryFields($Shop,$Basket,@POSITIV_MANDATORY_HASH);
    };
    if (ExistsError()) {
        is(1,0,'Positiv Example ok');
    }
    else {
        is(1,1,'Positiv Example ok');
    }
    # Case2: error should be executed
    eval {
        CheckMandatoryFields($Shop,$Basket,@NEGATIV_MANDATORY_HASH_WRONG_STRUCTURE);
    };
    if (ExistsError()) {
        is(1,1,'Negativ Structure Example ok');
    }
    else {
        is(1,0,'Negativ Structure Example ok');
    }
    # Case3: error should be executed
    eval {
        CheckMandatoryFields($Shop,$Basket,@NEGATIV_MANDATORY_HASH_WRONG_STRUCTURE2);
    };
    if (ExistsError()) {
        is(1,1,'Negativ Structure 2 Example ok');
    }
    else {
        is(1,0,'Negativ Structure 2 Example ok');
    }
}

sub _checkKeyValuesToStringParseKeyValues {
    heading_1('KeyValuesToString/ParseKeyValues');
    my $String = KeyValuesToString(\%KEY_VALUES_TOSTRING_HASH);
    my $hResult = ParseKeyValues($String);
    is_deeply($hResult,\%KEY_VALUES_TOSTRING_HASH,'String/Hash transformation correct');
}

RunScript( Sub => \&Main );

1;
