#==============================================================================
# package      DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#==============================================================================
package DE_EPAGES::Payment::UI::PaymentMethod;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use feature 'switch';
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw( InsertShopPaymentLogo DeleteShopPaymentLogo GetPKeyShopPaymentLogosByShopID );
use DE_EPAGES::Payment::API::Constants qw ( CASHDISCOUNTMODE_NO_DISCOUNT CASHDISCOUNTMODE_PERCENT_DISCOUNT CASHDISCOUNTMODE_FIX_DISCOUNT
                                            CASHDISCOUNTMODE_FIX_FEE CASHDISCOUNTMODE_PERCENT_FEE );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::File  qw ( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf );

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object;

    # test if default is visible and payment method settings are complete
    my $IsActivated = $Servlet->params('IsActivated');
    $Servlet->form->executeFormError({
        'Reason'=> 'SETTINGS_INCOMPLETE',
        'Name'  => 'IsActivated',
    }) if $IsActivated and not $PaymentMethod->get('UsableAtStorefront');

    my $IsDefault = $Servlet->params('IsDefault');
    $Servlet->form->executeFormError({
        'Reason'=> 'DEACTIVATE_DEFAULT',
        'Name'  => 'IsActivated',
    }) if not $IsActivated and $IsDefault;

    # hire purchase can only be enabled if supported by payment type
    my $IsHirePurchaseEnabled = $Servlet->params('IsHirePurchaseEnabled');
    my $PaymentType = $PaymentMethod->get('PaymentType');
    if ($IsHirePurchaseEnabled and not $PaymentType->get('IsHirePurchaseSupported')) {
        $Servlet->form->executeFormError({
            'Reason'=> 'HIREPURCHASE_NOT_SUPPORTED',
        }) ;
    }

    my $Shop = $PaymentMethod->getSite();
    if ($IsDefault) {
        $_->set({'IsDefault'=>0}) foreach grep { $_->get('IsDefault') && $_->id != $PaymentMethod->id} @{$Shop->get('PaymentMethods')};
    }

    #get all Selected ShopPaymentLogos, ShopPaymentLogos, Logos
    if (defined $Servlet->form->value('DoChangeLogos') and $Servlet->form->value('DoChangeLogos') == 1) {
       $self->SavePaymentLogos($Servlet);
    }

    # store old image maybe we have to delete it
    my $OldImage = $Servlet->object()->get('Image');

    # save all attributes
    $self->SUPER::Save($Servlet);

    # remove old image if new attribute and old image not equal
    if ($OldImage and ($OldImage ne $PaymentMethod->get('Image'))) {
            my $OldPaymentImage = join '/', $PaymentMethod->get('PublicPath'), $OldImage;
            DeleteFile($OldPaymentImage) if ExistsFile($OldPaymentImage);
    }

    # set payment method invisible if payment is incomplete
    if ($PaymentMethod->get('IsActivated')) {
        my $aShopCurrencies = $Shop->get('Currencies');
        my @ActiveCurrencies = grep { $_->{'IsActivated'} == 1 } @$aShopCurrencies;
        my $hVars = {'TaxModel' => $Shop->get('TaxModel')};
        foreach my $ActiveCurrency (@ActiveCurrencies) {
            $hVars->{'CurrencyID'} = $ActiveCurrency->{'CurrencyID'};
            if (!$PaymentMethod->usableAtStorefront($hVars)) {
                $PaymentMethod->set({'IsActivated' => 0});
                $Servlet->vars('SetInvisible', 1);
                last;
            }
        }
    }

    # save min and max price limits
    my $Form = $Servlet->form->form($PaymentMethod, 'SavePriceLimits');

    my $idx = 0;
    foreach my $Entry (@{$Form->{'PriceLimits'}}) {

        # we do not save limit values because we are on the settings tab AD-623
        next if $Servlet->params('TaxClass');

        my $CurrencyID = $Entry->{'CurrencyID'};
        my $MinValue = $Entry->{'MinPriceValue'};
        my $MaxValue = $Entry->{'MaxPriceValue'};
        $MinValue = undef if defined $MinValue and $MinValue !~ /\S/;
        $MaxValue = undef if defined $MaxValue and $MaxValue !~ /\S/;


        # test if the minvalue < maxvalue
        $Servlet->form->addFormError({
            'Index'  => $idx,
            'LoopName' => 'PriceLimits',
            'Reason'=> 'MaxPriceValueToSmall',
            'Form'  => 'SavePaymentMethod',
            'Name'  => 'MaxPriceValue',
        }) if (defined $MinValue and defined $MaxValue) and ($MaxValue <= $MinValue);

        # test if maxvalue > 0
        $Servlet->form->addFormError({
            'Index'  => $idx,
            'LoopName' => 'PriceLimits',
            'Reason'=> 'MaxPriceValueNeg',
            'Form'  => 'SavePaymentMethod',
            'Name'  => 'MaxPriceValue',
        }) if defined $MaxValue and not (0 < $MaxValue) ;

        # test if minvalue > 0
        $Servlet->form->addFormError({
            'Index'  => $idx,
            'LoopName' => 'PriceLimits',
            'Reason'=> 'MinPriceValueNeg',
            'Form'  => 'SavePaymentMethod',
            'Name'  => 'MinPriceValue',
        }) if defined $MinValue and not (0 < $MinValue);

        # set parameters if no error was added
        if(not $Servlet->form->existsFormError) {
            $PaymentMethod->setMinPriceValue( $CurrencyID, $MinValue );
            $PaymentMethod->setMaxPriceValue( $CurrencyID, $MaxValue );
        }
        $idx ++;
    }

    # save cash discounts only of we have correct form fields
    # we use the save function for serveral mbo tabs
    if (not defined $Servlet->params('CashDiscountMode')) {
        $Servlet->form->executeFormError;
        return;
    }

    my $DiscountForm = $Servlet->form->form($PaymentMethod, 'SaveCashDiscounts');
    my $CashDiscountMode = $DiscountForm->{'CashDiscountMode'} ;

    # save values
    given($CashDiscountMode) {
        when( [CASHDISCOUNTMODE_FIX_DISCOUNT,CASHDISCOUNTMODE_FIX_FEE]  ) {
            # save values for fix fee and discount
            $PaymentMethod->setPaymentMethodDiscount( $_->{'CurrencyID'}, $_->{'PriceValue'} ) foreach (@{$DiscountForm->{'DiscountOrFeePrices'}});
            # delete CashDiscount
            $PaymentMethod->set({'CashDiscount' => 0});
        }
        when( [CASHDISCOUNTMODE_PERCENT_DISCOUNT, CASHDISCOUNTMODE_PERCENT_FEE]  ) {
            # delete fix discount settings
            $PaymentMethod->setPaymentMethodDiscount( $_->{'CurrencyID'}, undef ) foreach (@{$DiscountForm->{'DiscountOrFeePrices'}});
        }
        when(CASHDISCOUNTMODE_NO_DISCOUNT) {
            # delete CashDiscount
            $PaymentMethod->set({'CashDiscount' => 0});
            # delete fix discount settings
            $PaymentMethod->setPaymentMethodDiscount( $_->{'CurrencyID'}, undef ) foreach (@{$DiscountForm->{'DiscountOrFeePrices'}});
        }
    }

    $Servlet->form->executeFormError;

    return;
}

sub AddCustomerGroup {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object;

    return unless $Servlet->params('AddCustomerGroupID');
    my $CustomerGroup = $PaymentMethod->getSite()->loadObject( $Servlet->params('AddCustomerGroupID'), 'CustomerGroup' );

    my $Form = $Servlet->form;
    eval {
        $PaymentMethod->addCustomerGroup($CustomerGroup);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'CustomerGroupAssignedToAnotherPaymentMethodAtThisTimeFrame';
        $Form->mapFormError({'Name' => 'AddCustomerGroupID'}, $Error);
    }
    $Form->executeFormError;
    return;
}

sub RemoveCustomerGroups {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod   = $Servlet->object;
    my @ListObjectIDs   = $Servlet->form->value('ListObjectID');
    my $aCustomerGroups = $PaymentMethod->getSite->loadObjects(\@ListObjectIDs, 'CustomerGroup');

    foreach my $CustomerGroup (@$aCustomerGroups) {
        $PaymentMethod->removeCustomerGroup($CustomerGroup);
    }
    return;
}

sub SavePaymentLogos {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object;
    my $Shop = $PaymentMethod->getSite();

    my $PaymentType = $PaymentMethod->get('PaymentType');

    #get all Selected ShopPaymentLogos, ShopPaymentLogos, Logos
    my @SelectedShopPaymentLogos = $Servlet->form->value('LogoID');
    my $Logos = $PaymentType->folder('Logos')->children;

    # get all activated ShopPaymentLogo
    my @ShopPaymentLogoIDs;
    foreach my $ShopPaymentLogo (@{GetPKeyShopPaymentLogosByShopID ( $Shop->id )}) {
        my $ShopPaymentLogoID = $ShopPaymentLogo->{'PaymentTypeLogoID'};
        if (grep {$ShopPaymentLogoID == $_->id} @$Logos) {
            push @ShopPaymentLogoIDs, $ShopPaymentLogoID;
        }
    }

    #insert new Logos
    InsertShopPaymentLogo({
        ShopID            => $Shop->id,
        PaymentTypeLogoID => $_
    }) foreach (grep { not ElementOf($_, @ShopPaymentLogoIDs) } @SelectedShopPaymentLogos );

    # delete not needed Logos
    DeleteShopPaymentLogo( $Shop->id, $_ ) foreach (grep {not ElementOf($_, @SelectedShopPaymentLogos) } @ShopPaymentLogoIDs);
}

sub UnityView {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

1;
