#========================================================================================
# package      DE_EPAGES::Presentation::API::Object::Pager
# state        public
# base         DE_EPAGES::Core::API::Object::BaseObject
#----------------------------------------------------------------------------------------
# description  Base class for pager objects.
#               Pagers are used for navigation in long lists.
#               Sub classes must override at least the 2 abstract methods
#               <function countItems> and <function items>.
#               <br/> see <package DE_EPAGES::Presentation::API::Object::PagerChildren>
#               for an example implementation.
# example      my $Pager = PagerClass->new({ PageSize => 10 });
#               $Pager->statistic;
#               my $Count = $Pager->countItems;
#               my $aItems = $Pager->items( 1 );
#========================================================================================
package DE_EPAGES::Presentation::API::Object::Pager;
use base qw ( DE_EPAGES::Core::API::Object::BaseObject );

use strict;
use URI::Escape qw( uri_escape_utf8 );
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger );
use DE_EPAGES::Core::API::Url qw ( UrlEncodeValue );
use DE_EPAGES::Object::API::Factory qw ( SortObjects LoadRootObject LoadObject);

sub ErrorDefinition {
    return {
        'NoObjectID' => 'need object id for new url',
        'WrongURLParameter' => 'url parameter "#Parameter" has wrong value "#Value"',
        'IMPLEMENT_countItems' => 'please implement function countItems',
        'IMPLEMENT_items' => 'please implement function items',
    };
}

my %DEFAULTS = (
    'Page'          =>  1,
    'PageSize'      => 10,
    'PagerSize'     => 10,
    'OrderBy'       => '',
    'OrderDesc'     => 0,
    'ObjectID'      => undef,
    'ObjectPath'    => undef,
    'ViewAction'    => 'View',
    'AltLanguageID' => '',
);

use constant PAGESIZE_HARD_LIMIT => 10_000;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Pager = DE_EPAGES::Presentation::API::Object::Pager->new($hInfo);
#----------------------------------------------------------------------------------------
# description  Creates a new pager object.
#----------------------------------------------------------------------------------------
# input        $hInfo | parameters with keys
#                       <ul>
#                         <li>Servlet - servlet object</li>
#                         <li>ViewAction - view action name (default=View)</li>
#                         <li>ObjectID - object id for ViewAction (required unless Servlet is given)</li>
#                         <li>Page - page number (default=1)</li>
#                         <li>PageSize - max. number of items per page (default=10)</li>
#                         <li>PagerSize - max. number of pages in pager area (default=10)</li>
#                         <li>OrderBy - sort attribute name (default='')</li>
#                         <li>OrderDesc - sort direction (0=ascending, 1=descending, default=0)</li>
#                         <li>UriParameters - additinal parameter that are included in
#                           the links, for example to next and previous page. Provide params as hash reference.
#                           Values may contain array references for multi value parameter.
#                           - ref-hash.string</li>
#                       </ul> | ref.hash
#----------------------------------------------------------------------------------------
# example
#
#         my $Pager = MyPager->new( {
#                Servlet => $Servlet,
#                Page => 5,
#                PageSize => 4,
#                PagerSize => 6,
#                OrderBy => 'X',
#                OrderDesc => 1,
#                # Example parameter for search: Look for Adidas in Shoes or Jackets, which are red or brown...
#                UriParameters => {
#                    SearchString => 'Adidas',
#                    IsNew => 1,
#                    Category => ['Shoes', 'Jackets'],
#                    Color => ['Brown', 'Red'],
#                    PriceMin => 31.22,
#                    PirceMax => 207.77,
#                }
#            } );
#----------------------------------------------------------------------------------------
# return       $Pager | pager |object
#========================================================================================
sub new  {
    my $class = shift;
    my ($hInfo) = @_;

    # default values that will not included in the URL
    my %Defaults = %DEFAULTS;
    foreach my $key (keys %DEFAULTS) {
        next unless defined $hInfo->{$key};
        $Defaults{$key} = $hInfo->{$key};
    }

    # options = %DEFAULTS + %$hInfo + $Servlet->form
    my %Options = ();
    foreach my $key (keys %$hInfo) {
        next if $key eq 'Servlet';
        next unless defined $hInfo->{$key};
        $Options{$key} = $hInfo->{$key};
    }

    if (defined $hInfo->{'Servlet'}) {
        my $Servlet = $hInfo->{'Servlet'};
        my $Object = $Servlet->object;
        foreach my $Parameter (keys %DEFAULTS) {
            my $Value = $Servlet->params($Parameter);
            $Options{$Parameter} = $Value if defined $Value;
        }
        $Options{'ObjectID'} = $Object->id;

        # EPG-5949 Page parameter not integer
        if ( $Options{'Page'} ) {
            eval { TestInteger('Page', $Options{'Page'}) };
            Error('WrongURLParameter', { Parameter => 'Page', Value => $Options{'Page'} }, GetError()) if ExistsError();
        }

        # EPG-12569 Check if value is numerical.
        if(defined $Options{'PageSize'} and $Options{'PageSize'} =~ /^\d+$/) {
            # EPG-12280 Limit pager to prevent possible DOS attacks.
            $Options{'PageSize'} = LoadRootObject()->get('MaxPageSize') if $Options{'PageSize'} > PAGESIZE_HARD_LIMIT;
        }
        else {
            # values stored as user attributes
            # EPG-10712 use user's PageSize property of present
            my $User = $Servlet->user;
            my $UserPagerEntries = defined $User ? $User->get('PagerEntries') : undef;
            $Options{'PageSize'} = $UserPagerEntries // $Object->getSite()->get('DefaultPageSize');
        }
    }

    # default values that will not included in the URL
    $Options{$_} = $DEFAULTS{$_} foreach grep { ! defined $Options{$_} } keys %DEFAULTS;
    $Options{'PagerSize'} = $DEFAULTS{'PagerSize'} if $Options{'PagerSize'} < 3;
    $Options{'PageSize'}  = $DEFAULTS{'PageSize'}  if $Options{'PageSize'}  < 1;

    Error('NoObjectID') unless defined $Options{'ObjectID'};

    $Options{'StatisticFinished'} = 0;
    $Options{'UriParameters'} = {} unless defined $Options{'UriParameters'};

    my $self = DE_EPAGES::Core::API::Object::BaseObject->new(%Options);
    $self->{Defaults} = \%Defaults;

    return bless $self, $class;
}

sub defaults {
    return shift->{Defaults};
}

#========================================================================================
# function     statistic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Pager->statistic;
#----------------------------------------------------------------------------------------
# description  Calls <function countPages> and <function countItems> to collect required
#               information for all pages.
#========================================================================================
sub statistic {
    my $self = shift;

    # count items
    my $CountOfItems = $self->countItems;
    $self->set('CountOfItems', $CountOfItems);

    # count pages
    my $CountOfPages = $self->countPages($CountOfItems);
    $self->set('CountOfPages', $CountOfPages);

    # correct page number
    my $Page = $self->get('Page');
    if (not defined $Page or $Page eq '' or $Page < 1) {
        $self->set('Page', 1);
    } elsif ( $CountOfPages < $Page) {
        $self->set('Page', $CountOfPages);
    } # else ok
    $self->set('StatisticFinished', 1);
    return;
}

#========================================================================================
# function     countPages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CountPages = $Pager->countPages($CountOfItems);
#----------------------------------------------------------------------------------------
# description  Calculates the number of pages for a given number of items.
#----------------------------------------------------------------------------------------
# input        $CountOfItems | total number of items | integer
# return       $CountPages | total number of pages | integer
#========================================================================================
sub countPages {
    my $self = shift;
    my ($CountOfItems) = @_;
    TestInteger( 'CountOfItems', $CountOfItems );

    my $PageSize = $self->get('PageSize');
    my $cPages = int ($CountOfItems / $PageSize);
    $cPages++ if $CountOfItems % $PageSize;
    return $cPages;
}

#========================================================================================
# function     firstItemOfPage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Index = $Pager->firstItemOfPage($Page);
#----------------------------------------------------------------------------------------
# description  Returns the 0-based index of the first item on a page.
#----------------------------------------------------------------------------------------
# input        $Page | page index | integer
# return       $Index | index of the first item | integer
#========================================================================================
sub firstItemOfPage {
    my $self = shift;
    my ($Page) = @_;
    TestInteger( 'Page', $Page, 1 );

    my $PageSize = $self->get('PageSize');
    return ($Page - 1) * $PageSize;
}

#========================================================================================
# function     lastItemOfPage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Index = $Pager->lastItemOfPage($Page);
#----------------------------------------------------------------------------------------
# description  Returns the 0-based index of the last item on a page.
#----------------------------------------------------------------------------------------
# input        $Page | page index | integer
# return       $Index | index of the last item | integer
#========================================================================================
sub lastItemOfPage {
    my $self = shift;
    my ($Page) = @_;
    TestInteger( 'Page', $Page );

    my $PageSize     = $self->get('PageSize');
    my $CountOfItems = $self->get('CountOfItems');
    my $End = $self->firstItemOfPage($Page) + $PageSize - 1;
    $End = $CountOfItems - 1 if $CountOfItems <= $End;
    return $End;
}

# Purpose ? Maybe used for overriding in subclasses
sub _uri {
    my $self = shift;
    return '';
}

sub _parameterNoSort {
    my $self = shift;
    my $hParameter = $self->get('UriParameters');
    my @Param = ();
    push @Param, 'ViewAction='.$self->get('ViewAction');
    push @Param, 'ObjectID=' . $self->get('ObjectID');
    push @Param, 'PageSize=' . $self->get('PageSize');
    push @Param, $_ foreach $self->_nonStdParameter(['PagerSize', 'AltLanguageID']);
    push @Param, $_ foreach $self->_uriParameter( $hParameter );

    return '?' . join( '&', @Param);
}

# return list of parameter, which are not standard settings
sub _nonStdParameter {
    my $self = shift;
    my ( $aParamList ) = @_;
    return map { "$_=". $self->get($_) }
           grep { $self->get($_) != $self->defaults->{$_} }
           @$aParamList
}

sub _uriParameter($;$) {
    my $self = shift;
    my ( $hParameter, $aBlockList ) = @_;
    $aBlockList = [] unless defined $aBlockList;
    return map { _ExpandParameter( $_, $hParameter->{$_} ) }
           grep { $_ !~ $aBlockList }
           keys %$hParameter;
}

# expand array references to multiple parameters, e.g.('color=brown', 'color=yellow')
# Standard parameter PageSize, OrderDesc, PageSize or PagerSize are not expanded
sub _ExpandParameter {
    my( $Key, $Value ) = @_;
    return unless defined $Value;
    return if $Value eq '';
    my @Values = ();

    if ( ref $Value eq 'ARRAY'){
        if ($Key ~~ [ qw ( ObjectID ObjectPath OrderDesc PageSize PagerSize ) ] ) {
            push @Values, $Values[0];
        } else {
            @Values = @$Value;
        }
    } else {
        push @Values, $Value;
    }

    my @list =
        map { "$Key=". uri_escape_utf8( $_ ) }
        grep { defined $_ && $_ ne '' }
        @Values;
        return @list;
}

sub _parameterPage {
    my $self = shift;
    my ($Page) = @_;
    my $OrderBy   = $self->orderBy;
    my $OrderDesc = $self->orderDesc;

    return join ('',
        $self->_parameterNoSort,
        ($Page == 1 ? '' : '&Page='.$Page),
        (!defined $OrderBy || $OrderBy eq $self->defaults->{'OrderBy'} ? '' : '&OrderBy='.$OrderBy),
        (!defined $OrderDesc || $OrderDesc == $self->defaults->{'OrderDesc'}  ? '' : '&OrderDesc='.$OrderDesc)
    );
}

sub _newButton {
    my $self = shift;
    my ($Page, $Text) = @_;

    return {
       'URLPage' => $self->_uri . $self->_parameterPage($Page),
       'Page' => $Page,
       'PageText' => $Text,
    };
}

#========================================================================================
# function     page
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPagerVars = $Pager->page
# example      print $Pager->page->{'Pager'}->{'LastPage'};
# example      # create a Pager from a UI (ViewAction) function
#               sub ViewPagedChildren {
#                   my $self = shift;
#                   my $Servlet = shift;
#
#                   my $Pager = DE_EPAGES::Presentation::API::Object::PagerChildren->new({
#                       'Servlet' => $Servlet,
#                       'Object' => $Servlet->object
#                   });
#                   return $Pager->page;
#               }
#----------------------------------------------------------------------------------------
# description  Returns the values that are required to display the current page.
#               Runs $Pager-><function statistic> if statistic wasnt run before.
#               Calls $Pager-><function items>($Page) for current page.
#----------------------------------------------------------------------------------------
# return       $hPagerVars | pager variables that can be used inside a template.
#                       <ul>
#                         <li>Pager - pager info (hash ref)
#                           <ul>
#                             <li>Items - list of items on this page - ref.array.hash</li>
#                             <li>FirstPage - 1 - integer</li>
#                             <li>LastPage- total number of pages - integer</li>
#                             <li>Page - current page number - integer</li>
#                             <li>Pages - array of pages - ref.array.hash
#                               <ul>
#                                 <li>Page - page number - integer</li>
#                                 <li>URLPage - page link - string</li>
#                               </ul>
#                             </li>
#                             <li>NextPage - next page number -integer</li>
#                             <li>PreviousPage - previous page number -integer</li>
#                             <li>ItemOffset - offset for items on this page - integer</li>
#                             <li>URL - current page link - string</li>
#                           </ul>
#                         </li>
#                       </ul> | ref.hash
#========================================================================================
sub page {
    my $self = shift;

    $self->statistic() unless $self->get('StatisticFinished');
    $self->set( { 'URLNoPageSize' => $self->_uri . $self->_makeURLParameters(['PageSize']) } );
    $self->set( { 'URLNoPage'     => $self->_uri . $self->_makeURLParameters(['Page', 'PageSize']) } );
    $self->set( { 'URLNoAltLanguageID' => $self->_uri . $self->_makeURLParameters(['AltLanguageID']) } );

    # uri has no page
    my $Uri = $self->_uri;

    # build page rider
    my $PageCount  = $self->get('CountOfPages');
    my $Page = $self->get('Page') || 1;
    my $PagerSize  = $self->get('PagerSize');
    my $StartPage  = 1;
    my $LastPage   = $PageCount;
    my $EndPage    = $PageCount;

    # define Pager
    if ($PagerSize < $PageCount) {
        $StartPage = $Page - int ($PagerSize / 2);
        $StartPage = 1 if $StartPage < 1;
        $EndPage = $StartPage + $PagerSize - 1;
        if ($PageCount < $EndPage) {
            $EndPage = $PageCount;
            $StartPage = $EndPage - $PagerSize + 1;
            $StartPage = 1 if $StartPage < 1;
        }
    }

    my @Pages = map {
        {
            'Page'    => $_,
            'URLPage' => $Uri . $self->_parameterPage($_),
        }
    } ($StartPage .. $EndPage);


#New Pager##################################################################
    my @PagesNew;
    my $NewPagerSize = 7;
    if ($PageCount > $NewPagerSize) {
        $StartPage = $Page - int ($NewPagerSize / 2);
        $StartPage = 1 if $StartPage < 1;
        $EndPage = $StartPage + $NewPagerSize - 1;
        if ($PageCount < $EndPage) {
            $EndPage = $PageCount;
            $StartPage = $EndPage - $NewPagerSize + 1;
            $StartPage = 1 if $StartPage < 1;
        }

        #First Element always Page 1
        push @PagesNew, $self->_newButton( 1, '1' );

        #pushing dot element
        if($Page >= $NewPagerSize - 2){
            $StartPage += 2;
            push @PagesNew, $self->_newButton( $StartPage-1, '...' );
        }
        else {
            $StartPage += 1;
        }

        #Defining last dot element
        if($Page <= $PageCount-4){
            $EndPage -= 2;
        }
        else{
            $EndPage -= 1;
        }
        #Calculating middle elements
        push @PagesNew, $self->_newButton( $_, $_ ) foreach( $StartPage .. $EndPage );

        #pushing last dot element
        if($Page <= $PageCount-4){
            push @PagesNew, $self->_newButton( $EndPage+1, '...' );
        }

        #last Element alwaly last Site
        push @PagesNew, $self->_newButton( $PageCount, $PageCount );
    }
    else {
        push @PagesNew, $self->_newButton( $_, $_ ) foreach( 1 .. $PageCount );
    }
#New Pager##################################################################

    # collect pager info
    my %PagerInfo = (
        'FirstPage'    => 1,
        'Items'        => $self->get('CountOfItems') ? $self->items($Page) : [],
        'LastPage'     => $LastPage,
        'NextPage'     => $Page < $LastPage ? $Page + 1 : $LastPage,
        'Page'         => $Page,
        'Pages'        => \@Pages,
        'PagesNew'     => \@PagesNew,
        'PreviousPage' => $Page < 2 ? 1 : $Page - 1,
        'ItemOffset'   => $self->firstItemOfPage($Page),
        'URL'          => $Uri . $self->_parameterNoSort,
    );

    foreach (qw (FirstPage PreviousPage Page NextPage LastPage)){
        $PagerInfo{'URL'.$_} = $Uri . $self->_parameterPage($PagerInfo{$_})
    };

    $self->set(\%PagerInfo);

    return { 'Pager'=> $self->tleHash };
}

#========================================================================================
# function     orderBy
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrderBy = $Pager->orderBy;
#----------------------------------------------------------------------------------------
# description  Returns the attribute by which the results are sorted.
#----------------------------------------------------------------------------------------
# return       $OrderBy | name of the attribute for sorting of results | string
#========================================================================================
sub orderBy {
    return shift->get('OrderBy');
}

#========================================================================================
# function     orderDesc
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrderDesc = $Pager->orderDesc;
#----------------------------------------------------------------------------------------
# description  Returns the sort direction (ascending or descending).
#----------------------------------------------------------------------------------------
# return       $OrderDesc | true if the results are sorted in descending direction
#                   | string
#========================================================================================
sub orderDesc {
    return shift->get('OrderDesc');
}


#========================================================================================
# function     sortObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSortedObjects = $Pager->sortObjects($aObjects);
#----------------------------------------------------------------------------------------
# description  Sorts the objects according to the 'OrderBy' and 'OrderDesc' attributes,
#               which are set by corresponding input fields.
#----------------------------------------------------------------------------------------
# input        $aObjects | list of objects | ref.array.object
# return       $aSortedObjects | sorted list of objects | ref.array.object
#========================================================================================
sub sortObjects {
    my $self = shift;
    my ($aObjects) = @_;

    return SortObjects($aObjects, $self->orderBy, $self->orderDesc);
}

#========================================================================================
# function     countItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Count = $Pager->countItems;
#----------------------------------------------------------------------------------------
# description  Returns the total number of items to be paged.
#----------------------------------------------------------------------------------------
# return       $Count | number of items | integer
#========================================================================================
sub countItems {
    Error('IMPLEMENT_countItems');
    return;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aItems = $Pager->items($Page);
#----------------------------------------------------------------------------------------
# description  Returns all objects of a page.
#----------------------------------------------------------------------------------------
# input        $Page | current page number | integer
# return       $aItems | list of objects | ref.array.object
#========================================================================================
sub items {
    my $self = shift;
    my ($Page) = @_;

    Error('IMPLEMENT_items');
    return;
}

#========================================================================================
# function     allItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aItems = $Pager->allItems;
#----------------------------------------------------------------------------------------
# description  Returns all objects of all pages.
#----------------------------------------------------------------------------------------
# return       $aItems | list of objects | ref.array.object
#========================================================================================
sub allItems {
    my $self = shift;

    return [ map { @{ $self->items($_) } } ( 1 .. $self->get('CountOfPages') ) ];
}

#========================================================================================
# function     pageOfItemByObjectID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Page = $Pager->pageOfItemByObjectID($ObjectID);
#----------------------------------------------------------------------------------------
# description  Returns the page where object with given objectid is positionate.
#               Can only used if pager contains real objects.
#----------------------------------------------------------------------------------------
# input        $ObjectID | id of object to search | integer
# return       $Page | page of pager where object is find | integer
#========================================================================================
sub pageOfItemByObjectID {
    my $self = shift;
    my ($ObjectID) = @_;

    #execute statistic before call count items
    $self->statistic() unless $self->get('StatisticFinished');

    #get informations about pager sizes
    my $CountItems = $self->countItems;
    my $CountPages = $self->countPages( $CountItems );

    #search object in each page
    for( my $Page = 1; $Page <= $CountPages; $Page++ ) {
        my $aItems = $self->items($Page);
        foreach my $Item ( @$aItems ) {
            if( $Item->id eq $ObjectID ) {
                return $Page;
            }
        }
    }
    return undef;
}

#========================================================================================
# function     pageOfItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Page = $Pager->pageOfItem(sub{ ... });
#----------------------------------------------------------------------------------------
# example      my $ObjectID = 0815;
#               $Page = $Pager->pageOfItem(sub{
#                   my ($Object) = @_;
#                   return $Object->id eq $ObjectID ? 1 : 0;
#               });
#----------------------------------------------------------------------------------------
# description  Returns the page where object with given objectid is positionate.
#               Can used if pager doesnt contain real objects.
#----------------------------------------------------------------------------------------
# input        $FindFunction | function that makes a comparison on pager items | integer
# return       $Page | page of pager where object is find | integer
#========================================================================================
sub pageOfItem {
    my $self = shift;
    my ($FindFunction) = @_;

    my $CountItems = $self->countItems;
    my $CountPages = $self->countPages( $CountItems );
    for( my $PageNo = 1; $PageNo <= $CountPages; $PageNo++ ) {
        my $aItems = $self->items( $PageNo );
        foreach my $Item ( @$aItems ) {
            if( $FindFunction->( $Item ) ) {
                return $PageNo;
            }
        }
    }

    return undef;
}

sub _makeURLParameters {
    my $self         = shift;
    my ($aBlockList) = @_;

    $aBlockList //= [];
    my $hParameter   = $self->get('UriParameters');
    my $Page         = $self->get('Page') || 1;
    my $OrderBy      = $self->orderBy;
    my $OrderDesc    = $self->orderDesc;
    my $AltLanguageID = $self->get('AltLanguageID');

    my @List = ();
    push @List, 'ViewAction=' . $self->get('ViewAction');
    push @List, $self->_objectParam();
    push @List, 'Page='       . $Page      if $Page && !('Page' ~~ $aBlockList);
    push @List, 'OrderBy='    . $OrderBy   if defined $OrderBy   && $OrderBy   ne $self->defaults->{'OrderBy'}   && !('OrderBy'  ~~ $aBlockList);
    push @List, 'OrderDesc='  . $OrderDesc if defined $OrderDesc && $OrderDesc != $self->defaults->{'OrderDesc'} && !('OrderDesc' ~~ $aBlockList);
    push @List, 'AltLanguageID=' . $AltLanguageID if $AltLanguageID && !('AltLanguageID' ~~ $aBlockList);
    push @List, $_ foreach $self->_uriParameter( $hParameter, $aBlockList );

    return '?' . join('&',@List);
}

# Get ObjectID=0815 or even better ObjectPath=/Shops/DemoShop/Categories/Jackets
sub _objectParam {
    my $self = shift;
    my $Path = $self->get('ObjectPath');

    if ( !defined $Path ) {
        my $ID = $self->get('ObjectID');
        die 'no ObjectID nor ObjectPath given, see EPG-4334' unless defined $ID;

        $Path = LoadObject($ID)->get('Path');
    }

    return 'ObjectPath='.UrlEncodeValue($Path);
}

1;
