use strict;
use threads;

use DE_EPAGES::WebInterface::API::Crawler::Crawler;
use DE_EPAGES::WebInterface::API::Crawler::Requester;
use DE_EPAGES::Presentation::API::ContentAnalyzer qw( FindLinkedPages );

use Test::More;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use HTML::Lint;

sub Main {
    my ($StoreFrontURL, $Verbose, $Help, $Warnings, $Fluff);

    GetOptions(
        'start=s'  => \$StoreFrontURL,
        'verbose'  => \$Verbose,
        'help' =>  \$Help,
        'warnings' => \$Warnings,
        'fluff' => \$Fluff
    );

    usage() if $Help;
    usage() unless $StoreFrontURL;



    my @ValidationMode = (HTML::Lint::Error::STRUCTURE);
    push @ValidationMode, HTML::Lint::Error::HELPER if $Warnings;
    push @ValidationMode, HTML::Lint::Error::FLUFF if $Fluff;

    my $Requester = DE_EPAGES::WebInterface::API::Crawler::Requester->new();

    # Find all reachable storefront pages -> change to imported demoshop
    print "Searching for pages reachable from $StoreFrontURL\n";
    my $SFCrawler = DE_EPAGES::WebInterface::API::Crawler::Crawler->new( $Requester, $StoreFrontURL, \&FindLinkedPages );
    $SFCrawler->start();

    # Check all found pages for html errors
    foreach my $Page ( keys %{$SFCrawler->{'pages'}} ) {
        next unless $SFCrawler->{'pages'}->{$Page}->{'contenttype'} =~ /^text\/html/;

        print "Checking: $Page\n" if $Verbose;
        my $lint = HTML::Lint->new;
        $lint->only_types( @ValidationMode );
        $lint->parse( $SFCrawler->{'pages'}->{$Page}->{'content'} );
        my $ErrorCount = 0;
        foreach my $error ( $lint->errors ) {
            my $ErrorCode = $error->errcode();
            next if $ErrorCode eq 'text-use-entity';
            # ignore new errcodes unknown to HTML::Lint 2.20 (AD-9091)
            next if $ErrorCode eq 'attr-use-entity';
            next if $ErrorCode eq 'attr-unknown-entity';
            $ErrorCount++;
            print $error->errcode(), ':',  $error->as_string(), "\n";
        }
        ok( $SFCrawler->{'pages'}->{$Page}->{'statuscode'} < 400, "Check status code (<400): $SFCrawler->{'pages'}->{$Page}->{'statuscode'}");
        ok( $SFCrawler->{'pages'}->{$Page}->{'content'} ne '' && $ErrorCount == 0, "Validate $Page");
        print "Contentsize: $SFCrawler->{'pages'}->{$Page}->{'contentsize'} kB  Response time: $SFCrawler->{'pages'}->{$Page}->{'responsetime'} s\n" if $Verbose;
    }
    done_testing();
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -start          url of the storefront servlet (required)

flags:
    -help           show the command line options
    -verbose        print verbose information about content size and response time
    -warnings       use HTML::Lint validation mode helper
    -fluff          use HTML::Lint validation mode fluff

Example:
  perl $0 -start=http://qa-vm-lin64-1.epages.de/epages/mpfeiffer.sf
END_USAGE
    exit 2;
}

RunScript(Sub=>\&Main, Debug=>1);
