#==============================================================================
# package      DE_EPAGES::Product::UI::ProductAttribute
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::ProductAttribute;
use base DE_EPAGES::Presentation::UI::Attribute;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf IndexOf);
use DE_EPAGES::Core::API::Unicode qw (Unicode2Alias);
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Table::SelectPreDefAttribute qw ( GetPKeySelectPreDefAttributesByPreDefAttributeID GetPKeySelectPreDefAttributesByAttributeID);
use DE_EPAGES::Product::API::ProductAttribute qw (CheckProductAttributeAlias);
use DE_EPAGES::Product::API::Table::SelectedVariation qw ( GetPKeySelectedVariationsByPreDefAttributeID GetPKeySelectedVariationsByAttributeID);

#========================================================================================
# function     checkAlias
# state        private
#----------------------------------------------------------------------------------------
# description  Checks if product attribute alias is valid. Adds form errors if
#               product type alias is taken.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Alias | product attribute alias | string
# input        $Attribute | product attribute (optional) | object
# input        $ProductType | product type | object
#========================================================================================
sub checkAlias {
    my $self = shift;
    my ( $Servlet, $Alias, $Attribute, $ProductType ) = @_;

    eval {
        CheckProductAttributeAlias($ProductType, $Alias, $Attribute);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless(ElementOf($Error->code, qw (AliasReserved AliasWithUmlauts ProductAttributeExistsInOtherProductClass ProductAttributeExists)));
        $Servlet->form->mapFormError({'Name' => 'Alias'}, $Error);
    }

    return;
}

#==============================================================================
# Event handlers
#==============================================================================
sub DeletePreDefAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductAttribute = $Servlet->object;
    my $Shop = $ProductAttribute->getSite();

    # test if variations exists
    my $Form = $Servlet->form;
    my $hValues = $Form->form($ProductAttribute, 'ListedObjects');
    my @PreDefAttributeIDs = map { $_->{'ListObjectID'} } @{$hValues->{'ListObjectIDs'}};
    my $rAllPreDefAttributes = $Servlet->form->{'Values'}->{'PreDefAttributeID'};

    my @DeleteObjects;
    foreach my $Index ( 0 .. scalar(@PreDefAttributeIDs)-1 ) {
        if  ( scalar @{GetPKeySelectedVariationsByPreDefAttributeID($PreDefAttributeIDs[$Index])} or scalar @{GetPKeySelectPreDefAttributesByPreDefAttributeID($PreDefAttributeIDs[$Index])} ) {
           $Form->addFormError( {
               Reason => 'ProductsWithThisVariationValueExists',
               Index  => IndexOf($PreDefAttributeIDs[$Index], @$rAllPreDefAttributes),
               LoopName => 'ListObjectIDs',
               Form => 'ListedObjects',
               Name => 'ListObjectID',
           } );
        }
        push @DeleteObjects, $Shop->loadObject($PreDefAttributeIDs[$Index], 'PreDefAttribute');
    }
    $Servlet->vars('DeleteObjects',\@DeleteObjects) if $Form->existsFormError;
    $Form->executeFormError;

    $self->SUPER::DeletePreDefAttributes( $Servlet );
    return;
}

sub NewObject {
    my $self = shift;
    my ($Servlet, $ProductTypeFolder, $ProductAttributeClass, $hValues) = @_;

    my $Attribute;
    eval {
        $Attribute = $self->SUPER::NewObject( $Servlet, $ProductTypeFolder, $ProductAttributeClass, $hValues);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code =~ /VariationFeatureLimitExceeded/){
            Warning($Error);
            $Servlet->form->executeFormError({
                'Reason'=> 'VariationFeatureMaxValueReached',
                'Name'  => 'Type',
            });
        }elsif($Error->code =~ /FeatureLimitExceeded/){
            Warning($Error);
            $Servlet->form->executeFormError({
                'Reason'=> 'FeatureMaxValueReached',
                'Name'  => 'Type',
            });
        } else {
            Error($Error);
        }
    }
    $ProductTypeFolder->renumberChildren;
    return $Attribute;
}

sub Delete {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    my $Form = $Servlet->form;
    if  ( scalar @{GetPKeySelectedVariationsByAttributeID($Object->id)} and scalar @{GetPKeySelectPreDefAttributesByAttributeID($Object->id)} ) {
       $Form->executeFormError( {
           Reason => 'ProductsWithThisVariationAttributeExist',
           Form =>   'Save',
           Name =>   'Delete',
       } );
    }

    my $AttributeClass = $Object->get('AttributeClass');
    $self->SUPER::Delete( $Servlet );
    return if $Servlet->params('DELETE');

    $AttributeClass->folder('Attributes')->renumberChildren;
    return $AttributeClass;
}

#========================================================================================
# function     ViewPagedChildren
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hVars = $Handler->ViewPagedChildren( $Servlet );
#----------------------------------------------------------------------------------------
# description  Creates a pager for PreDefAttributes of the ProductAtrribute.
#               The default sort order is by Position, ascending.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub ViewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object->existsChild('PreDefAttributes') ? $Servlet->object->child('PreDefAttributes') : $Servlet->object;
    my $Pager = DE_EPAGES::Presentation::API::Object::PagerChildren->new({'Servlet' => $Servlet, 'Object' => $Object});
    return $Pager->page;
}


sub SavePreDefAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::SavePreDefAttributes($Servlet);

    # fetch objectids of the predefattribute folder objects and write them to servlet
    my $Attribute = $Servlet->object;
    my $Type = $Attribute->get('Type');
    if ( $Type  =~ /^PreDef/ )  {
        my $hPreDefAttributeChildren;
        foreach my $PreDefChild (@{$Attribute->folder('PreDefAttributes')->children}) {
            $hPreDefAttributeChildren->{$PreDefChild->alias} = $PreDefChild->id;
        }
        $Servlet->vars('ResponseContent', {
            'PreDefAttributeChildren' => $hPreDefAttributeChildren,
        });
    }
}

sub UnitySavePreDefAttributes {
    my $self = shift;
    my ($Servlet) = @_;


    my $Attribute = $Servlet->object;
    my $Shop      = $Attribute->getSite();
    my $Form      = $Servlet->form->form($Attribute, 'ProductAttributeDetail');
    my $LanguageID = $Form->{'LanguageID'};
    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Shop->get('Languages')};

    # Save Attribute Settings
    $Attribute->set({ 'Name' => $Form->{'Name'} }, $LanguageID);
    $Attribute->set({ 'IsVisible' => $Form->{'IsVisible'} });

    my $Folder = $Attribute->folder('PreDefAttributes');
    my @PreDefAttributeIDs;
    # loop over all form entries
    foreach my $PreDefAttributeEntry (@{$Form->{'PreDefAttributes'}}) {
        my $PreDefAttributeID = $PreDefAttributeEntry->{'PreDefAttributeID'};
        # create new object if no existing PreDefAttributeID given
        if ($PreDefAttributeID == -1) {
            # create unique alias from value
            my $Alias = $Folder->uniqueChildAlias(Unicode2Alias($PreDefAttributeEntry->{'ValueLocString'}));
            my $PreDefAttribute = $Folder->insertChild('PreDefAttribute',{
                'Alias'       => $Alias,
                'Attribute'   => $Attribute,
                'PreDefClass' => $Attribute->class,
                'Position'    => $PreDefAttributeEntry->{'Position'}
            });
            foreach my $LanguageID (@ShopLanguageIDs) {
                $PreDefAttribute->set({'ValueLocString' => $PreDefAttributeEntry->{'ValueLocString'}}, $LanguageID);
            }
            push @PreDefAttributeIDs, $PreDefAttribute->id;
        } else {
            # set values for already existing attributes
            my $PreDefAttribute = $Shop->loadObject($PreDefAttributeID, 'PreDefAttribute');
            die 'invalid PreDefAttribute:'.$PreDefAttribute->id.' used in Attribute:'.$Attribute->id if $PreDefAttribute->get('Attribute')->id != $Attribute->id;

            $PreDefAttribute->set({'Position' => $PreDefAttributeEntry->{'Position'}});
            $PreDefAttribute->set({'ValueLocString' => $PreDefAttributeEntry->{'ValueLocString'}}, $LanguageID);
            push @PreDefAttributeIDs, $PreDefAttributeID;
        }
    }

    # compare form attributes to exiting attibutes and delete the missing ones
    my @DeleteObjects;
    foreach my $PreDefAttribute (@{$Folder->children}) {
        my $PreDefAttributeID = $PreDefAttribute->id;
        next if $PreDefAttributeID ~~ @PreDefAttributeIDs;

        # only delete values that are not used yet
        if  ( scalar @{GetPKeySelectedVariationsByPreDefAttributeID($PreDefAttributeID)} or scalar @{GetPKeySelectPreDefAttributesByPreDefAttributeID($PreDefAttributeID)} ) {
           $Servlet->form->addFormError( {
               Reason => 'ProductsWithThisVariationValueExists',
               Name => $PreDefAttribute->get('ValueLocString'),
           } );
        } else {
            push @DeleteObjects, $PreDefAttribute;
        }
    }
    $self->deleteObjectListWrapper( \@DeleteObjects ) if scalar @DeleteObjects;

    $Folder->renumberChildren;

    $Servlet->form->executeFormError;

    return;
}

sub UnityView {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

1;

