#!perl -w
use strict;
use Test::More tests => 9;

use DE_EPAGES::Core::API::Error qw( HandleError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Basket qw( AddProducts );
use DE_EPAGES::Trigger::API::Table::Hook qw( GetPKeyHookByName );
use DE_EPAGES::Trigger::API::Table::HookFunction qw( ExistsHookFunctionByHookID GetPKeyHookFunctionByHookID DeleteHookFunction InsertHookFunction );

my $ROOTPATH = '/Test';

## no critic qw( ProhibitMagicNumbers )

sub OnProductLineItemQuantity {
    my ($hArgs) = @_;

    $hArgs->{'Quantity'} = 99.1 if $hArgs->{'Quantity'} > 99.1;
}

sub TestLineItemProduct {

    my $Shop = LoadObjectByPath('/Test/TestShop');
    my $User = $Shop->folder('AnonymousUsers')->insertChild('AnonymousUser', {
        'Alias' => 'TestUser',
        'Name' => 'TestUser',
    });
    my $Product = $Shop->get('ProductFolder')->child('MyPetrol');
    ok( $Product->get('IsVisible'), 'product is visible' );
    ok( $Product->get('CanAddToBasket'), 'product can be added to the basket' );

    my $Basket;
    my $aWarnings = BlockWarnings( sub {
        my $aBaskets = AddProducts($User, { 'CurrencyID' => 'EUR' },
            [{ Product => $Product, Quantity => 135.7 }]);
        $Basket = $aBaskets->[0];
    });
    is( scalar @$aWarnings, 0, 'AddProducts - no warnings' );
    Test::More::diag $_->longMessage foreach @$aWarnings;

    ok( $Basket->get('User')->id == $User->id, 'Basket.User' );

    my $aLineItems = $Basket->container->get('LineItems');
    is( scalar @$aLineItems, 1, 'number of lineitems' );

    my ($LineItem) = grep { $_->instanceOf('LineItemProduct') } @$aLineItems;
    ok( defined $LineItem, 'LineItemProduct exists in basket' );
    is( $LineItem->get('Quantity'), 100, 'modified Quantity, corrected for minimum order qty and interval' );
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    my $HookID = GetPKeyHookByName('ProductLineItemQuantity');
    if( ExistsHookFunctionByHookID( $HookID, 'OnProductLineItemQuantity' ) ) {
        my $HookFunctionID = GetPKeyHookFunctionByHookID( $HookID, 'OnProductLineItemQuantity' );
        DeleteHookFunction( $HookFunctionID );
    }

    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    ImportObjects('initial.xml', $ROOTPATH);

    my $HookID = GetPKeyHookByName('ProductLineItemQuantity');
    InsertHookFunction({
        HookID => $HookID,
        FunctionName => 'OnProductLineItemQuantity',
        OrderNo => 0,
    });
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        ok( Init(), 'initialization' );

        eval { TestLineItemProduct(); };
        HandleError( Finally => sub {
            ok( Cleanup(), 'cleanup');
        });
    });
});
