package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::Exception;
use Test::More tests => 14;

use Data::Dumper;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::ProductVariation;

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_master;
    $self->test_get_sub;
    $self->test_errors;
}

sub test_get {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductVariation->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyPetrol');

    throws_ok { $Resource->get( $Product ) } qr/ResourceNotFound/, 'GET non-master/variations';
}

sub _SubProduct {
    my ($Product) = @_;

    return unless $Product->get('IsVisible');
    return {
        attributeSelection => [
            map {
                {
                    name => $_,
                    value => $Product->get($_)->alias,
                }
            } qw( Color Size )
        ],
        link => {
            href => 'https://test-domain/api/products/'.$Product->guid,
            rel  => 'variation'
        }
    };
}

sub test_get_master {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductVariation->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $hData = $Resource->get( $Product );

    is_deeply( $hData, {
        items => [
            map { _SubProduct($_) } @{ $Product->subProducts }
        ],
        links => [],
        page => 1,
        results => 9,
        resultsPerPage => 100,
        variationAttributes => [
            {
                name => 'Color',
                values => [
                {
                    displayValue => 'Black',
                    value => 'Black'
                },
                {
                    displayValue => 'ExtremeRed',
                    value => 'ExtremeRed'
                }
                ],
                displayName => 'Color'
            },
            {
                name => 'Size',
                values => [
                    {
                        displayValue => 'S',
                        value => 'S'
                    },
                    {
                        displayValue => 'M',
                        value => 'M'
                    },
                    {
                        displayValue => 'L',
                        value => 'L'
                    },
                    {
                        displayValue => 'XL',
                        value => 'XL'
                    },
                    {
                        displayValue => 'XXL',
                        value => 'XXL'
                    }
                ],
                displayName => 'Size'
            }
        ],
    }, 'GET MyJacket' ) or Test::More::diag Dumper($hData);
}

sub test_get_sub {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductVariation->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2');

    throws_ok { $Resource->get( $Product ) } qr/ResourceNotFound/, 'GET subproduct/variations';
}

sub test_errors {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductVariation->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $LocaleID = 'en_GB';

    throws_ok { $Resource->get( $Product, $LocaleID, 0 ) } qr/BadRequest/, 'invalid page=0';
    throws_ok { $Resource->get( $Product, $LocaleID, -5 ) } qr/BadRequest/, 'invalid page=-5';
    throws_ok { $Resource->get( $Product, $LocaleID, 'X' ) } qr/BadRequest/, 'invalid page=X';

    lives_ok { $Resource->get( $Product, $LocaleID, 1, 1000 ) } 'maximum page size';
    throws_ok { $Resource->get( $Product, $LocaleID, 1, 1001 ) } qr/BadRequest/, 'page size too large';
    throws_ok { $Resource->get( $Product, $LocaleID, 1, 0 ) } qr/BadRequest/, 'invalid pageSize=0';
    throws_ok { $Resource->get( $Product, $LocaleID, 1, -5 ) } qr/BadRequest/, 'invalid pageSize=-5';
    throws_ok { $Resource->get( $Product, $LocaleID, 1, 'X' ) } qr/BadRequest/, 'invalid pageSize=X';
    throws_ok { $Resource->get( $Product, $LocaleID, 1, '' ) } qr/BadRequest/, 'invalid pageSize=""';

    throws_ok { $Resource->get( $Product, 'xx_YY' ) } qr/BadRequest/, 'invalid locale';
    throws_ok { $Resource->get( $Product, 'ru_RU' ) } qr/BadRequest/, 'unsupported locale';
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
