#========================================================================================
# package     TopSellerTest
# description Test depending on Store datbase. This test will trigger complex query,
#              which will take 10 minutes or more on large databases to complete.
#========================================================================================
package TopSellerTest;
use base DE_EPAGES::Shop::API::Test;
use strict;

use Test::More tests => 11;
use Test::Deep;
use JSON;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Product::API::TopSeller
  qw ( GetTopSellerForShop CollectTopSellerStatistic );

use constant DAYS        => 30;
use constant MAX_RESULTS => 30;

sub test {
    my $self = shift;
    my $Shop = $self->shop;
    my $Dbi  = GetCurrentDBHandle();

    # is required for GetTopSellerForShop()
    $self->{LanguageID} = GetPKeyLanguageByCode('en');

    # Simple Execution Test
    eval { CollectTopSellerStatistic(); };
    ok( !ExistsError(), 'executing CollectTopSellerStatistic()' )
      or Test::More::diag( GetError()->longMessage() );

    eval { CollectTopSellerStatistic( Days => DAYS ); };
    ok( !ExistsError(), 'executing CollectTopSellerStatistic(Days => 3650)' )
      or Test::More::diag( GetError()->longMessage() );

    my $aaTopSellerCount =
      $Dbi->execute( 'SELECT COUNT(*) FROM topseller', [] );
    ok( scalar(@$aaTopSellerCount) > 0, 'there a number of topseller' );

    my $hTopSeller =
      GetTopSellerForShop( $Shop, $self->{LanguageID}, MAX_RESULTS );
    is( $hTopSeller->{Count}, 0, 'testshop has no orders -> no topseller' );

    # Add Order and Products
    $self->_importTopSellerXML();

    CollectTopSellerStatistic( Days => DAYS );
    $hTopSeller =
      GetTopSellerForShop( $Shop, $self->{LanguageID}, MAX_RESULTS );
    is( $hTopSeller->{Count}, 4, 'has topseller' );

    # Check restult for TestShop in detail
    my $ahTopSeller = from_json( $hTopSeller->{JSON} );

    cmp_deeply( $ahTopSeller, _ExpectedTopSeller(),
'GetTopSellerForShop - results is sorted, bundles are there, lineitemcount are acounted for'
    );
    cmp_deeply(
        $ahTopSeller,
        array_each( _ExpectedTyps() ),
        'GetTopSellerForShop type checking results'
    );

    # Delete product which is part of a bundle
    $ahTopSeller = $self->_getDeleteProductAndGetTopSellerArray('001_simple');
    ok( !( grep { $_->{name} eq '001_simple' } @$ahTopSeller ),
        'delete product of bundle' );

    # Delete bundle product - only bundle is deleted
    $ahTopSeller = $self->_getDeleteProductAndGetTopSellerArray('001_bundle');
    ok( !( grep { $_->{name} eq '001_bundle' } @$ahTopSeller ),
        'delete bundle itself' );

    # Delete Product
    $ahTopSeller = $self->_getDeleteProductAndGetTopSellerArray('004_other');
    ok( !( grep { $_->{name} eq '004_other' } @$ahTopSeller ),
        'delete simple product' );

    # preperation delete TestShop (See sub cleanup)
    $self->{ShopID}         = $Shop->id();
    $self->{DoCleanUpTests} = 1;
}

sub cleanup {
    my $self = shift;

    $self->SUPER::cleanup();
    if ( $self->{DoCleanUpTests} ) {
        my $TopSellerCnt =
          GetCurrentDBHandle()
          ->execute( 'SELECT COUNT(*) FROM topseller WHERE shopid = ?',
            [ $self->{ShopID} ] )->[0][0];
        is( $TopSellerCnt, 0, 'no TopSeller for deleted shop' );
    }
    return;
}

####################################################################################################################
#
#   Helper
#
####################################################################################################################
sub _importTopSellerXML {
    my $self    = shift;
    my $XmlFile = 'ImportTopSeller.xml';
    eval { ImportObjects( $XmlFile, $self->shop()->parent()->get('Path') ); };
    if ( ExistsError() ) {
        my $Error = GetError();
        Test::More::diag( GetError()->longMessage() );
        Error($Error);
    }
    else {
        note("import $XmlFile");
    }
}

sub _getDeleteProductAndGetTopSellerArray {
    my $self    = shift;
    my ($Alias) = @_;
    my $Shop    = $self->shop();

    $Shop->child('Products')->child($Alias)->delete;
    CollectTopSellerStatistic( Days => DAYS );
    my $hTopSeller =
      GetTopSellerForShop( $Shop, $self->{LanguageID}, MAX_RESULTS );

    return from_json( $hTopSeller->{JSON} );

}
####################################################################################################################
#
#   Rules for cmp_deepley
#
####################################################################################################################
sub _ExpectedTyps {
    return {
        productId      => re('\d+'),
        count          => re('^\d+$'),
        name           => re('\w+'),
        thumbnail      => re('\w+'),
        variationString => ignore(),

    };
}

sub _ExpectedTopSeller {
    return [
        {

            # two lineitem in order is counted
            count          => '2',
            name           => '002_double',
            productId      => ignore(),
            thumbnail      => 'NO_IMAGE',
            variationString => undef,
        },
        {

            # simple lineitem
            count          => '1',
            name           => '001_bundle',
            productId      => ignore(),
            thumbnail      => 'BUNDLED_PRODUCT_IMAGE',
            variationString => undef,
        },
        {

            # simple lineitem
            count          => '1',
            name           => '004_other',
            productId      => ignore(),
            thumbnail      => 'NO_IMAGE',
            variationString => undef,
        },
        {

            # product bundles are tracked
            count          => '1',
            name           => '001_simple',
            productId      => ignore(),
            thumbnail      => 'NO_IMAGE',
            variationString => undef,
        }
    ];
}

__PACKAGE__->runScript;

1;
