## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
package MyTest;
use parent DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 9;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( GetDestinctSelections GetPageSizeForSearch );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);

use JSON::XS qw ( decode_json );

my $ROOTALIAS = 'Test';
my $ROOTPATH = JoinObjectPath(['',$ROOTALIAS]);
my $SHOP = JoinObjectPath(['',$ROOTALIAS, 'TestShopRemoteSearch']);


sub test {
    TestGetDestinctSelectionsString();
    TestGetDestinctSelectionsBoolean();
    TestPageSize();
}

sub TestGetDestinctSelectionsString {
    my $TestSet1 = GetDestinctSelections( TestSet1() );
    my $PerlDS;
    eval { $PerlDS = decode_json($TestSet1); };
    ok( !ExistsError(), 'Correct input produce correct JSON output (TestSet1)' ) or Test::More::diag( GetError->longMessage() );
    eval { $PerlDS = decode_json( $TestSet1 . 'malicious code from attacker goes here' ); };
    ok( ExistsError(), 'Invalid input produce error (TestSet1) - hacking protection' );
    my $hTestSet1 = decode_json($TestSet1);
    ok( exists $hTestSet1->{Size} && $hTestSet1->{Size} eq 'S', 'Size "S" is selected (PreDefString)' );
    ok( exists $hTestSet1->{Colour} && $hTestSet1->{Colour} eq 'Extreme red',
        'Colour "S" is selected (PreDefLocalizedString)' );
    ok( 2 == scalar keys %$hTestSet1, 'No false positive (TestSet1)' );
}

sub TestGetDestinctSelectionsBoolean {
    my $TestSet2 = GetDestinctSelections( TestSet2() );
    ok( $TestSet2 !~ m/DontCareForStandard/, 'Selected standard attributes are dropped' );
    my $hTestSet2 = decode_json($TestSet2);
    ok(  exists $hTestSet2->{TrueFalse} && $hTestSet2->{TrueFalse} == 0, 'Boolean "0" is selected' );
    ok( 1 == scalar keys %$hTestSet2, 'No false positive (TestSet2)' );
}

sub TestPageSize {
    my $Shop = GetShop();
    # this will dig up defualt System.PageSizes.*,
    # because no shop specific PageSize was defined
    my $PageSize = GetPageSizeForSearch($Shop);
    is($PageSize, 30, 'GetPageSizeForSearch'); ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
}

######################################### Helper ##################################################

sub init {
    return if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};

    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return;
}

sub cleanup {
    return if $ENV{'SKIP_CLEANUP'};

    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return;
}

sub GetShop {
    return LoadObjectByPath($SHOP);
}

######################################### TestData ##################################################


# custom PreDefString && PreDefLocalizedString
sub TestSet1 {
    return [ {
            'attributeName' => 'Category',
            'attributeType' => 'Standard',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => '16961',
                        'fieldName'      => 'CategoryID'
                    },
                    'matches'       => 1,
                    'name'          => 'Jackets',
                    'selectedValue' => '16961'
                }
            ]
        }, {
            'attributeName' => 'Manufacturer',
            'attributeType' => 'Standard',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => 'Berghaus',
                        'fieldName'      => 'Manufacturer'
                    },
                    'matches' => 1,
                    'name'    => 'Berghaus'
                }
            ]
        }, {
            'attributeName' => 'Colour',
            'attributeType' => 'PreDefLocalizedString',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => 'Black',
                        'fieldName'      => 'PreDefLocalizedString_en_2023991788'
                    },
                    'matches' => 3,
                    'name'    => 'Black'
                }, {
                    'filter' => {
                        'attributeValue' => 'Extreme red',
                        'fieldName'      => 'PreDefLocalizedString_en_2023991788'
                    },
                    'matches'       => 1,
                    'name'          => 'Extreme red',
                    'selectedValue' => 'Extreme red'
                }, {
                    'filter' => {
                        'attributeValue' => 'Extreme blue',
                        'fieldName'      => 'PreDefLocalizedString_en_2023991788'
                    },
                    'matches' => 1,
                    'name'    => 'Extreme blue'
                }, {
                    'filter' => {
                        'attributeValue' => 'Croft',
                        'fieldName'      => 'PreDefLocalizedString_en_2023991788'
                    },
                    'matches' => 1,
                    'name'    => 'Croft'
                }
            ]
        }, {
            'attributeName' => 'Size',
            'attributeType' => 'PreDefString',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => 'S',
                        'fieldName'      => 'PreDefString_en_2577441'
                    },
                    'matches'       => 1,
                    'name'          => 'S',
                    'selectedValue' => 'S',

                }, {
                    'filter' => {
                        'attributeValue' => 'M',
                        'fieldName'      => 'PreDefString_en_2577441'
                    },
                    'matches' => 1,
                    'name'    => 'M'
                }, {
                    'filter' => {
                        'attributeValue' => 'L',
                        'fieldName'      => 'PreDefString_en_2577441'
                    },
                    'matches' => 1,
                    'name'    => 'L'
                }, {
                    'filter' => {
                        'attributeValue' => 'XL',
                        'fieldName'      => 'PreDefString_en_2577441'
                    },
                    'matches' => 1,
                    'name'    => 'XL'
                }, {
                    'filter' => {
                        'attributeValue' => 'XXL',
                        'fieldName'      => 'PreDefString_en_2577441'
                    },
                    'matches' => 1,
                    'name'    => 'XXL'
                }
            ]
        }
    ];
}

# custom boolean selected && one standard attrbitute selected
sub TestSet2 {

    return [ {
            'attributeName' => 'DontCareForStandard',
            'attributeType' => 'Standard',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => '16961',
                        'fieldName'      => 'CategoryID'
                    },
                    'matches'       => 1,
                    'name'          => 'Jackets',
                    'selectedValue' => '16961'
                }
            ]
        }, {
            'attributeName' => 'TrueFalse',
            'attributeType' => 'Boolean',
            'values'        => [ {
                    'filter' => {
                        'attributeValue' => '0',
                        'fieldName'      => 'Boolean_en_411235867'
                    },
                    'matches'       => 1,
                    'name'          => 'false',
                    'selectedValue' => '0'
                }, {
                    'filter' => {
                        'attributeValue' => '1',
                        'fieldName'      => 'Boolean_en_411235867'
                    },
                    'matches'       => 4,
                    'name'          => 'true',
                }
            ]
        }
    ];
}

__PACKAGE__->runScript;

1;