use strict;

sub RemoteSearchTestGetTaskLink {
    my ($hInfo) = @_;
    $DE_EPAGES::RemoteSearch::t::indexIntegration::UPDATE_TASK = $hInfo->{'Task'};
}

#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::indexIntegration
# state        private
# description  Index integration tests
#               Requirement for integration testing in a distributed environment:
#               $EPAGESJ_CONFIG/epagesj.conf must be readable on the host where you run
#               this test. This is used to find the URLs pointing to the Solr servers.
#========================================================================================
package DE_EPAGES::RemoteSearch::t::indexIntegration;
use base 'Test::Class';

use strict;
use DE_EPAGES::Catalog::API::Catalog qw (
    AssignProductToCategory
    RemoveProductFromCategory
);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Url qw ( StringIsURL );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw ( ImportAttributes );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::RemoteSearch::API::IndexStatus qw (
    INDEXSTATUS_COMPLETE
    INDEXSTATUS_ERROR
    INDEXSTATUS_OFF
    IndexStatusString
);
use DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    DropAllSearchIndexes
    GetOrCreateSearchIndex
    RunIfRemoteSearchInitialized
);
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Trigger::API::Table::Hook qw ( GetPKeyHookByName );
use DE_EPAGES::Trigger::API::Table::HookFunction qw (
    DeleteHookFunction
    ExistsHookFunctionByHookID
    GetPKeyHookFunctionByHookID
    InsertHookFunction
);
use DE_EPAGES::WebInterface::API::FakeRequest;
use File::Basename;
use HTTP::Request::Common qw ( GET POST );
use HTTP::Status qw ( :constants );
use LWP::UserAgent;
use Test::More;
use URI;

my $ROOTALIAS = 'Test';
my $ROOTPATH = JoinObjectPath(['',$ROOTALIAS]);
my $HTTP_TIMEOUT = 8;
our $UPDATE_TASK = undef;

sub checkPreconditions {
    my $class = shift;
    if (! LoadRootObject->get('RemoteSearchEnabled')) {
        $class->SKIP_CLASS('remote search not enabled');
    }
    unless( _PingEpagesJ()->is_success()) {
        $class->SKIP_CLASS('ePagesJ is not reachable');
    }
}

sub _PingEpagesJ() {

    my $ua = LWP::UserAgent->new;
    $ua->timeout($HTTP_TIMEOUT);

    return $ua->request(GET(LoadRootObject->get('RemoteSearchURL')));
}

sub init : Test(startup) {
    my $self = shift;
    $self->{SolrConfig} = $self->readEPJConfig();
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();

    InsertHookFunction({
            'HookID' => GetPKeyHookByName('API_RemoteSearchUpdateTaskStatus'),
            'OrderNo' => 1,
            'FunctionName' => 'RemoteSearchTestGetTaskLink',
    });

    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');
    ImportAttributes( 'import_en.xml', $ROOTPATH . '/TestShopRemoteSearch');
    $self->{Shop}  = LoadObjectByPath(JoinObjectPath(['',$ROOTALIAS, 'TestShopRemoteSearch']));
    return 1;
}

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    my $HookID = GetPKeyHookByName('API_RemoteSearchUpdateTaskStatus');
    if (ExistsHookFunctionByHookID($HookID, 'RemoteSearchTestGetTaskLink')) {
        DeleteHookFunction(GetPKeyHookFunctionByHookID($HookID, 'RemoteSearchTestGetTaskLink'));
    }
}

sub setUpTest : Test(setup) {
    my $self = shift;
    $self->{Index} = GetOrCreateSearchIndex($self->shop);
}

sub testInitializeIndex : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
}

sub testDropSearch : Test(no_plan) {
    my $self = shift;
    my $Task = $self->index->drop();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_OFF, 'testDropSearch: index off');
    $self->checkForTaskCompletion($Task);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_OFF, 'testDropSearch: index off')
    or Test::More::diag($Task->link);
}

sub testDropAllSearchIndexes : Test(no_plan) {
    my $self = shift;
    DropAllSearchIndexes($self->shop);
    ok(!$self->shop->existsChild('SearchIndexes'), 'reset removes all search indexes');
}

sub testUpdateIndexItems : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $Task = $self->index()->updateItems($self->shop->get('Products'));
    $self->checkForTaskCompletion($Task);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIndexItems: index clean')
    or Test::More::diag($Task->link);
    is($self->index()->get('IndexStatusString'), INDEXSTATUS_COMPLETE, 'check read only attribute IndexStatusString');
}

sub testUpdateIndexItemsByHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->shop->folder('Products')->child('cg_0101104470')->set({ 'Name' => 'My New Product Name'}, 1);
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIndexItemsByHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testUpdateIndexItemsInBatch : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    RunBatchProcess('UpdateProducts', sub {
        $self->shop->folder('Products')->child('cg_0101104470')->set({ 'Name' => 'My New Product Name 2'}, 1);
    });
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIndexItemsInBatch: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testDeleteIndexItemsInBatch : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    RunBatchProcess('DeleteProducts', sub {
        $self->shop->folder('Products')->child('de_3206199099')->delete;
    });
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testDeleteIndexItemsInBatch: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testDeleteProductTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    $self->shop->folder('Products')->child('cg_0101104471')->delete;
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testDeleteProductTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testInsertProductTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    $self->shop->folder('Products')->insertChild('Product', {Alias => 'xx_0101104472'});
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testInsertProductTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testUpdateProductCategoryPositionTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    my $aProductMaps = $self->shop->folder('Products')->child('cg_0101104470')->get('CategoryProductMaps');
    $aProductMaps->[0]->set({'Position' => 999 });
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateProductCategoryPositionTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testInsertCategoryProductMapTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    my $Product = $self->shop->folder('Products')->child('hm_0100401001');
    my $Category = $self->shop->child('Categories')->child('Equipment');
    AssignProductToCategory($Product, $Category);
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testInsertCategoryProductMapTriggersHook: assign - index clean')
    or Test::More::diag($UPDATE_TASK->link);

    RemoveProductFromCategory($Product, $Category);
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testInsertCategoryProductMapTriggersHook: unassign index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testRenameShopTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    $self->shop->set({Alias => "RemoteSearchShopDifferentAlias"});
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testInsertProductTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
    # revert back
    $self->shop->set({Alias => "TestShopRemoteSearch"});
}

sub testQuerySuggest : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $hResult = $self->index->remoteSuggest->suggestAsHash('Deuter','de');
    is($hResult->{manufacturers}->[0]->{name},'Deuter','testQuerySuggest: suggest contains Deuter.');
}

sub testDeleteIndexItems : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    $self->index()->deleteItems($self->shop->get('Products'));
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testDeleteIndexItems: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testUpdateProductTypeTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    my $Black = $self->shop->path('ProductTypes/Jacket/Attributes/Color/PreDefAttributes/Black');
    $Black->set({'ValueString' => 'Blaaack'}, 1);
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateProductTypeTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testUpdateIsNewTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    $self->shop->folder('Products')->child('cg_0101104470')->set({IsNew => 1});
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIsNewTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testUpdateDefaultSubProductTriggersHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    local $UPDATE_TASK = undef;
    my $Jacket = $self->shop->path('Products/MyJacket');
    $Jacket->set({'DefaultSubProduct' => $Jacket->path('SubProducts/JacketRed1')});
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateDefaultSubProductTriggersHook: index clean')
    or Test::More::diag($UPDATE_TASK->link);

    # unset
    $UPDATE_TASK = undef;
    $Jacket->set({'DefaultSubProduct' => undef});
    $self->checkForTaskCompletion($UPDATE_TASK);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateDefaultSubProductTriggersHook: index clean again')
    or Test::More::diag($UPDATE_TASK->link);
}

sub testSimpleSearch : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $hParams = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new()->lang('en')->query('Grangers')->currency('EUR')->params();
    my $hResponse = $self->index->remoteSearch->searchAsHash($hParams);
    is($hResponse->{results}, 1, 'testSimpleSearch: Grangers has one result');
    is($hResponse->{items}->[0]->{'filter'}
        , $self->shop->folder('Products')->child('ht_0801099030')->id
        , 'testSimpleSearch: Grangers ObjectID matches');
}

sub testRunIfRemoteSearchInitialized : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $aWarnings = BlockWarnings( sub {
        RunIfRemoteSearchInitialized($self->shop, sub {
            Error('SOME_ERROR', {});
        });
    });
    is ($aWarnings->[0]->code, 'SOME_ERROR', 'caught a non-native die');
}

#========================================================================================
#   Helper Methods
#========================================================================================
sub initializeIndex {
    my $self = shift;
    my  (undef,undef, undef,$subroutine) = caller(1);
    $subroutine = substr ($subroutine, rindex($subroutine, '::')+2);
    return if (IndexStatusString($self->index()->get('IndexStatus')) eq INDEXSTATUS_COMPLETE);
    my $Task = GetOrCreateSearchIndex($self->shop)->initialize();
    ok(StringIsURL($Task->link), $subroutine .  ': task url') or Test::More::diag($Task->link);
    $self->checkForTaskCompletion($Task);
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'initializeIndex: index ready')
    or Test::More::diag($subroutine .  ': '.$Task->link);
    return;
}

sub checkForTaskCompletion {
    my $self = shift;
    my ($Task) = @_;
    my  (undef,undef, undef,$subroutine) = caller(1);
    $subroutine = substr ($subroutine, rindex($subroutine, '::')+2);
    ok(StringIsURL($Task->link), $subroutine .  ': task url') or Test::More::diag($Task->link);
    for ( my $i = 0; $i< 9; ++$i) {
        sleep 1;
        my $Index = $self->index();
        $Index->updateIndexStatus($Index->getUpdateTask($Task->link));
        my $IndexStatusString = IndexStatusString($Index->get('IndexStatus'));
        last if (($IndexStatusString eq INDEXSTATUS_COMPLETE) or
                 ($IndexStatusString eq INDEXSTATUS_ERROR)
        );
    }
    $self->forceSolrUpdate();
}

# make sure the changes have been committed to solr index.
sub forceSolrUpdate {
    my $self = shift;

    my $ua = LWP::UserAgent->new;
    $ua->timeout($HTTP_TIMEOUT);
    my $commitMaster  = POST( $self->solrConfig->{'masterUrl'} . 'Products/update/?commit=true' );
    my $Response      = $ua->request($commitMaster);
    ok($Response->is_success, '-> force commit on solr master via ' . $commitMaster->uri) or Test::More::diag($Response->status_line);

    # slaves are optional
    foreach my $Slave ( @{ $self->solrConfig->{'slaveUrls'} } ) {
        my $forceSlaveReplication = GET( $Slave . 'Products/replication?command=fetchindex' );
        $Response = $ua->request($forceSlaveReplication);
        ok($Response->is_success, '-> force slave update via ' . $forceSlaveReplication->uri) or Test::More::diag($Response->status_line);
    }
}

# - read diffrent sections form config file depending on OS
# - remove quotes in config values.
sub readEPJConfig {
    my $self = shift;
    my $FilePath = $ENV{EPAGESJ_CONFIG}.'/epagesj.conf';
    my $hConfig = {'masterUrl' => 'http://localhost:8983/solr/', 'slaveUrls' => []};
    return $hConfig unless (-r $FilePath);

    my $IniConfig = DE_EPAGES::Core::API::IniConfig->new(
        FileName => $FilePath,
        CaseSensitive => 1,
    );

    if ( IsWindows() ) {
        # Windows (sections in camel case)
        $hConfig->{'masterUrl'} = $IniConfig->get('SolrClient','masterUrl')
                               // $hConfig->{'masterUrl'};
        $hConfig->{'slaveUrls'} = [ map  { $_ =~ s/"//g; $_ }
                                    grep { $_ ne $hConfig->{'masterUrl'} } split(/,/,
                                    $IniConfig->get('SolrClient','slaveUrls') ) ];
    }
    else {
        # Linux (lower case sections first)
        $hConfig->{'masterUrl'} =  $IniConfig->get('epagesj','masterUrl')
                                // $IniConfig->get('SolrClient','masterUrl')
                                // $hConfig->{'masterUrl'};
        $hConfig->{'slaveUrls'} = [ map  { $_ =~ s/"//g; $_ }
                                    grep { $_ ne $hConfig->{'masterUrl'} } split(/,/,
                                      $IniConfig->get('epagesj','slaveUrls')
                                   // $IniConfig->get('SolrClient','slaveUrls')
                                ) ];
    }

    $hConfig->{'masterUrl'} =~ s/"//g;
    # dump solr info
    Test::More::diag 'found solr servers to connect to';
    Test::More::diag explain $hConfig;

    return $hConfig;
}

sub IsWindows {
    return $^O =~ m/MSWin32/;
}
sub shop {
    return shift->{Shop};
}

sub index {
    return shift->{Index};
}

sub solrConfig {
    return shift->{SolrConfig};
}

#========================================================================================
#   MAIN
#========================================================================================
RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        DE_EPAGES::RemoteSearch::t::indexIntegration->checkPreconditions();
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);
