use Test::More;
use Test::Deep;

use strict;

use Getopt::Long;
use DE_EPAGES::Core::API::File qw ( GetFileContentLines DeleteFile );
use DE_EPAGES::Core::API::PerlTools qw ( unique );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Database::API::Connection qw (
    GetTestSite
    RunOnStore
    Transaction
);
use DE_EPAGES::Object::API::Factory qw (
    LoadClassByAlias
    LoadObjectByPath
);
use DE_EPAGES::Shop::API::Shop qw ( GetNextNo );
use DE_EPAGES::Object::API::Table::IntegerAttribute qw( DeleteIntegerAttribute );

use Readonly;
Readonly::Scalar my $BASE_FILENAME => 'GetNextNo';
Readonly::Scalar my $NUMBER_OF_TASKS  => 8;
Readonly::Scalar my $NUMBERS_PER_TASK => 10;
Readonly::Scalar my $ROOTPATH      => '/Shops';

# Cannot use /Test/TestShop, because every attempt to access the Store database
# prior to invoking the subroutine GetNextCustomerNo* results in MySQL errors:
# Mysql complains that AutoCommit is not set, although it is set.
local $ENV{'EPAGES_TESTSITE'} = 'Store';

my $Verbose;

#========================================================================================
# description  Tests simultaneous access to the subroutine $FunctionRef by creating
#               several child processes, each calling $FunctionRef in parallel. That
#               way, race conditions that might be raised during the execution of
#               $FunctionName can be detected. We assume that the subroutine
#               'IncrementIntegerAttribute' (cf. Object/DAL/SQL/DefaultAttribute.pm)
#               is already thread-safe.
#========================================================================================
sub TestGetNextNo {
    my ($Identifier, $FunctionName) = @_;

    my @ForkedTasks;

    foreach my $TaskID ( 1 .. $NUMBER_OF_TASKS ) {
        my $PID = fork();
        if ($PID) {  # parent
            push @ForkedTasks, $PID;
        } elsif ($PID == 0) {  # child
            my $CommandLine = "$ENV{PERL} $0 -task $TaskID -function $FunctionName >$BASE_FILENAME$TaskID";
            print "run $CommandLine\n" if $Verbose;
            SystemCommand( $CommandLine );
            exit 0;
        } else {
            die "Couldn't fork: $!\n";
        }
    }

    foreach my $Task ( @ForkedTasks ) {
        my $KID;
        do {
            $KID = waitpid($Task, 0);
        } while $KID > 0;
    }

    my @AllNumbers;
    foreach my $TaskID ( 1 .. $NUMBER_OF_TASKS ) {
        my $aLines = GetFileContentLines("$BASE_FILENAME$TaskID");
        my @Numbers = map { chomp; $_ } @$aLines;
        is( scalar @Numbers, $NUMBERS_PER_TASK, "Numbers generated by process $TaskID" );
        push @AllNumbers, @Numbers;
    }

    is( scalar @AllNumbers, $NUMBER_OF_TASKS * $NUMBERS_PER_TASK, "Total numbers of numbers ($Identifier)" );
    is( scalar @AllNumbers, scalar( unique @AllNumbers ), "All numbers are unique ($Identifier)" );
    return;
}

#========================================================================================
# description  Invokes the subroutine 'GetNextNo' in order to retrieve the next
#               customer number.
#========================================================================================
sub GetNextCustomerNoTask {
    my ($TaskID) = @_;

    my @Numbers;
    RunOnStore(
        Store => GetTestSite(),
        Sub   => sub {
            my $Shop = LoadObjectByPath($ROOTPATH)->child('DemoShop');
            for ( 1 .. $NUMBERS_PER_TASK ) {
                push @Numbers, GetNextNo($Shop, 'Customer') ;
                sleep(rand(2));  # let the other child processes interfere
            }
        }
    );

    return \@Numbers;
}

#========================================================================================
# description  Invokes the subroutine 'GetNextNo' in order to retrieve the next
#               customer number; nested by a Transaction.
#========================================================================================
sub GetNextCustomerNoTaskByTransaction {
    my ($TaskID) = @_;

    my @Numbers;
    RunOnStore(
        Store => GetTestSite(),
        Sub   => sub {
            my $Shop = LoadObjectByPath($ROOTPATH)->child('DemoShop');
            for ( 1 .. $NUMBERS_PER_TASK ) {
                Transaction(sub {
                    push @Numbers, GetNextNo($Shop, 'Customer');
                    sleep(rand(2));  # let the other child processes interfere
                });
            }
        }
    );

    return \@Numbers;
}

sub Main {

    my ($Help, $TaskID, $FunctionName);
    GetOptions(
        'help'           => \$Help,
        'task=i'         => \$TaskID,
        'function=s'     => \$FunctionName,
        'output!'        => \$Verbose,
    );
    usage() if $Help;

    if( $FunctionName ) {
        # sub task
        usage('missing parameter -task') unless $TaskID;

        {
            no strict 'refs';  ## no critic qw(ProhibitNoStrict)
            my $aNumbers = $FunctionName->($TaskID);
            print "$_\n" foreach @$aNumbers;
        }
    }
    else {
        ok( Init(), 'intialization done' );

        # main test case
        RunOnStore( Store => GetTestSite(), Sub => sub {
            my $Shop = LoadObjectByPath($ROOTPATH)->child('DemoShop');
            my $NextCustomerNo = GetNextNo($Shop, 'Customer');
            is( $NextCustomerNo, 1000, 'First CustomerNo' ); ## no critic qw(ProhibitMagicNumbers)
        });

        TestGetNextNo('NextCustomerNo', 'GetNextCustomerNoTask');
        TestGetNextNo('NextCustomerNoWithTransaction', 'GetNextCustomerNoTaskByTransaction');

        ok( Cleanup(), 'cleanup done' );
        done_testing();
    }

    return;
}

sub Init {
    Cleanup();

    return 1;
}

#========================================================================================
# description  Issue: Cannot reset NextCustomerNo to original value.
#========================================================================================
sub Cleanup {
    DeleteFile("$BASE_FILENAME$_") foreach ( 1 .. $NUMBER_OF_TASKS);

    RunOnStore( Store => GetTestSite(), Sub => sub {
        my $Shop = LoadObjectByPath($ROOTPATH)->child('DemoShop');

        my $Attribute = LoadClassByAlias('Shop')->attribute('NextCustomerNo');
        DeleteIntegerAttribute( $Shop->id, $Attribute->id );
    });

    return 1;
}

sub usage {
    my ($Msg) = @_;
    print "$Msg\n\n" if $Msg;
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Runs a sub task of the test case. Call this script without parameters to
    start the main task.

Options:
    -function   function name
    -task       task id

Flags:
    -nooutput       be silent, only display errors          (optional)
    -help           show the command line options           (optional)

Examples:
    perl $0
    perl $0 -nooutput -function GetNextCustomerNoTask -count 1

END_USAGE
exit 1;
}

RunScript(
    Message => 'long',
    Sub     => \&Main,
);

1;
