use Test::More tests => 25;
use strict;
use Data::Dumper qw( Dumper );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectToSite DisconnectSite );
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode); # $LANGUAGEID
use DE_EPAGES::Object::API::Factory qw( LoadRootObject ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::ShopConfiguration::API::Table::CartridgeRef qw(
    DeleteCartridgeRef
    GetPKeyCartridgeRefsByPackage
    InsertCartridgeRef
);

sub Test_ImportFiles {
    my ($ShopType) = @_;

    my $OriginalValue = $ShopType->get('ShopImportFile');

    $ShopType->set({'ShopImportFile' => ''});
    my $aFiles = $ShopType->get('ShopImportFiles');
    is( scalar @$aFiles, 0, '0 import files' ) or Test::More::diag Dumper($aFiles);

    $ShopType->set({'ShopImportFile' => 'test.xml'});
    $aFiles = $ShopType->get('ShopImportFiles');
    is( scalar @$aFiles, 1, '1 import file' ) or Test::More::diag Dumper($aFiles);
    is( $aFiles->[0],  'test.xml', '1 import file' );

    $ShopType->set({'ShopImportFile' => 'test.xml,data.xml'});
    $aFiles = $ShopType->get('ShopImportFiles');
    is( scalar @$aFiles, 2, '2 import files' ) or Test::More::diag Dumper($aFiles);
    is( $aFiles->[0],  'test.xml', '2 import files' );
    is( $aFiles->[1],  'data.xml', '2 import files' );

    $ShopType->set({'ShopImportFile' => $OriginalValue});
}

sub Main {
    ConnectToSite();

    Cleanup();

    my $System = LoadRootObject();

    # insert a cartridge
    my $CartridgeID = InsertCartridgeRef( {
        Package => 'C1::C1',
        Version => 25,
    } );
    ok( $CartridgeID, 'InsertCartridgeRef' );

    # add features to the cartridge
    my $FeatureFolder = $System->folder('FeatureRefs');
    my $Feature1 = $FeatureFolder->insertChild( 'FeatureRef', {
        Alias => 'F1',
        MaxValue => 1
    } );
    my $Feature2 = $FeatureFolder->insertChild( 'FeatureRef', {
        Alias => 'F2',
        MaxValue => 1000
    } );
    ok( $Feature2->id, 'create feature' );
    ok( $Feature2->get('MaxValue') == 1000, 'Feature.MaxValue' );

    # insert a provider
    my $Provider = $System->folder('Providers')->insertChild( 'Provider', {
        Alias => 'TestProvider1',
    } );
    ok( $Provider->id, 'create provider' );
    ok( 'TestProvider1' eq $Provider->alias, 'Provider.Alias' );

    my $EN = GetPKeyLanguageByCode( 'en' );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $EN;

    # new shop type
    my $ShopType1 = $Provider->insertShopType( { Alias => 'ST1', Name => 'ShopType1' } );

    ok( $ShopType1->id, 'new shop type' );
    ok( $ShopType1->get('Provider')->id == $Provider->id, 'ShopType.Provider' );
    ok( $ShopType1->get('Name') eq 'ShopType1', 'ShopType.Name' );
    ok( !$ShopType1->get('IsActive'), 'ShopType.IsActive is false by default' );

    ok( $Provider->shopTypes->[0]->id == $ShopType1->id, 'Provider.shopTypes' );
    ok( $Provider->shopType('ST1')->id == $ShopType1->id, 'Provider.shopType' );
    ok( $Provider->existsShopType('ST1'), 'Provider.existsShopType' );

    ok( scalar @{ $System->get('FeatureRefs') } >= 2, 'there are at least 2 features in the system' );

    my $FeatureSum = 0;
    $FeatureSum += $ShopType1->featureMaxValue( $_ ) foreach @{ $System->get('FeatureRefs') };
    ok( 0 == $FeatureSum, 'new shop type has all feature values set to zero' );

    ok( scalar @{ $System->get('FeatureRefs') } == scalar @{ $ShopType1->get('Features') }, 'shop type has all system features' );

    $ShopType1->featureMaxValue( $Feature2, 100 );
    ok( $ShopType1->featureMaxValue( $Feature2 ) == 100, 'featureMaxValue' );

    $FeatureSum = 0;
    $FeatureSum += $ShopType1->featureMaxValue( $_ ) foreach @{ $System->get('FeatureRefs') };
    ok( 100 == $FeatureSum, 'sum of feature values' );

    $FeatureSum = 0;
    $FeatureSum += $_->{MaxValue} foreach @{ $ShopType1->get('Features') };
    ok( 100 == $FeatureSum, 'property ShopType.Features[].MaxValue' );

    my $FeatureCount = scalar @{ $ShopType1->get('Features') };

    $Feature1->delete;
    ok( $FeatureCount-1 == scalar @{ $ShopType1->get('Features') }, 'delete FeatureRef' );

    Test_ImportFiles( $ShopType1 );

    Cleanup();
    DisconnectSite();
}

RunScript(
    Sub => \&Main
);

sub Cleanup {

    foreach my $Path (qw( /Test /FeatureRefs/F1 /FeatureRefs/F2 /Providers/TestProvider1 )) {
        LoadObjectByPath($Path)->delete if ExistsObjectByPath($Path);
    }

    DeleteCartridgeRef( $_ ) foreach @{ GetPKeyCartridgeRefsByPackage('C1::C1') };
}
