#==============================================================================
# package      DE_EPAGES::ShopMailType::UI::ShopMailType
# state        private
#==============================================================================
package DE_EPAGES::ShopMailType::UI::ShopMailType;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use Email::Address;
use DE_EPAGES::Core::API::Html qw ( CloseOpenHTMLTags );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );

#==============================================================================
# Event handlers
#==============================================================================

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShopMailType = $Servlet->object;
    my $Shop = $ShopMailType->getSite();
    my $Form = $Servlet->form;
    my $hFormValues = $Form->form($ShopMailType, 'Save');

    my $TO = $hFormValues->{'TO'};
    my $Subject = $hFormValues->{'Subject'};
    my $ShopMailFrom = $Shop->get('MailFrom');

    # throw error if fall back TO address is not present
    if (not defined $ShopMailFrom and not defined $TO) {
        $Form->executeFormError({
            'Reason' => 'MISSING_TO_ADDRESS',
            'Form'   => 'Form',
            'Name'   => 'TO',
            'Value'  => $TO,
        });
        return;
    }

    # check if subject is undefined, empty or includes only whitespace
    $Servlet->vars('EmptySubject', 1) if (defined $Subject and $Subject =~ /^\s*$/ or not defined $Subject);

    # save
    foreach my $Attribute (qw( FROM TO CC BCC )) {
        $ShopMailType->set({$Attribute => $hFormValues->{$Attribute}});
    }
    # EPG-27176: specific undef check for IsActive attribute is necessary
    $ShopMailType->set({'IsActive' => $hFormValues->{'IsActive'}}) if defined $hFormValues->{'IsActive'};

    # save stuff in shown language (not necessary servlet language!)
    my $LanguageID = $hFormValues->{'AltLanguageID'} // $Servlet->languageID;
    foreach my $Attribute (qw( Header Subject AdditionalText )) {
        $ShopMailType->set({ $Attribute => $hFormValues->{$Attribute} }, $LanguageID);
    }

    # validate and save shop signature
    my $Signature = $Form->formValue('Signature');
    if(defined $Signature) {
        $Signature = CloseOpenHTMLTags($Signature);
        $Shop->set({'Signature' => $Signature}, $LanguageID);
    }

    $ShopMailType->set({'TO' => $TO}) if $TO;

    return;
}

sub UnitySave {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object->getSite();
    my $Form = $Servlet->form;

    $self->Save($Servlet);

    # validate and save display shoplogo
    my $EmailEventsDisplayLogoUnity = $Form->formValue('EmailEventsDisplayLogoUnity');
    if(defined $EmailEventsDisplayLogoUnity) {
        $Shop->set({'EmailEventsDisplayLogoUnity' => $EmailEventsDisplayLogoUnity});
    }
}

sub ViewEmailEvent {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopMailType = $Servlet->object;
    my $Shop = $ShopMailType->getSite();

    # pack TO, CC and BCC into json format and save on servlet
    foreach my $Attribute (qw ( TO CC BCC )) {
        my $aAddresses = _SplitEmail($ShopMailType->get($Attribute));
        $Servlet->vars($Attribute . '_JSON', { $Attribute => $aAddresses });
    }

    # pack and format FROM into json format and save on servlet
    my $From = $ShopMailType->get('FROM');
    unless ($From) {
        # guess it isn't set, so use shop default data and override from TLE on serlvet
        my $MailFromName = $Shop->get('MailFromName') // $Shop->get('Address')->get('FullName');
        $From = $ShopMailType->joinMailAddress($MailFromName, $Shop->get('MailFrom'));
        $Servlet->vars('FROM', $From);
    }
    my $aAddresses = _SplitEmail($From);
    $Servlet->vars('FROM_JSON', { 'FROM' => $aAddresses });

    # pack TLEs into json format and save on servlet
    my $aFormattedMailTypeTemplateTLEs = _FormatMailTypeTemplateTLEs($ShopMailType);
    $Servlet->vars('PLACEHOLDER_JSON', { 'placeholder' => $aFormattedMailTypeTemplateTLEs });
    $Servlet->vars('HAS_PLACEHOLDERS', scalar @$aFormattedMailTypeTemplateTLEs ? 1 : 0);

    my $LanguageID = $Servlet->languageID;
    my $Locale = $Servlet->locale;

    # set language id and locale if AltLanguageID servlet var is set
    if ($Servlet->params('AltLanguageID')) {
        $LanguageID = $Servlet->params('AltLanguageID');
        $Locale = GetInfoLanguage($LanguageID)->{'DefaultLocaleID'};
    }
    $Servlet->vars('Subject', $ShopMailType->get('Subject', $LanguageID));
    $Servlet->vars('Header', $ShopMailType->get('Header', $LanguageID));
    $Servlet->vars('AdditionalText', $ShopMailType->get('AdditionalText', $LanguageID));
    $Servlet->vars('Signature', $Shop->get('Signature', $LanguageID));

    # process mail template and save on servlet
    $Servlet->vars('ProcessedTemplate', _ProcessTemplate($ShopMailType, $Shop, $LanguageID, $Locale));

    return;
}

sub UnityViewEmailEvent {
    my $self = shift;
    my ($Servlet) = @_;

    $self->ViewEmailEvent($Servlet);

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    return;
}

sub _FormatMailTypeTemplateTLEs {
    my ($ShopMailType) = @_;

    return [
        map {
            {
                'description' => $_->get('NameOrAlias'),
                'code' => $_->get('Code')
            }
        }
        @{ $ShopMailType->get('MailTypeTemplate')->get('MailTypeTemplateTLEs') }
    ];
}

sub _ProcessTemplate {
    my ($ShopMailType, $Shop, $LanguageID, $Locale) = @_;

    my %RegionalParams = (
        'Locale'   => $Locale,
        'Language' => GetInfoLanguage($LanguageID)->{'Code2'},
    );
    return $ShopMailType->content(
        $Shop, undef,
        \%RegionalParams, {
            'IsTest'               => 1,
            'Shop'                 => $Shop->tleHash()
        },
        1
    );

}

sub _SplitEmail {
    my ($MailString) = @_;

    my $aEmails = [];
    my @Addresses = Email::Address->parse($MailString);

    foreach my $Address (@Addresses) {
        my $Name = $Address->phrase;
        $Name =~ s/"//g; # "
        push @$aEmails, {
            'name'  => $Name,
            'email' => $Address->address
        };
    }

    return $aEmails;
}

1;
