package DE_EPAGES::ShopTransfer::t::transform;
use base DE_EPAGES::Database::API::Test;
use Test::More tests => 35;
use Test::Deep;

use strict;
use warnings;

use Config;
use English;
use File::Basename;

use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::File qw (
    AbsolutePath
    CreateDirectory
    DeleteDirectory
    DeleteFile
    ExistsFile
    GetFileContent
    MoveFile
    OpenFile
    RenameDirectory
);
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::DataCache::API::Cache qw ( UpdateLocalConnection );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Presentation::API::Form;
use DE_EPAGES::ShopTransfer::API::BCP qw ( BCPInIterator BCPOutMapper );
use DE_EPAGES::ShopTransfer::API::BCPEditor;
use DE_EPAGES::ShopTransfer::API::Import;

use Readonly;
Readonly::Scalar my $DEFAULT_OUT   => './out';
Readonly::Scalar my $DE_EPAGES     => $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES';
Readonly::Scalar my $PERL          => $EXECUTABLE_NAME;
Readonly::Scalar my $SHOPALIAS     => 'DemoShop';
Readonly::Scalar my $SCRIPTNAME    => 'DemoShop';
Readonly::Scalar my $NEW_SHOPALIAS => 'DemoShopX';
Readonly::Scalar my $SHOP_TXT      => 'shop.txt';
Readonly::Scalar my $STORENAME     => 'Store';
Readonly::Scalar my $SOURCENAME    => 'epages';
Readonly::Scalar my $ROOTPATH      => '/Shops';

sub TestTransformShop {
    _ExportShop( $STORENAME, $SHOPALIAS );
    ok( 1, 'export shop' );
    _ExportSystem( $STORENAME );
    ok( 1, 'export system' );

    # (1) perform plain shop transfer without modifying the shop alias
    _TransformShop( $STORENAME, $STORENAME, $SHOPALIAS );
    ok( 1, 'transform shop' );
    _Validate( $SHOPALIAS, $SCRIPTNAME );

    # clear output for next test
    my $ImportPath = join('/', $DEFAULT_OUT, $STORENAME, 'import');
    DeleteDirectory( $ImportPath, { 'recursive' => 1 } );

    # (2) perform shop transfer and rename old shop alias
    _TransformShopAndRename( $STORENAME, $STORENAME, $SHOPALIAS, $NEW_SHOPALIAS );
    ok( 1, 'transform shop and rename alias' );
    _Validate( $NEW_SHOPALIAS, $SCRIPTNAME );

    # clear output for next test
    DeleteDirectory( $ImportPath, { 'recursive' => 1 } );

    # (3) perform shop transfer and rename old shop alias with creating new guids
    _TransformShopAndRenameWithNewGUID( $STORENAME, $STORENAME, $SHOPALIAS, $NEW_SHOPALIAS );
    ok( 1, 'transform shop and rename alias with creating new guids' );
    _Validate( $NEW_SHOPALIAS, $SCRIPTNAME );

    return;
}

sub TestTransformShopHavingDifferentLocales {
    DeleteDirectory( $DEFAULT_OUT, { 'recursive' => 1 } );

    # (1) Create source shop by renaming the store to $SOURCENAME
    _ExportShop( $STORENAME, $SHOPALIAS );
    _ExportSystem( $STORENAME );
    RenameDirectory(
        "$DEFAULT_OUT/$STORENAME",
        "$DEFAULT_OUT/$SOURCENAME", {
            create_directory => 1
        }
    );
    ok( 1, 'exported source shop successfully' );


    # (2) Create target shop
    _ExportShop( $STORENAME, $SHOPALIAS );
    _ExportSystem( $STORENAME );
    ok( 1, 'exported target shop successfully' );


    # (3) Set a language to 'unused' in the target shop; hence, previously defined
    #     localized string attributes should not be imported while transforming the
    #     source into the target shop.
    my ($LocaleID, $LanguageID, $Found);
    my $FilenameIN  = "$DEFAULT_OUT/$STORENAME/export/SYSTEM/locale.bcp";
    my $FilenameOUT = "$DEFAULT_OUT/$STORENAME/export/SYSTEM/locale.tmp";
    BCPOutMapper($FilenameIN, $FilenameOUT, sub {
        my (@Parameter) = @_;
        if ($Parameter[1] && !$Found) {
            $LocaleID = $Parameter[0];
            $LanguageID = $Parameter[2];
            $Parameter[1] = '0';  # $Parameter[1] equals 'isused' in the table 'locale'
            $Found = 1;
        }
        return @Parameter;
    }, 'utf8');
    MoveFile($FilenameOUT, $FilenameIN);
    ok( 1, 'modified target shop successfully' );


    # (4) Transform shop
    #     --> unsupported language strings will be removed from 'localizedstringattribute.bcp'
    _TransformShop( $STORENAME, $SOURCENAME, $SHOPALIAS );
    ok( 1, "transformed $SHOPALIAS from $SOURCENAME to $STORENAME" );


    # (5) Verify that no corrupted lines were written to 'localizedstringattribute.bcp'
    my $NumColumns;
    $FilenameIN = "$DEFAULT_OUT/$STORENAME/export/$SHOPALIAS/localizedstringattribute.bcp";
    BCPInIterator($FilenameIN, sub {
        my (@Parameter) = @_;
        $NumColumns = scalar @Parameter;
        return;
    });
    my $NoError = 1;
    $FilenameIN = "$DEFAULT_OUT/$STORENAME/import/$SHOPALIAS/localizedstringattribute.bcp";
    BCPInIterator($FilenameIN, sub {
        my (@Parameter) = @_;
        $NoError = 0 if (scalar @Parameter != $NumColumns);
    });
    ok( $NoError, 'verified transformed localizedstringattribute.bcp' );


    # (6) Final check: import table 'localizedstringattribute' as '*_test'
    (my $FilenameNew = $FilenameIN) =~ s/attribute/attribute_test/g;
    MoveFile($FilenameIN, $FilenameNew);
    my $hOptions = { storePath => "$DEFAULT_OUT/$STORENAME" };
    my $Importer = DE_EPAGES::ShopTransfer::API::Import->new();
    eval {  # alternatively, the import.pl script could be used, but then we cannot catch errors
        $Importer->importTable('localizedstringattribute_test', $SHOPALIAS, $hOptions);
    };
    ok( !ExistsError(), 'localized string attributes imported' )
        or print "Import failed: $@!";

    return;
}

sub TestTransformShopStyleSheets {
    my ($StoreCacheKey) = @_;
    DeleteDirectory( $DEFAULT_OUT, { 'recursive' => 1 } );

    # (1) upload test.jpg to mediagallery and set this image as the new background
    my $Shops = LoadObjectByPath($ROOTPATH);
    my $SourceShop = $Shops->child($SHOPALIAS);
    my $MediaGallery = $SourceShop->child('MediaGallery');

    my $Filename = 'test.jpg';
    my $MGPublicPath = $MediaGallery->get('PublicPath');
    my $FileContent = ${GetFileContent($Filename, { binary => 1 })};
    $MediaGallery->insertFile($Filename, \$FileContent, { overwrite => 1 });
    ok( ExistsFile("$MGPublicPath/$Filename"), 'new background file exists' );

    my $Style;
    $Style = $SourceShop->get('DefaultStyle');
    my $PreviousBackground = $Style->get('LayoutBackgroundImage');
    $Style->set({
        LayoutBackgroundImage => "MediaGallery/$Filename",
    });
    # CSS is written with a background path containing the old shop alias
    # background: url("/WebRoot/Store/Shops/DemoShop/MediaGallery/test.jpg") white;
    $Style->get($_) foreach ( qw(URL EditorURL) );
    ok ( _VerifyBackgroundImage($MediaGallery, $Style, $Filename),
        "new background image was found in CSS file of $SHOPALIAS" );

    # (2) export old shop
    _ExportShop( $STORENAME, $SHOPALIAS );
    _ExportSystem( $STORENAME );
    ok( 1, 'exported shop and system' );

    # (3) transform shop by renaming it from $SHOPALIAS to $NEWSHOPALIAS
    _TransformShopAndRenameWithNewGUID( $STORENAME, $STORENAME, $SHOPALIAS, $NEW_SHOPALIAS);
    ok( 1, "transformed shop from $SHOPALIAS to $NEW_SHOPALIAS" );

    # (4) import new shop
    _ImportShop( $STORENAME, $NEW_SHOPALIAS );
    UpdateLocalConnection($StoreCacheKey); # ensures that the new shop is available
    ok( $Shops->existsChild($NEW_SHOPALIAS), "imported $NEW_SHOPALIAS" );

    # (5) reload style (re-generate CSS files)
    my $TargetShop = $Shops->child($NEW_SHOPALIAS);
    $Style = $TargetShop->get('DefaultStyle');
    $Style->get($_) foreach ( qw(URL EditorURL) );
    $MediaGallery = $TargetShop->child('MediaGallery');
    ok ( _VerifyBackgroundImage($MediaGallery, $Style, $Filename),
        "new background image was found in CSS file of $NEW_SHOPALIAS" ) or
        Test::More::diag "the old shop alias ($SHOPALIAS) was not replaced " .
             "by the new one ($NEW_SHOPALIAS) in " . basename($Style->get('URL'));

    # (6) cleanup
    $Style = $SourceShop->get('DefaultStyle');
    $Style->set({
        LayoutBackgroundImage => $PreviousBackground,
    });
    return;
}

sub _Validate {
    my ($ShopAlias, $ScriptName) = @_;

    # validate import directory having the name of the shop alias
    my $Path = join('/', $DEFAULT_OUT, $STORENAME, 'import', $ShopAlias);
    ok( -d $Path, 'directory with shop alias exists' );

    # validate correct shop alias in object.bcp and site.bcp
    my $SiteBCP = DE_EPAGES::ShopTransfer::API::BCPEditor->new( "$Path/site.bcp" );
    my $aSiteRows = $SiteBCP->findRows({});
    is( scalar @$aSiteRows, 1, 'site.bcp contains 1 row' );
    my $aSite = $aSiteRows->[0];
    is( $aSite->[10], $ScriptName, "site.bcp contains the webserverscriptnamepart $ScriptName" );## no critic qw(ProhibitMagicNumbers)

    my $ShopID = $aSite->[0];
    my $ObjectBCP = DE_EPAGES::ShopTransfer::API::BCPEditor->new( "$Path/object.bcp" );
    my $aObjectRows = $ObjectBCP->findRows({ 0 => $ShopID });
    is( scalar @{$aObjectRows}, 1, 'object.bcp contains the shopid' );
    my $aSiteObject = $aObjectRows->[0];
    is( $aSiteObject->[3], $ShopAlias, "object.bcp contains the alias $ShopAlias" );

    # extract public.tar in order to perform the following test
    my $SourceTar = AbsolutePath( join('/', $Path, 'public.tar') );
    my $TmpDirectory = join('/', $DEFAULT_OUT, 'temp');
    CreateDirectory( $TmpDirectory );
    my $Tar = $Config{osname} eq 'MSWin32' ?
        File::Spec->catfile($ENV{'EPAGES'},'bin','tar.exe') : 'tar';
    my $Command = sprintf '%s -xf %s -C %s', $Tar, $SourceTar, $TmpDirectory;
    SystemCommand( $Command );

    # validate correct shop alias within the *.tar archive
    $Path = join('/', $TmpDirectory, $ShopAlias);
    ok( -d $Path, "directory with shop alias '$ShopAlias' exists within tar archive" );

    return;
}

sub _VerifyBackgroundImage {
    my ($MediaGallery, $Style, $Filename) = @_;

    my $Success = 0;
    my $File = join('/', $Style->get('PublicPath'), basename($Style->get('URL')));
    my $Pattern = join('/', $MediaGallery->get('WebPath'), $Filename);
    OpenFile( '<', $File, sub {
        my ($FileHandle) = @_;
        while (<$FileHandle>) {
            if ($_ =~ m/\Q$Pattern/) {
                $Success = 1;
                last;
            }
        }
    });

    return $Success;
}

sub _DeleteShop {
    my ($Storename, $ShopAlias) = @_;

    my $Shops = LoadObjectByPath($ROOTPATH);
    my $PublicPath = $Shops->child($ShopAlias)->get('PublicPath');

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/delete.pl",
        -storename => $Storename,
        -shop      => $ShopAlias,
    ]);

    DeleteDirectory( $PublicPath, { recursive => 1 } );

    return;
}

sub _ExportShop {
    my ($Storename, $ShopAlias) = @_;

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $Storename,
        -shop      => $ShopAlias,
    ]);

    return;
}

sub _ExportSystem {
    my ($Storename) = @_;

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $Storename,
        '-system',
    ]);

    return;
}

sub _ImportShop {
    my ($Storename, $ShopAlias) = @_;

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/import.pl",
        -storename => $Storename,
        -shop      => $ShopAlias,
    ]);

    return;
}

sub _TransformShop {
    my ($Storename, $Sourcename, $ShopAlias) = @_;

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $Storename,
        -source    => $Sourcename,
        -shop      => $ShopAlias,
    ]);

    return;
}

sub _TransformShopAndRename {
    my ($Storename, $Sourcename, $ShopAliasOld, $ShopAliasNew) = @_;

    OpenFile('>', $SHOP_TXT, sub{
       my ($FileHandle) = @_;
       print $FileHandle "$ShopAliasOld=$ShopAliasNew";
    });

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $Storename,
        -source    => $Sourcename,
        -list      => $SHOP_TXT,
    ]);

    return;
}

sub _TransformShopAndRenameWithNewGUID {
    my ($Storename, $Sourcename, $ShopAliasOld, $ShopAliasNew) = @_;

    OpenFile('>', $SHOP_TXT, sub{
       my ($FileHandle) = @_;
       print $FileHandle "$ShopAliasOld=$ShopAliasNew";
    });

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $Storename,
        -source    => $Sourcename,
        -list      => $SHOP_TXT,
        '-newguid',
    ]);

    return;
}

sub init {
    my $self = shift;

    # cf. TestTransformShopHavingDifferentLocales
    $self->dbi->do('CREATE TABLE localizedstringattribute_test LIKE localizedstringattribute');
    return;
}

sub cleanup {
    my $self = shift;

    DeleteDirectory( $DEFAULT_OUT, { 'recursive' => 1 } );

    # cf. TestTransformShop and TestTransformShopStyles
    DeleteFile( $SHOP_TXT );

    # cf. TestTransformShopHavingDifferentLocales
    $self->dbi->dropTableIfExists('localizedstringattribute_test');

    # cf. TestTransformShopStyleSheets
    my $Shops = LoadObjectByPath($ROOTPATH);
    my $Shop = $Shops->child($SHOPALIAS);
    my $Style = $Shop->get('DefaultStyle');
    my $hValues = $Style->get([ qw(URL EditorURL PublicPath StyleTemplateID) ]);
    my @aFilenames = ( basename($hValues->{URL}), basename($hValues->{EditorURL}) );
    my $PathToCSS = $hValues->{PublicPath};
    DeleteFile( "$PathToCSS/$_" ) foreach (@aFilenames);

    # delete previously imported shop
    _DeleteShop( $STORENAME, $NEW_SHOPALIAS ) if $Shops->existsChild($NEW_SHOPALIAS);
    return;
}

sub test {
    my $self = shift;

    TestTransformShop();
    TestTransformShopHavingDifferentLocales();
    TestTransformShopStyleSheets( $self->dbi->cacheKey );
}

__PACKAGE__->runScript;
