use strict;
use Getopt::Long;
use List::Util qw(min max);
use List::MoreUtils qw(uniq);

use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath JoinObjectPath);
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

sub ShowShops {
    print "Error: Missing parameter -shopalias, choose one of following shops:\n\n";
    print $_->alias, ' ' foreach @{ LoadObjectByPath('/Shops')->children };
    print "\n\n";
    usage();
}

sub FixTaxClassArea {
    my ($Options) = @_;
    my (%TaxMatrix);
    my $dgt = $Options->{digital} ? 'digital' : '';
    my $Dgt = $Options->{digital} ? 'Digital' : '';

    # get shopid
    my $ShopID = LoadObjectByPath(JoinObjectPath([ 'Shops', $Options->{shopalias} ]))->id;
    print "-> ShopID=$ShopID\n";

    my $dbi = GetCurrentDBHandle();

    # show taxmatrix for given tax class/area
    if ($Options->{taxclass}) {
        my $sql = "select taxmatrix${dgt}id from taxclass$dgt where taxclass${dgt}id = %d";
        my ($Matrix) = @{$dbi->execute(sprintf $sql, $Options->{taxclass})};
        $TaxMatrix{givenclass} = $Matrix->[0];
        print "-> GivenTaxClass=$Options->{taxclass} has TaxMatrix: $TaxMatrix{givenclass}\n";
    }
    if ($Options->{taxarea}) {
        my $sql = "select taxmatrix${dgt}id from taxarea$dgt where taxarea${dgt}id = %d";
        my ($Matrix) = @{$dbi->execute(sprintf $sql, $Options->{taxarea})};
        $TaxMatrix{givenarea} = $Matrix->[0];
        print "-> GivenTaxArea=$Options->{taxarea} has TaxMatrix: $TaxMatrix{givenarea}\n";
    }

    # get taxmatrix used by shop
    $TaxMatrix{used} = LoadObjectByPath
        (JoinObjectPath([ 'Shops', $Options->{shopalias} ]))->get("TaxMatrix$Dgt")->get('Path');
    print "-> UsedTaxMatrix=$TaxMatrix{used}\n";

    # get base/shop taxmatrixid
    $TaxMatrix{base} = '/'. (split /\//, $TaxMatrix{used})[-1];
    $TaxMatrix{shop} = "/Shops/$Options->{shopalias}$TaxMatrix{base}";
    ##no critic qw(RequireExistsError)
    $TaxMatrix{baseid} = eval { LoadObjectByPath($TaxMatrix{base})->id };
    $TaxMatrix{shopid} = eval { LoadObjectByPath($TaxMatrix{shop})->id };
    ##use critic

    # find what is the right one
    $TaxMatrix{correct} = $TaxMatrix{used} =~
        m,^/Shops/$Options->{shopalias}/, ? 'shop' : 'base';

    print "-> BaseTaxMatrixID=$TaxMatrix{baseid}";
    print $TaxMatrix{correct} eq 'base' ? " (correct)\n" : "\n";
    print "-> ShopTaxMatrixID=$TaxMatrix{shopid}";
    print $TaxMatrix{correct} eq 'shop' ? " (correct)\n" : "\n";

    # get base/shop taxmatrix
    my (%ShopTaxes, %BaseTaxes);
    my $sql = 'select objectid,alias from object where parentid=%d';
    map { $ShopTaxes{$_->[1]} = $_->[0]; }
    @{$dbi->execute(sprintf $sql, $TaxMatrix{shopid})} if $TaxMatrix{shopid};
    map { $BaseTaxes{$_->[1]} = $_->[0]; }
    @{$dbi->execute(sprintf $sql, $TaxMatrix{baseid})} if $TaxMatrix{baseid};

    # define hash %FixTaxes that maps the wrong taxmatrix id to the right one
    # $Condition defines what items are wrong
    my ($FixTaxes, $Condition) = _defineFixTaxes(\%TaxMatrix, \%ShopTaxes, \%BaseTaxes);

    # find wrong items
    my $FixItems = _findWrongItems($ShopID, $Condition, $dgt);

    # fix wrong items
    _updateWrongItems($ShopID, $FixItems, $FixTaxes, \%TaxMatrix, $dgt);
}

sub _defineFixTaxes {
    my ($TaxMatrix, $ShopTaxes, $BaseTaxes) = @_;
    my (%FixTaxes, $Condition);

    if ($TaxMatrix->{correct} eq 'shop') {
        foreach (uniq (keys %$BaseTaxes, keys %$ShopTaxes)) {
            $FixTaxes{$BaseTaxes->{$_}} = $ShopTaxes->{$_};
            print "-> $_: '$BaseTaxes->{$_}' -> '$ShopTaxes->{$_}'\n";
        }
        $Condition = '< '. min(values %$ShopTaxes);
    } else {
        foreach (uniq (keys %$BaseTaxes, keys %$ShopTaxes)) {
            $FixTaxes{$ShopTaxes->{$_}} = $BaseTaxes->{$_};
            print "-> $_: '$ShopTaxes->{$_}' -> '$BaseTaxes->{$_}'\n";
        }
        $Condition = '> '. max(values %$BaseTaxes);
    }
    $Condition = '> 0' unless scalar keys %$ShopTaxes and scalar keys %$BaseTaxes;

    return(\%FixTaxes, $Condition);
}

sub _findWrongItems {
    my ($ShopID, $Condition, $dgt) = @_;
    my (%FixItems);

    my $dbi = GetCurrentDBHandle();

    my %sql;
    unless ($dgt) {
        $sql{customer} = 'select customerid,taxareaid from customer '.
            'inner join object on objectid = customerid and shopid = %d '.
            'where taxareaid %s';
    }
    $sql{lineitemcontainer} = "select lineitemcontainerid,taxarea${dgt}id from ".
        'lineitemcontainer inner join object on objectid = lineitemcontainerid '.
        "and siteid = %d where taxarea${dgt}id %s";

    $sql{lineitem} = "select lineitemid,taxclass${dgt}id from lineitem inner join ".
        "object on objectid = lineitemid and siteid = %d where taxclass${dgt}id %s";
    $sql{product} = "select productid,taxclass${dgt}id from product ".
        'inner join object on objectid = productid and shopid = %d '.
        "where taxclass${dgt}id %s";
    unless ($dgt) {
        $sql{paymentmethod} = 'select paymentmethodid,taxclassid from paymentmethod '.
            'inner join object on objectid = paymentmethodid and shopid = %d '.
            'where taxclassid %s';
        $sql{shippingmethod} = 'select shippingmethodid,taxclassid from shippingmethod '.
            'inner join object on objectid = shippingmethodid and shopid = %d '.
            'where taxclassid %s';
        $sql{shippingoption} = 'select shippingoptionid,taxclassid from shippingoption '.
            'inner join object on objectid = shippingoptionid and shopid = %d '.
            'where taxclassid %s';
    }

    foreach my $Item (sort keys %sql) {
        printf $sql{$Item}, $ShopID, "$Condition\n";
        $FixItems{$Item}{$_->[0]} = $_->[1] foreach
            @{$dbi->execute(sprintf($sql{$Item}, $ShopID, $Condition))};
        if ($Condition eq '> 0' and scalar keys %{$FixItems{$Item}}) {
            my $TableHeading = $sql{$Item};
            $TableHeading =~ s,^select (\S+).*,$1,;
            $TableHeading =~ s/,/ /g;
            print "$TableHeading\n";
            print "$_ $FixItems{$Item}{$_}\n" foreach sort {$a <=> $b} keys %{$FixItems{$Item}};
        }
    }

    print "\n";
    return \%FixItems;
}

sub _updateWrongItems {
    my ($ShopID, $FixItems, $FixTaxes, $TaxMatrix, $dgt) = @_;

    my $dbi = GetCurrentDBHandle();

    my %sql;
    unless ($dgt) {
        $sql{customer} = 'update customer as C inner join object as O on '.
            'O.objectid = C.customerid and C.shopid = %d '.
            'set C.taxareaid = %d where C.taxareaid = %s';
    }
    $sql{lineitemcontainer} = 'update lineitemcontainer as L inner join object as O on '.
        'O.objectid = L.lineitemcontainerid and O.siteid = %d '.
        "set L.taxarea${dgt}id = %d where L.taxarea${dgt}id = %s";
    $sql{lineitem} = 'update lineitem as L inner join object as O on '.
        'O.objectid = L.lineitemid and O.siteid = %d '.
        "set L.taxclass${dgt}id = %d where L.taxclass${dgt}id = %s";
    $sql{product} = 'update product as P inner join object as O on '.
        'O.objectid = P.productid and P.shopid = %d '.
        "set P.taxclass${dgt}id = %d where P.taxclass${dgt}id = %s";
    unless ($dgt) {
        $sql{product} = 'update product as P inner join object as O on '.
            'O.objectid = P.productid and P.shopid = %d '.
            'set P.taxclassid = %d where P.taxclassid = %s';
        $sql{paymentmethod} = 'update paymentmethod as P inner join object as O on '.
            'O.objectid = P.paymentmethodid and P.shopid = %d '.
            'set P.taxclassid = %d where P.taxclassid = %s';
        $sql{shippingmethod} = 'update shippingmethod as P inner join object as O on '.
            'O.objectid = P.shippingmethodid and P.shopid = %d '.
            'set P.taxclassid = %d where P.taxclassid = %s';
        $sql{shippingoption} = 'update shippingoption as P inner join object as O on '.
            'O.objectid = P.shippingoptionid and P.shopid = %d '.
            'set P.taxclassid = %d where P.taxclassid = %s';
    }

    foreach my $Item (sort keys %sql) {
        next unless defined $FixItems->{$Item};
        printf $sql{$Item}, $ShopID, $FixTaxes->{$_}, "$_\n"
            foreach sort {$a <=> $b} uniq values %{$FixItems->{$Item}};
    }

    if (not scalar keys %$FixItems or not $TaxMatrix->{shopid} or not $TaxMatrix->{baseid}) {
        print "\n" if scalar keys %$FixItems;

        if ($TaxMatrix->{shopid} and $TaxMatrix->{baseid}) {
            if ($TaxMatrix->{givenclass} and $TaxMatrix->{givenarea} and
                $TaxMatrix->{givenclass} ne $TaxMatrix->{givenarea}) {
                print "Tax matrix from class and area differ but no item to fix found.\n";
            } else {
                print "All tax areas and classes match and no item to fix found.\n";
            }
        } else {
            print 'Tax matrix missing:';
            print " shop:$TaxMatrix->{shop}" unless $TaxMatrix->{shopid};
            print " base:$TaxMatrix->{base}" unless $TaxMatrix->{baseid};
            print "\n";
        }
        exit 0;
    }

    print "\nDo you really want to run all update statements mentioned above? (y/n): ";
    my $Answer = <STDIN>; ##no critic qw(ProhibitExplicitStdin)
    chomp($Answer);

    if (lc($Answer) ne 'y') {
        print "Aborted on user request.\n";
        exit 0;
    }

    foreach my $Item (sort keys %sql) {
        next unless defined $FixItems->{$Item};
        $dbi->execute(sprintf $sql{$Item}, $ShopID, $FixTaxes->{$_}, $_)
            foreach sort {$a <=> $b} uniq values %{$FixItems->{$Item}};
    }
}


sub Main {

    my %Options = ();
    my @GetOpt = qw(passwd=s storename=s shopalias=s taxclass=s taxarea=s digital help);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};

    unless ($Options{storename}) {
        print "Error: Missing parameter -storename\n\n";
        usage();
    }

    RunOnStore(
        Store       => $Options{storename},
        DBPassword  => $Options{passwd},
        Sub         => sub  {
            if ($Options{shopalias}) {
                FixTaxClassArea(\%Options);
                SynchronizeCache();
            } else {
                ShowShops();
            }
        },
    );
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 -storename STORE -shopalias SHOP [-digital] [-taxclass TC] [-taxarea TA]
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
