#========================================================================================
# Package: DE_EPAGES::Trigger::API::Object::BatchProcessHandler and
#          DE_EPAGES::Trigger::API::BatchProcessor
#
# Test: Testing basic functions (see BatchProcessor2.t as better documented example)
#
#========================================================================================
use strict;
my $countProcessItem = 0;                       # counter for test
my $BatchTaskName    = 'DummyImportProducts';

#----------------------------------------------------------------------
#   Preparations
#----------------------------------------------------------------------
## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
package My::Hooks::ProductBatchHooks;

use DE_EPAGES::Trigger::API::BatchProcessor qw( RegisterHandler HandleItem GetHandler);
use Test::More;

sub OnAfterUpdateProduct($) {
    my ($ProductID) = @_;
    HandleItem( $BatchTaskName, 'My::API::Object::ProductBatchHandler', { ProductID => $ProductID } );
}

use warnings;

#----------------------------------------------------------------------
# ProductBatchHandler - minimal implementation
#----------------------------------------------------------------------
## no critic (Modules::ProhibitMultiplePackages)
package My::API::Object::ProductBatchHandler;
use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );
use Test::More;

sub processItem {
    my $self = shift;
    my ($hProduct) = @_;

    $countProcessItem++;

    Test::More::diag( 'process Product ' . $hProduct->{ProductID} . ' QueueSize: ' . $self->countItems() . "\n" );
}

#----------------------------------------------------------------------
#   Begin Tests
#----------------------------------------------------------------------
package My::UI::BusinessLogic;

use base 'Test::Class';

use strict;
use Test::More;

use DE_EPAGES::Trigger::API::BatchProcessor qw( RunBatchProcess GetHandler RegisterHandler IsHandlerRegistered );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );

sub setUp : Test(setup) {
    $countProcessItem = 0;
}

sub testDieOnMissingPackage : Test(no_plan) {
    eval { GetHandler( $BatchTaskName, 'My::Hooks::NotExistingHandler' ); };
    ok( ExistsError(), 'Works as intended: Throw error for GetHandler(...) on non existing BatchHandler' );
}

#----------------------------------------------------------------------
# Use case  - CSV import of products
#----------------------------------------------------------------------
# We emulate a csv import like Product::Scripts::ProductExportImport.pl
sub testImportProductList : Test(no_plan) {
    my $Handler;

    # manually register batch handler. We cannot mock the TriggerHook call.
    RegisterHandler( $BatchTaskName, My::API::Object::ProductBatchHandler->new() );
    ok( IsHandlerRegistered( $BatchTaskName, 'My::API::Object::ProductBatchHandler' ), 'register handler was successful' );

    # running batch process
    RunBatchProcess(
        $BatchTaskName,
        sub {
            Test::More::diag('start Import');
            $Handler = GetHandler( $BatchTaskName, 'My::API::Object::ProductBatchHandler' );

            ok( $Handler, 'Register BatchHandler' );

            foreach my $ProductID ( 0 .. 9 ) {
                DummyImportProduct($ProductID);
            }

            is( $Handler->countItems(), 10, 'adding 10 items to BatchHandler queue' );    ## no critic (ProhibitMagicNumbers)
            Test::More::diag('end Import -> start batch processing');
        }
    );

    # manually flushing, same reason as above.
    ## no critic (ProtectPrivateVars ProtectPrivateSubs)
    DE_EPAGES::Trigger::API::BatchProcessor::_RunBatchHandlers( $DE_EPAGES::Trigger::API::BatchProcessor::_HANDLERS->{$BatchTaskName} );
    is( $Handler->countItems(), 0, 'all items consumed and queue cleared' );

    # uses global scalar declared on top
    is( $countProcessItem, 10, 'called 10 times processItem' );                           ## no critic qw(ProhibitMagicNumbers)
}

sub testRunOutsideBatchProcess : Test(no_plan) {
    ## no critic (ProtectPrivateVars)
    local $DE_EPAGES::Trigger::API::BatchProcessor::_HANDLERS = {};

    # RegisterHandler( $BatchTaskName, My::API::Object::ProductBatchHandler->new() );
    # calling batch handler outside batchprocess should be possible
    foreach my $ProductID ( 0 .. 9 ) {
        DummyImportProduct($ProductID);
    }

    # uses global scalar declared on top
    is( $countProcessItem, 10, 'called 10 times processItem' );    ## no critic qw(ProhibitMagicNumbers)
}

sub DummyImportProduct {
    my ($ProductID) = @_;

    # trigger ProductBatchHandler -> addItem through Hook
    # in production TriggerHook('OnAfterUpdateProduct') would be used
    My::Hooks::ProductBatchHooks::OnAfterUpdateProduct($ProductID);

    return;
}

RunScript(
    Sub   => sub {
        Test::Class->runtests;
    }
);
