use strict;

use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError ResetError);
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw (RunOnStore);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::WebInterface::API::MessageCenter qw (SynchronizeCache);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName
    );
    usage() if $Help;
    usage() unless defined $StoreName;

    my $time1 = time;

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            # make sure that the TrustedShops cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::TrustedShops')) {
                print "Store: $StoreName : Cartridge TrustedShops is not installed.\n" unless $NoOutput;
                return;
            }
            my $aCertificates = LoadClassByAlias('TrustedShopsCertificate')->objects;
            printf("Start updating trusted shops certificates %s \n", scalar @$aCertificates) unless $NoOutput;
            foreach my $Certificate (@$aCertificates) {
                printf("Updating certificate %s path %s : " , $Certificate->id,$Certificate->pathString) unless $NoOutput;
                if ($Certificate->getSite()->get('ClosedByProvider')) {
                    printf("Skipped : site ClosedByProvider \n")  unless $NoOutput;
                    next;
                }
                if (isValid($Certificate->get('CertificationLanguage'))) {
                    printf("Skipped : CertificationLanguage is already set to '%s' \n",$Certificate->get('CertificationLanguage')) unless $NoOutput;
                    next;
                }
                my $ret;
                eval {
                    my $ShopLanguageID = $Certificate->getSite()->get('LanguageID');
                    my $ShopLanguageCode2 = GetInfoLanguage($ShopLanguageID)->{'Code2'};
                    if(isValid($ShopLanguageCode2)) {
                        $ret = $Certificate->set({'CertificationLanguage' => $ShopLanguageCode2});
                        printf(" ... CertificationLanguage '%s' done.\n" ,$ShopLanguageCode2) unless $NoOutput;
                    }
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogError($Error);
                    print STDERR 'Update of certificate ' . $Certificate->id . ' (' . $Certificate->pathString .
                        ') failed: ' . $Error->message . "\n" ;
                    ResetError();
                }
            }
        }
    );
    printf ("need %.3f seconds\n", time() - $time1) unless ($NoOutput);

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Description:
   The script is looking for all installed TrustedShops certificates that have an empty "CertificationLanguage".
   Then sets the field to the language code of the main shop language.
   The check is performed on all existing shops.

Usage:
    perl $0 [options] flags

options:
    -passwd     database user password
    -storename  name of store

flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store

END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);
