#!perl -w
use strict;
use Test::More tests => 52;
use URI;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::WebService::API::Client;
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

## no critic qw(ProhibitMagicNumbers)
RunScript( Message => 'long', Sub => sub {
    ok( Init(), 'Init' );

    my $WebserviceURL;
    RunOnStore( Store => GetTestSite(), Sub => sub {
        my $h = LoadRootObject()->get([qw(DomainName WebServerDirectory WebServerScriptNamePart)]);
        $WebserviceURL = "http://$h->{DomainName}/$h->{WebServerDirectory}/$h->{WebServerScriptNamePart}.soap";
    });

    my $soap = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/ErrorService/2008/09')
        ->proxy($WebserviceURL);

    # Error()
    eval { $soap->error( 'Paul' ) };
    ok( ExistsError(), 'ExistsError' );
    my $Error = GetError();
    cmp_ok( $Error->code, 'eq', 'WebServiceFault', 'Error.code eq \'WebServiceFault\'' ) or Test::More::diag $Error->longMessage;
    cmp_ok( $Error->shortMessage, 'eq', 'WebService error: This is a test error for Paul. (soap:Server)', 'Error.shortMessage' );
    ok( $Error->message =~ /^WebService error: This is a test error for Paul\./, 'Error.message' );
    ok( $Error->longMessage =~ /^WebService error: This is a test error for Paul\./, 'Error.longMessage' );
    ok( $Error->longMessage =~ /\bError\.t line \d+/, 'Error.longMessage contains local stack trace' );
    is $Error->innerLongMessage, undef, 'Error.innerLongMessage is empty';
    is( $Error->vars->{'EndpointURI'}, $WebserviceURL, 'Error.vars.EndpointURI' );
    is $Error->vars->{'FaultString'}, 'This is a test error for Paul.', 'Error.vars.FaultString';

    ok( 'HASH' eq ref $Error->vars->{'FaultDetail'}, 'vars->{FaultDetail} exists' );
    my $hDetails = $Error->vars->{'FaultDetail'}->{'ErrorDetails'};
    is( ref $hDetails, 'HASH', 'ErrorDetails exists' );
    is scalar(keys %$hDetails), 3, 'remote error details';
    is $hDetails->{'Code'}, 'TestErrorCode', 'error: details.code';
    is length $hDetails->{'GUID'}, 36, 'error: details.guid';
    is length $hDetails->{'Hash'}, 32, 'error: details.hash';
    ok !exists $hDetails->{'LongMessage'}, 'error: no long message';
    ok !exists $hDetails->{'Vars'}, 'error: no vars';

    # error with details
    eval { $soap->errorWithDetails( 'James' ) };
    ok( ExistsError(), 'errorWithDetails: ExistsError' );
    $Error = GetError();
    is $Error->vars->{'FaultString'}, 'FeatureLimitExceeded: ErrorVar=ErrorValue', 'errorWithDetails: faultstring';
    $hDetails = $Error->vars->{'FaultDetail'}->{'ErrorDetails'};
    is scalar(keys %$hDetails), 4, 'rerrorWithDetails: emote error details';
    is $hDetails->{'Code'}, 'FeatureLimitExceeded', 'errorWithDetails: details.code';
    is length $hDetails->{'GUID'}, 36, 'errorWithDetails: error.guid';
    is length $hDetails->{'Hash'}, 32, 'errorWithDetails: error.hash';
    ok !exists $hDetails->{'LongMessage'}, 'errorWithDetails: no long message';
    ok exists $hDetails->{'Vars'}, 'errorWithDetails: vars exist';
    is $hDetails->{'Vars'}->{'ErrorVar'}, 'ErrorValue', 'errorWithDetails: error variable value';

    # die
    eval { $soap->nativeDie('Fred'); };
    ok( ExistsError(), 'nativeDie: ExistsError' );
    $Error = GetError();
    is( $Error->code, 'WebServiceFault', 'nativeDie: Error.code eq \'WebServiceFault' );
    ok( $Error->shortMessage =~ /^WebService error: failed with: Fred/, 'nativeDie: shortMessage' );
    is $Error->vars->{'FaultString'}, 'failed with: Fred', 'nativeDie: FaultString';
    $hDetails = $Error->vars->{'FaultDetail'}->{'ErrorDetails'};
    is scalar(keys %$hDetails), 3, 'nativeDie: remote error details';
    is $hDetails->{'Code'}, 'NativeDie', 'nativeDie: details.code';
    is length $hDetails->{'GUID'}, 36, 'nativeDie: details.guid';
    is length $hDetails->{'Hash'}, 32, 'nativeDie: details.hash';
    ok !exists $hDetails->{'LongMessage'}, 'nativeDie: no long message';
    ok !exists $hDetails->{'Vars'}, 'nativeDie: no vars';

    # permission denied
    eval { $soap->permissionDenied; };
    ok( ExistsError(), 'permissionDenied: ExistsError' );
    $Error = GetError();
    cmp_ok( $Error->code, 'eq', 'WebServiceTransport', 'permissionDenied: Error.code eq \'WebServiceTransport' );
    ok( $Error->vars->{'Code'} == 401, 'WebServiceTransport: status code: 401' );
    ok( $Error->shortMessage =~ /^WebService transport error: 401/, 'permissionDenied: shortMessage' );

    # invalid method
    eval { $soap->invalidMethod; };
    ok( ExistsError(), 'invalidMethod: ExistsError' );
    $Error = GetError();
    is $Error->vars->{'FaultString'}, 'Object /WebServices/Error/Methods doesn\'t contain a child alias invalidMethod', 'invalidMethod: faultstring';
    $hDetails = $Error->vars->{'FaultDetail'}->{'ErrorDetails'};
    is $hDetails->{'Code'}, 'NoResults_ChildAlias', 'invalidMethod: error.code';
    is length $hDetails->{'GUID'}, 36, 'invalidMethod: error.guid';
    is length $hDetails->{'Hash'}, 32, 'invalidMethod: error.hash';
    ok !exists $hDetails->{'LongMessage'}, 'invalidMethod: no long message';
    ok !exists $hDetails->{'Vars'}, 'invalidMethod: no vars';

    # provoke a connection error
    $soap = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/ErrorService/2008/09')
        ->proxy('http://user:password@unknown.local/path');
    eval { $soap->error( 'Paul' ) };
    ok( ExistsError(), 'ExistsError' );
    $Error = GetError();
    is( $Error->code, 'WebServiceTransport', 'Error.code eq \'WebServiceTransport' ) or Test::More::diag $Error->longMessage;
    my $EndpointURI = URI->new( $Error->vars->{'EndpointURI'} );
    is( $EndpointURI->userinfo, undef, 'Error.EndpointURI does not contain a password' );

    ok( Cleanup(), 'Cleanup' );
});

sub Init {
    Cleanup();
    RunOnStore( Store => GetTestSite(), Sub => sub {
        ImportObjects('ErrorService.xml');
    });
    SynchronizeCache();
    return 1;
}

sub Cleanup {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        DeleteObjects('ErrorService.xml');
    });
    SynchronizeCache();
    return 1;
}
